@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -282,10 +282,12 @@ class BatchSysManager(object):
             ctx_list.append(ctx)
 
             if not ctx.batch_sys_name or not ctx.batch_sys_job_id:
+                # Lost batch system information for some reason.
+                # Mark the job as if it is no longer in the batch system.
+                ctx.batch_sys_exit_polled = 1
                 sys.stderr.write(
                     "%s/job.status: incomplete batch system info\n" % (
                         ctx.job_log_dir))
-                continue
 
             # We can trust:
             # * Jobs previously polled to have exited the batch system.
@@ -389,7 +391,8 @@ class BatchSysManager(object):
                     command = shlex.split(
                         batch_sys.KILL_CMD_TMPL % {"job_id": job_id})
                     try:
-                        proc = Popen(command, stderr=PIPE)
+                        proc = Popen(
+                            command, stdin=open(os.devnull), stderr=PIPE)
                     except OSError as exc:
                         # subprocess.Popen has a bad habit of not setting the
                         # filename of the executable when it raises an OSError.
@@ -517,7 +520,8 @@ class BatchSysManager(object):
                 # Simple poll command that takes a list of job IDs
                 cmd = [batch_sys.POLL_CMD] + exp_ids
             try:
-                proc = Popen(cmd, stderr=PIPE, stdout=PIPE)
+                proc = Popen(
+                    cmd, stdin=open(os.devnull), stderr=PIPE, stdout=PIPE)
             except OSError as exc:
                 # subprocess.Popen has a bad habit of not setting the
                 # filename of the executable when it raises an OSError.
@@ -591,7 +595,7 @@ class BatchSysManager(object):
         # Submit job
         batch_sys = self._get_sys(batch_sys_name)
         proc_stdin_arg = None
-        proc_stdin_value = None
+        proc_stdin_value = open(os.devnull)
         if hasattr(batch_sys, "get_submit_stdin"):
             proc_stdin_arg, proc_stdin_value = batch_sys.get_submit_stdin(
                 job_file_path, submit_opts)
