@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+"gcylc config file format."
 
 import os
 import sys
@@ -23,7 +24,7 @@ from copy import deepcopy, copy
 
 from parsec import ParsecError
 from parsec.config import config, ItemNotFoundError, itemstr
-from parsec.validate import validator as vdr
+from parsec.validate import coercers, validator as vdr
 from parsec.upgrade import upgrader
 from parsec.util import printcfg
 from cylc.gui.view_tree import ControlTree
@@ -33,14 +34,13 @@ from cylc.task_state import (
     TASK_STATUS_READY, TASK_STATUS_SUBMITTED, TASK_STATUS_SUBMIT_FAILED,
     TASK_STATUS_SUBMIT_RETRYING, TASK_STATUS_RUNNING, TASK_STATUS_SUCCEEDED,
     TASK_STATUS_FAILED, TASK_STATUS_RETRYING)
+from cylc.cfgspec.utils import (coerce_interval, DurationFloat)
 
 
-"gcylc config file format."
-
+coercers['interval'] = coerce_interval
 SITE_FILE = os.path.join(
     os.environ['CYLC_DIR'], 'conf', 'gcylcrc', 'themes.rc')
 USER_FILE = os.path.join(os.environ['HOME'], '.cylc', 'gcylc.rc')
-
 SPEC = {
     'dot icon size': vdr(
         vtype='string',
@@ -48,6 +48,8 @@ SPEC = {
         options=["small", "medium", "large", "extra large"]),
     'initial side-by-side views': vdr(vtype='boolean', default=False),
     'initial views': vdr(vtype='string_list', default=["text"]),
+    'maximum update interval': vdr(
+        vtype='interval', default=DurationFloat(15)),
     'sort by definition order': vdr(vtype='boolean', default=True),
     'sort column': vdr(
         vtype='string',
@@ -98,6 +100,39 @@ def upg(cfg, descr):
 class gconfig(config):
     """gcylc user configuration - default view panels, task themes etc."""
 
+    _INST = None
+
+    @classmethod
+    def get_inst(cls):
+        """Return default instance."""
+        if cls._INST is None:
+            cls._INST = cls(SPEC, upg)
+            try:
+                cls._INST.loadcfg(SITE_FILE, "site config")
+            except ParsecError as exc:
+                sys.stderr.write(
+                    "WARNING: ignoring bad site GUI config %s:\n"
+                    "%s\n" % (SITE_FILE, str(exc)))
+
+            if os.access(USER_FILE, os.F_OK | os.R_OK):
+                try:
+                    cls._INST.loadcfg(USER_FILE, "user config")
+                except ParsecError as exc:
+                    sys.stderr.write(
+                        "ERROR: bad user GUI config %s:\n" % USER_FILE)
+                    raise
+
+            # check and correct initial view config etc.
+            cls._INST.check()
+            # add spec defaults and do theme inheritance
+            cls._INST.transform()
+        return cls._INST
+
+    def __init__(self, *args):
+        config.__init__(self, *args)
+        self.default_theme = None
+        self.use_theme = None
+
     def transform(self):
         """
         1) theme inheritance
@@ -216,7 +251,8 @@ class gconfig(config):
                 "WARNING: no initial views defined, defaulting to 'text'")
             cfg['initial views'] = ['text']
 
-    def parse_state(self, theme, name, cfglist=[]):
+    @staticmethod
+    def parse_state(theme, name, cfglist):
         allowed_keys = ['style', 'color', 'fontcolor']
         cfg = {}
         for item in cfglist:
@@ -243,19 +279,21 @@ class gconfig(config):
             else:
                 target[item] = source[item]
 
-    def dump(self, keys=[], sparse=False, pnative=False, prefix='',
+    def dump(self, keys, sparse=False, pnative=False, prefix='',
              none_str=''):
-        # override parse.config.dump() to restore the list-nature of
-        # theme state items
+        """Override parse.config.dump().
+
+        To restore the list-nature of theme state items.
+        """
         cfg = deepcopy(self.get([], sparse))
         try:
             for theme in cfg['themes'].values():
-                for state in theme.keys():
+                for state in theme:
                     clist = []
                     for attr, val in theme[state].items():
-                        clist.append(attr + '=' + val)
+                        clist.append('%s=%s' % (attr, val))
                     theme[state] = clist
-        except:
+        except (KeyError, AttributeError):
             pass
 
         parents = []
@@ -272,25 +310,6 @@ class gconfig(config):
         else:
             printcfg(cfg, prefix=prefix, level=len(keys))
 
+
 # load on import if not already loaded
-gcfg = None
-if not gcfg:
-    gcfg = gconfig(SPEC, upg)
-    try:
-        gcfg.loadcfg(SITE_FILE, "site config")
-    except ParsecError as exc:
-        sys.stderr.write(
-            "WARNING: ignoring bad site GUI config %s:\n"
-            "%s\n" % (SITE_FILE, str(exc)))
-
-    if os.access(USER_FILE, os.F_OK | os.R_OK):
-        try:
-            gcfg.loadcfg(USER_FILE, "user config")
-        except ParsecError as exc:
-            sys.stderr.write("ERROR: bad user GUI config %s:\n" % USER_FILE)
-            raise
-
-    # check and correct initial view config etc.
-    gcfg.check()
-    # add spec defaults and do theme inheritance
-    gcfg.transform()
+gcfg = gconfig.get_inst()
