@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,13 +28,13 @@ from parsec.validate import validator as vdr
 from parsec.validate import coercers
 from parsec import ParsecError
 from parsec.upgrade import upgrader, converter
-from cylc.suite_host import get_user
+from cylc.hostuserutil import is_remote_user
 from cylc.envvar import expandvars
 from cylc.mkdir_p import mkdir_p
 import cylc.flags
 from cylc.cfgspec.utils import (
     coerce_interval, coerce_interval_list, DurationFloat)
-from cylc.network import PRIVILEGE_LEVELS
+from cylc.network import PRIVILEGE_LEVELS, PRIV_STATE_TOTALS, PRIV_SHUTDOWN
 
 coercers['interval'] = coerce_interval
 coercers['interval_list'] = coerce_interval_list
@@ -278,8 +278,9 @@ SPEC = {
         # control.
         'public': vdr(
             vtype='string',
-            options=PRIVILEGE_LEVELS[:PRIVILEGE_LEVELS.index('shutdown') + 1],
-            default="state-totals"),
+            options=(
+                PRIVILEGE_LEVELS[:PRIVILEGE_LEVELS.index(PRIV_SHUTDOWN) + 1]),
+            default=PRIV_STATE_TOTALS),
     },
 }
 
@@ -484,7 +485,8 @@ class GlobalConfig(config):
 
         return value
 
-    def get_host_item(self, item, host=None, owner=None, replace_home=False):
+    def get_host_item(self, item, host=None, owner=None, replace_home=False,
+                      owner_home=None):
         """This allows hosts with no matching entry in the config file
         to default to appropriately modified localhost settings."""
 
@@ -495,8 +497,6 @@ class GlobalConfig(config):
         if not host:
             # if no host is given the caller is asking about localhost
             host = 'localhost'
-        if not owner:
-            owner = get_user()
 
         # is there a matching host section?
         host_key = None
@@ -523,37 +523,43 @@ class GlobalConfig(config):
             if replace_home or modify_dirs:
                 # Replace local home dir with $HOME for eval'n on other host.
                 value = value.replace(os.environ['HOME'], '$HOME')
-            elif owner != get_user():
+            elif is_remote_user(owner):
                 # Replace with ~owner for direct access via local filesys
                 # (works for standard cylc-run directory location).
-                value = value.replace(
-                    os.environ['HOME'], os.path.expanduser('~%s' % owner))
+                if owner_home is None:
+                    owner_home = os.path.expanduser('~%s' % owner)
+                value = value.replace(os.environ['HOME'], owner_home)
+        if item == "task communication method" and value == "default":
+            # Translate "default" to client-server comms: "https" or "http".
+            value = cfg['communication']['method']
         return value
 
-    def roll_directory(self, d, name, archlen=0):
-        """
-        Create a directory after rolling back any previous instances of it.
-        e.g. if archlen = 2 we keep: d, d.1, d.2. If 0 keep no old ones.
+    def roll_directory(self, dir_, name, archlen=0):
+        """Create a directory after rolling back any previous instances of it.
+
+        E.g. if archlen = 2 we keep:
+            dir_, dir_.1, dir_.2. If 0 keep no old ones.
         """
         for n in range(archlen, -1, -1):  # archlen...0
             if n > 0:
-                dpath = d + '.' + str(n)
+                dpath = dir_ + '.' + str(n)
             else:
-                dpath = d
+                dpath = dir_
             if os.path.exists(dpath):
                 if n >= archlen:
                     # remove oldest backup
                     shutil.rmtree(dpath)
                 else:
                     # roll others over
-                    os.rename(dpath, d + '.' + str(n + 1))
-        self.create_directory(d, name)
+                    os.rename(dpath, dir_ + '.' + str(n + 1))
+        self.create_directory(dir_, name)
 
-    def create_directory(self, d, name):
+    @staticmethod
+    def create_directory(dir_, name):
         """Create directory. Raise GlobalConfigError on error."""
         try:
-            mkdir_p(d)
-        except Exception, exc:
+            mkdir_p(dir_)
+        except OSError as exc:
             print >> sys.stderr, str(exc)
             raise GlobalConfigError(
                 'Failed to create directory "' + name + '"')
