@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,22 +15,29 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+"""gscan config file format."""
 
 import os
 import sys
 
 from parsec import ParsecError
-from parsec.validate import validator as vdr
 from parsec.config import config
+from parsec.validate import coercers, validator as vdr
+from cylc.cfgspec.utils import (coerce_interval, DurationFloat)
 
-"""gscan config file format."""
 
+coercers['interval'] = coerce_interval
 USER_FILE = os.path.join(os.environ['HOME'], '.cylc', 'gscan.rc')
 
 SPEC = {
-    'columns': vdr(vtype='string_list', default=['suite', 'status']),
     'activate on startup': vdr(vtype='boolean', default=False),
+    'columns': vdr(vtype='string_list', default=['suite', 'status']),
+    'suite listing update interval': vdr(
+        vtype='interval', default=DurationFloat(60)),
+    'suite status update interval': vdr(
+        vtype='interval', default=DurationFloat(15)),
     'window size': vdr(vtype='integer_list', default=[300, 200]),
+    'hide main menubar': vdr(vtype='boolean', default=True),
 }
 
 
@@ -55,13 +62,13 @@ class GScanConfig(config):
         if 'columns' in cfg:
             for column in cfg['columns']:
                 if column not in self.COLS:
-                    print >> sys.stderr, (
-                        "WARNING: illegal column name '%s'" % column)
+                    sys.stderr.write(
+                        "WARNING: illegal column name '%s'\n" % column)
                     cfg['columns'].remove(column)
             if not cfg['columns']:
-                print >> sys.stderr, (
+                sys.stderr.write(
                     'WARNING: at least one column must be specified,' +
-                    ' defaulting to "%s, %s"' % self.COLS_DEFAULT)
+                    ' defaulting to "%s, %s"\n' % self.COLS_DEFAULT)
                 cfg['columns'] = list(self.COLS_DEFAULT)
 
 
