@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,11 +31,12 @@ from isodatetime.parsers import TimePointParser, DurationParser
 from cylc.cfgspec.utils import (
     coerce_interval, coerce_interval_list, DurationFloat)
 from cylc.cfgspec.globalcfg import GLOBAL_CFG
-from cylc.network import PRIVILEGE_LEVELS
+from cylc.network import PRIVILEGE_LEVELS, PRIV_SHUTDOWN
 from cylc.task_id import TaskID
 
 
-REC_PARAM_INT_RANGE = re.compile(r'\A(\d+)\.\.(\d+)(?:\.\.(\d+))?\Z')
+REC_PARAM_INT_RANGE = re.compile(
+    r'\A([\+\-]?\d+)\.\.([\+\-]?\d+)(?:\.\.(\d+))?\Z')
 
 
 def _coerce_cycleinterval(value, keys, _):
@@ -163,7 +164,7 @@ def _coerce_parameter_list(value, keys, _):
             if not step:
                 step = 1
             items.extend(range(int(lower), int(upper) + 1, int(step)))
-        elif TaskID.NAME_REC.match(item):
+        elif TaskID.NAME_SUFFIX_REC.match(item):
             if not item.isdigit():
                 if can_only_be == int:
                     raise IllegalValueError(
@@ -177,9 +178,8 @@ def _coerce_parameter_list(value, keys, _):
             not str(item).isdigit() for item in items):
         return items
     else:
-        items = [int(item) for item in items]
-        n_digits = len(str(max(items)))
-        return [str(item).zfill(n_digits) for item in sorted(items)]
+        return [int(item) for item in items]
+
 
 coercers['cycletime'] = _coerce_cycletime
 coercers['cycletime_format'] = _coerce_cycletime_format
@@ -192,10 +192,10 @@ coercers['parameter_list'] = _coerce_parameter_list
 
 
 SPEC = {
-    'group': vdr(vtype='string', default="(ungrouped)"),
     'meta': {
-        'title': vdr(vtype='string', default=""),
         'description': vdr(vtype='string', default=""),
+        'group': vdr(vtype='string', default=""),
+        'title': vdr(vtype='string', default=""),
         'URL': vdr(vtype='string', default=""),
         '__MANY__': vdr(vtype='string', default=""),
     },
@@ -285,7 +285,7 @@ SPEC = {
             'public': vdr(
                 vtype='string',
                 options=PRIVILEGE_LEVELS[
-                    :PRIVILEGE_LEVELS.index('shutdown') + 1],
+                    :PRIVILEGE_LEVELS.index(PRIV_SHUTDOWN) + 1],
                 default=GLOBAL_CFG.get(['authentication', 'public']))
         },
     },
@@ -340,7 +340,6 @@ SPEC = {
             'script': vdr(vtype='string', default=""),
             'post-script': vdr(vtype='string', default=""),
             'extra log files': vdr(vtype='string_list', default=[]),
-            'enable resurrection': vdr(vtype='boolean', default=False),
             'work sub-directory': vdr(vtype='string'),
             'meta': {
                 'title': vdr(vtype='string', default=""),
@@ -409,6 +408,7 @@ SPEC = {
                 'submission retry handler': vdr(vtype='string_list'),
                 'execution timeout handler': vdr(vtype='string_list'),
                 'submission timeout handler': vdr(vtype='string_list'),
+                'custom handler': vdr(vtype='string_list'),
             },
             'suite state polling': {
                 'user': vdr(vtype='string'),
@@ -428,6 +428,9 @@ SPEC = {
             'outputs': {
                 '__MANY__': vdr(vtype='string'),
             },
+            'parameter environment templates': {
+                '__MANY__': vdr(vtype='string'),
+            },
         },
     },
     'visualization': {
@@ -543,25 +546,22 @@ def upg(cfg, descr):
         '7.5.0',
         ['URL'],
         ['meta', 'URL'])
+    u.deprecate(
+        '7.6.0',
+        ['group'],
+        ['meta', 'group'])
     u.obsolete('7.2.2', ['cylc', 'dummy mode'])
     u.obsolete('7.2.2', ['cylc', 'simulation mode'])
     u.obsolete('7.2.2', ['runtime', '__MANY__', 'dummy mode'])
     u.obsolete('7.2.2', ['runtime', '__MANY__', 'simulation mode'])
+    u.obsolete('7.6.0', ['runtime', '__MANY__', 'enable resurrection'])
     u.upgrade()
 
 
 class RawSuiteConfig(config):
     """Raw suite configuration."""
-    _INSTANCES = {}
 
-    @classmethod
-    def get_inst(cls, fpath, is_reload=False, tvars=None, output_fname=None):
+    def __init__(self, fpath, output_fname, tvars):
         """Return the default instance."""
-        if fpath not in cls._INSTANCES or is_reload:
-            if tvars is None:
-                tvars = []
-            # TODO - output_fname should be in loadcfg
-            cls._INSTANCES[fpath] = cls(
-                SPEC, upg, tvars=tvars, output_fname=output_fname)
-            cls._INSTANCES[fpath].loadcfg(fpath, "suite definition")
-        return cls._INSTANCES[fpath]
+        config.__init__(self, SPEC, upg, output_fname, tvars)
+        self.loadcfg(fpath, "suite definition")
