@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -52,6 +52,7 @@ from isodatetime.parsers import DurationParser
 from parsec.OrderedDict import OrderedDictWithDefaults
 from parsec.util import replicate
 from cylc.suite_logging import OUT, ERR
+from cylc.suite_srv_files_mgr import SuiteSrvFilesManager
 from cylc.task_outputs import TASK_OUTPUT_SUCCEEDED
 
 RE_CLOCK_OFFSET = re.compile(r'(' + TaskID.NAME_RE + r')(?:\(\s*(.+)\s*\))?')
@@ -83,38 +84,15 @@ class SuiteConfigError(Exception):
 class SuiteConfig(object):
     """Class for suite configuration items and derived quantities."""
 
-    _INSTANCES = {}
     Q_DEFAULT = 'default'
-
-    @classmethod
-    def get_inst(cls, suite=None, fpath=None, template_vars=None,
-                 owner=None, run_mode='live', is_validate=False, strict=False,
-                 collapsed=[], cli_initial_point_string=None,
-                 cli_start_point_string=None, cli_final_point_string=None,
-                 is_reload=False, output_fname=None,
-                 vis_start_string=None, vis_stop_string=None,
-                 mem_log_func=None):
-        """Return a singleton instance.
-
-        On 1st call, instantiate the singleton.
-        Argument list is only relevant on 1st call.
-
-        """
-        if suite not in cls._INSTANCES or is_reload:
-            cls._INSTANCES[suite] = cls(
-                suite, fpath, template_vars, owner,
-                run_mode, is_validate, strict, collapsed,
-                cli_initial_point_string, cli_start_point_string,
-                cli_final_point_string, is_reload, output_fname,
-                vis_start_string, vis_stop_string, mem_log_func)
-            # Default instance
-            if suite is not None:
-                cls._INSTANCES[None] = cls._INSTANCES[suite]
-        return cls._INSTANCES[suite]
+    TASK_EVENT_TMPL_KEYS = (
+        'event', 'suite', 'point', 'name', 'submit_num', 'id', 'message',
+        'batch_sys_name', 'batch_sys_job_id', 'submit_time', 'start_time',
+        'finish_time', 'user@host')
 
     def __init__(self, suite, fpath, template_vars=None,
                  owner=None, run_mode='live', is_validate=False, strict=False,
-                 collapsed=[], cli_initial_point_string=None,
+                 collapsed=None, cli_initial_point_string=None,
                  cli_start_point_string=None, cli_final_point_string=None,
                  is_reload=False, output_fname=None,
                  vis_start_string=None, vis_stop_string=None,
@@ -176,11 +154,9 @@ class SuiteConfig(object):
 
         # parse, upgrade, validate the suite, but don't expand with default
         # items
-        self.mem_log("config.py: before RawSuiteConfig.get_inst")
-        self.pcfg = RawSuiteConfig.get_inst(
-            fpath, is_reload=is_reload, tvars=template_vars,
-            output_fname=output_fname)
-        self.mem_log("config.py: after RawSuiteConfig.get_inst")
+        self.mem_log("config.py: before RawSuiteConfig init")
+        self.pcfg = RawSuiteConfig(fpath, output_fname, template_vars)
+        self.mem_log("config.py: after RawSuiteConfig init")
         self.mem_log("config.py: before get(sparse=True")
         self.cfg = self.pcfg.get(sparse=True)
         self.mem_log("config.py: after get(sparse=True)")
@@ -275,16 +251,21 @@ class SuiteConfig(object):
 
         # Set default parameter expansion templates if necessary.
         for pname, pvalues in parameter_values.items():
-            if pname not in parameter_templates:
-                try:
-                    [int(i) for i in pvalues]
-                except ValueError:
-                    # Don't prefix string values with the parameter name.
-                    parameter_templates[pname] = "_%(" + pname + ")s"
+            if pvalues and pname not in parameter_templates:
+                if any(not isinstance(pvalue, int) for pvalue in pvalues):
+                    # Strings, bare parameter values
+                    parameter_templates[pname] = r'_%%(%s)s' % pname
+                elif any(pvalue < 0 for pvalue in pvalues):
+                    # Integers, with negative value(s)
+                    # Prefix values with signs and parameter names
+                    parameter_templates[pname] = r'_%s%%(%s)+0%dd' % (
+                        pname, pname,
+                        max(len(str(pvalue)) for pvalue in pvalues))
                 else:
-                    # All int values, prefix values with the parameter name.
-                    parameter_templates[pname] = (
-                        "_" + pname + "%(" + pname + ")s")
+                    # Integers, positive only
+                    # Prefix values with parameter names
+                    parameter_templates[pname] = r'_%s%%(%s)0%dd' % (
+                        pname, pname, len(str(max(pvalues))))
 
         # Expand parameters in 'special task' lists.
         if 'special tasks' in self.cfg['scheduling']:
@@ -300,15 +281,16 @@ class SuiteConfig(object):
                 self.cfg['scheduling']['queues'][queue]['members'] = (
                     self._expand_name_list(cfg['members']))
 
+        # Check environment variable names and parameter environment templates
+        # Done before inheritance to avoid repetition
+        self.check_env_names()
+        self.check_param_env_tmpls()
         self.mem_log("config.py: before _expand_runtime")
         self._expand_runtime()
         self.mem_log("config.py: after _expand_runtime")
 
         self.ns_defn_order = self.cfg['runtime'].keys()
 
-        # check var names before inheritance to avoid repetition
-        self.check_env_names()
-
         self.mem_log("config.py: before compute_family_tree")
         # do sparse inheritance
         self.compute_family_tree()
@@ -507,10 +489,12 @@ class SuiteConfig(object):
                     'ERROR [visualization]collapsed families: '
                     '%s is not a first parent' % fam)
 
-        if is_reload:
+        if is_reload and collapsed:
             # on suite reload retain an existing state of collapse
             # (used by the "cylc graph" viewer)
             self.closed_families = collapsed
+        elif is_reload:
+            self.closed_families = []
         else:
             self.closed_families = self.collapsed_families_rc
         for cfam in self.closed_families:
@@ -694,18 +678,21 @@ class SuiteConfig(object):
             if vfcp > final_point:
                 self.cfg['visualization']['final cycle point'] = str(
                     final_point)
-        # Replace suite name in suite  URL.
-        url = self.cfg['meta']['URL']
-        if url is not '':
-            self.cfg['meta']['URL'] = RE_SUITE_NAME_VAR.sub(self.suite, url)
 
-        # Replace suite and task name in task URLs.
+        # Replace suite and task name in suite and task URLs.
+        self.cfg['meta']['URL'] = self.cfg['meta']['URL'] % {
+            'suite_name': self.suite}
+        # back-compat $CYLC_SUITE_NAME:
+        self.cfg['meta']['URL'] = RE_SUITE_NAME_VAR.sub(
+            self.suite, self.cfg['meta']['URL'])
         for name, cfg in self.cfg['runtime'].items():
-            if cfg['meta']['URL']:
-                cfg['meta']['URL'] = RE_TASK_NAME_VAR.sub(
-                    name, cfg['meta']['URL'])
-                cfg['meta']['URL'] = RE_SUITE_NAME_VAR.sub(
-                    self.suite, cfg['meta']['URL'])
+            cfg['meta']['URL'] = cfg['meta']['URL'] % {
+                'suite_name': self.suite, 'task_name': name}
+            # back-compat $CYLC_SUITE_NAME and $CYLC_TASK_NAME:
+            cfg['meta']['URL'] = RE_SUITE_NAME_VAR.sub(
+                self.suite, cfg['meta']['URL'])
+            cfg['meta']['URL'] = RE_TASK_NAME_VAR.sub(
+                name, cfg['meta']['URL'])
 
         if is_validate:
             self.mem_log("config.py: before _check_circular()")
@@ -801,26 +788,25 @@ class SuiteConfig(object):
             for name, indices in name_expander.expand(namespace_heading):
                 if name not in newruntime:
                     newruntime[name] = OrderedDictWithDefaults()
-                # Put parameter index values in task environment.
                 replicate(newruntime[name], namespace_dict)
                 if indices:
+                    # Put parameter values in task environments.
+                    self.task_param_vars[name] = indices
                     new_environ = OrderedDictWithDefaults()
-                    self.task_param_vars[name] = {}
-                    for p_name, p_val in indices.items():
-                        p_var_name = 'CYLC_TASK_PARAM_%s' % p_name
-                        self.task_param_vars[name][p_var_name] = p_val
                     if 'environment' in newruntime[name]:
                         for k, v in newruntime[name]['environment'].items():
                             new_environ[k] = v
                     newruntime[name]['environment'] = new_environ
-                    if 'inherit' in newruntime[name]:
-                        parents = newruntime[name]['inherit']
-                        origin = 'inherit = %s' % ' '.join(parents)
-                        repl_parents = []
-                        for parent in parents:
-                            repl_parents.append(name_expander.replace_params(
+                if 'inherit' in newruntime[name]:
+                    # Allow inheritance from parameterized namespaces.
+                    parents = newruntime[name]['inherit']
+                    origin = 'inherit = %s' % ', '.join(parents)
+                    repl_parents = []
+                    for parent in parents:
+                        repl_parents.append(
+                            name_expander.expand_parent_params(
                                 parent, indices, origin))
-                        newruntime[name]['inherit'] = repl_parents
+                    newruntime[name]['inherit'] = repl_parents
         self.cfg['runtime'] = newruntime
 
         # Parameter expansion of visualization node attributes. TODO - do vis
@@ -837,7 +823,8 @@ class SuiteConfig(object):
                 expanded_node_attrs[name] = val
         self.cfg['visualization']['node attributes'] = expanded_node_attrs
 
-    def is_graph_defined(self, dependency_map):
+    @staticmethod
+    def is_graph_defined(dependency_map):
         for item, value in dependency_map.items():
             if item == 'graph':
                 # Async graph.
@@ -851,33 +838,63 @@ class SuiteConfig(object):
                             return True
         return False
 
-    def dequote(self, s):
+    @staticmethod
+    def dequote(s):
         """Strip quotes off a string."""
         if (s[0] == s[-1]) and s.startswith(("'", '"')):
             return s[1:-1]
         return s
 
     def check_env_names(self):
-        # check for illegal environment variable names
+        """Check for illegal environment variable names"""
         bad = {}
-        for label in self.cfg['runtime']:
-            res = []
-            if 'environment' in self.cfg['runtime'][label]:
-                res = check_varnames(self.cfg['runtime'][label]['environment'])
-            if res:
-                bad[label] = res
+        for label, item in self.cfg['runtime'].items():
+            for key in ('environment', 'parameter environment templates'):
+                if key in item:
+                    res = check_varnames(item[key])
+                    if res:
+                        bad[(label, key)] = res
         if bad:
             err_msg = "bad env variable names:"
-            for label, vars_ in bad.items():
-                err_msg += '\nNamespace:\t' + label
-                for var in vars_:
-                    err_msg += "\n\t\t" + var
+            for (label, key), names in bad.items():
+                err_msg += '\nNamespace:\t%s [%s]' % (label, key)
+                for name in names:
+                    err_msg += "\n\t\t%s" % name
             ERR.error(err_msg)
             raise SuiteConfigError(
                 "Illegal environment variable name(s) detected")
 
+    def check_param_env_tmpls(self):
+        """Check for illegal parameter environment templates"""
+        parameter_values = dict(
+            (key, values[0])
+            for key, values in self.parameters[0].items() if values)
+        bads = set()
+        for namespace, item in self.cfg['runtime'].items():
+            if 'parameter environment templates' not in item:
+                continue
+            for name, tmpl in item['parameter environment templates'].items():
+                try:
+                    value = tmpl % parameter_values
+                except KeyError:
+                    bads.add((namespace, name, tmpl, 'bad parameter'))
+                except TypeError:
+                    bads.add((
+                        namespace, name, tmpl,
+                        'wrong data type for parameter'))
+                except ValueError:
+                    bads.add((namespace, name, tmpl, 'bad template syntax'))
+                else:
+                    if value == tmpl:  # Not a template
+                        bads.add((namespace, name, tmpl, 'not a template'))
+        if bads:
+            ERR.error("bad parameter environment template:\n  %s" % (
+                "\n  ".join('[%s]%s=%s  # %s' % bad for bad in sorted(bads))))
+            raise SuiteConfigError(
+                "Illegal parameter environment template(s) detected")
+
     def filter_env(self):
-        # filter environment variables after sparse inheritance
+        """Filter environment variables after sparse inheritance"""
         for ns in self.cfg['runtime'].values():
             try:
                 oenv = ns['environment']
@@ -1173,8 +1190,8 @@ class SuiteConfig(object):
         # Check custom script is not defined for automatic suite polling tasks.
         for l_task in self.suite_polling_tasks:
             try:
-                cs = self.pcfg.getcfg(sparse=True)['runtime'][l_task]['script']
-            except:
+                cs = self.pcfg.get(sparse=True)['runtime'][l_task]['script']
+            except KeyError:
                 pass
             else:
                 if cs:
@@ -1289,10 +1306,11 @@ class SuiteConfig(object):
     def get_first_parent_descendants(self):
         return self.runtime['first-parent descendants']
 
-    def define_inheritance_tree(self, tree, hierarchy):
+    @staticmethod
+    def define_inheritance_tree(tree, hierarchy):
         """Combine inheritance hierarchies into a tree structure."""
-        for rt in hierarchy:
-            hier = copy(hierarchy[rt])
+        for rt_ in hierarchy:
+            hier = copy(hierarchy[rt_])
             hier.reverse()
             cur_tree = tree
             for item in hier:
@@ -1306,7 +1324,7 @@ class SuiteConfig(object):
                 if 'title' in self.cfg['runtime'][key]['meta']:
                     tree[key] = self.cfg['runtime'][key]['meta']['title']
                 else:
-                    tree[key] = 'No title provided'
+                    tree[key] = SuiteSrvFilesManager.NO_TITLE
             elif isinstance(val, dict):
                 self.add_tree_titles(val)
 
@@ -1330,7 +1348,7 @@ class SuiteConfig(object):
                 result[ns] = self.cfg['runtime'][ns]['meta']['title']
             else:
                 # no need to flesh out the full runtime just for title
-                result[ns] = "No title provided"
+                result[ns] = SuiteSrvFilesManager.NO_TITLE
 
         return result
 
@@ -1356,12 +1374,12 @@ class SuiteConfig(object):
             self.add_tree_titles(tree)
             # compute pre-title padding
             maxlen = 0
-            for ns in pruned_ancestors:
-                items = copy(pruned_ancestors[ns])
+            for namespace in pruned_ancestors:
+                items = copy(pruned_ancestors[namespace])
                 items.reverse()
-                for i in range(len(items)):
-                    tmp = 2 * i + 1 + len(items[i])
-                    if i == 0:
+                for itt, item in enumerate(items):
+                    tmp = 2 * itt + 1 + len(item)
+                    if itt == 0:
                         tmp -= 1
                     if tmp > maxlen:
                         maxlen = tmp
@@ -1402,7 +1420,28 @@ class SuiteConfig(object):
                     ('deprecated: [runtime][%s][job]shell=%s: '
                      'use of ksh to run cylc task job file') %
                     (taskdef.name, job_shell))
-
+            # Check custom event handler templates compat with task meta
+            if taskdef.rtconfig['events']:
+                subs = dict((key, key) for key in self.TASK_EVENT_TMPL_KEYS)
+                for key, value in self.cfg['meta'].items():
+                    subs['suite_' + key.lower()] = value
+                subs.update(taskdef.rtconfig['meta'])
+                # Back compat.
+                try:
+                    subs['task_url'] = subs['URL']
+                except KeyError:
+                    pass
+                for key, values in taskdef.rtconfig['events'].items():
+                    if values and (
+                            key == 'handlers' or key.endswith(' handler')):
+                        for value in values:
+                            try:
+                                value % subs
+                            except (KeyError, ValueError) as exc:
+                                raise SuiteConfigError(
+                                    'ERROR: bad task event handler template'
+                                    ' %s: %s: %s' % (
+                                        taskdef.name, value, repr(exc)))
         if cylc.flags.verbose:
             OUT.info("Checking for defined tasks not used in the graph")
             for name in self.cfg['runtime']:
@@ -1564,7 +1603,7 @@ class SuiteConfig(object):
                     ltaskdef.intercycle_offsets.add((None, seq))
                 else:
                     ltaskdef.intercycle_offsets.add(
-                        (str(first_point - last_point), seq))
+                        (str(-(last_point - first_point)), seq))
             elif offset:
                 if offset_is_irregular:
                     offset_tuple = (offset, seq)
@@ -1878,13 +1917,10 @@ class SuiteConfig(object):
         # Note we could exclude 'root' from this and disallow use of 'root' in
         # the graph (which would probably be quite reasonable).
         family_map = {}
-        runtime_families = self.runtime['descendants'].keys()
-        runtime_tasks = [
-            t for t in self.runtime['parents'].keys()
-            if t not in runtime_families]
-        for fam in runtime_families:
-            desc = self.runtime['descendants'][fam]
-            family_map[fam] = [t for t in desc if t in runtime_tasks]
+        for family, tasks in self.runtime['descendants'].iteritems():
+            family_map[family] = [t for t in tasks if (
+                t in self.runtime['parents'] and
+                t not in self.runtime['descendants'])]
 
         # Move a cylc-5 non-cycling graph to an R1 section.
         non_cycling_graph = self.cfg['scheduling']['dependencies']['graph']
@@ -2030,7 +2066,7 @@ class SuiteConfig(object):
             reversed(self.runtime['linearized ancestors'][name]))
 
         if name in self.task_param_vars:
-            taskd.param_var = self.task_param_vars[name]
+            taskd.param_var.update(self.task_param_vars[name])
 
         return taskd
 
