@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -342,6 +342,8 @@ class ISO8601Sequence(SequenceBase):
 
     def __init__(self, dep_section, context_start_point=None,
                  context_end_point=None):
+        SequenceBase.__init__(
+            self, dep_section, context_start_point, context_end_point)
         self.dep_section = dep_section
 
         if context_start_point is None:
@@ -415,12 +417,12 @@ class ISO8601Sequence(SequenceBase):
         """Deprecated: return the offset used for this sequence."""
         return self.offset
 
-    def set_offset(self, offset):
-        """Deprecated: alter state to offset the entire sequence."""
+    def set_offset(self, i_offset):
+        """Deprecated: alter state to i_offset the entire sequence."""
         if self.recurrence.start_point is not None:
-            self.recurrence.start_point += interval_parse(str(offset))
+            self.recurrence.start_point += interval_parse(str(i_offset))
         if self.recurrence.end_point is not None:
-            self.recurrence.end_point += interval_parse(str(offset))
+            self.recurrence.end_point += interval_parse(str(i_offset))
         self._cached_first_point_values = {}
         self._cached_next_point_values = {}
         self._cached_valid_point_booleans = {}
@@ -621,7 +623,7 @@ class ISO8601Sequence(SequenceBase):
             for recurrence_iso_point in self.recurrence:
                 prev = curr
                 curr = recurrence_iso_point
-            ret = ISO8601Point(str(recurrence_iso_point))
+            ret = ISO8601Point(str(curr))
             if self.exclusions and ret in self.exclusions:
                 return ISO8601Point(str(prev))
             return ret
@@ -655,8 +657,6 @@ def init_from_cfg(cfg):
         'cycle point num expanded year digits']
     time_zone = cfg['cylc']['cycle point time zone']
     custom_dump_format = cfg['cylc']['cycle point format']
-    initial_cycle_point = cfg['scheduling']['initial cycle point']
-    final_cycle_point = cfg['scheduling']['final cycle point']
     assume_utc = cfg['cylc']['UTC mode']
     cycling_mode = cfg['scheduling']['cycling mode']
 
@@ -724,16 +724,14 @@ def init(num_expanded_year_digits=0, custom_dump_format=None, time_zone=None,
 def get_point_relative(offset_string, base_point):
     """Create a point from offset_string applied to base_point."""
     try:
-        interval = ISO8601Interval(
-            str(interval_parse(offset_string)))
-    except Exception:
-        pass
+        interval = ISO8601Interval(str(interval_parse(offset_string)))
+    except ValueError:
+        return ISO8601Point(str(
+            SuiteSpecifics.abbrev_util.parse_timepoint(
+                offset_string, context_point=_point_parse(base_point.value))
+        ))
     else:
         return base_point + interval
-    return ISO8601Point(str(
-        SuiteSpecifics.abbrev_util.parse_timepoint(
-            offset_string, context_point=_point_parse(base_point.value))
-    ))
 
 
 def interval_parse(interval_string):
