@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,11 +15,18 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+"""Cylc-modified xdot windows for the "cylc graph" command.
+
+TODO - factor more commonality out of MyDotWindow, MyDotWindow2
+"""
+
 
 import gtk
 import os
 import re
+import sys
 import xdot
+
 from cylc.config import SuiteConfig
 from cylc.cycling.loader import get_point
 from cylc.graphing import CGraphPlain, CGraph
@@ -27,11 +34,6 @@ from cylc.gui import util
 from cylc.task_id import TaskID
 from cylc.suite_logging import ERR
 
-"""
-Cylc-modified xdot windows for the "cylc graph" command.
-TODO - factor more commonality out of MyDotWindow, MyDotWindow2
-"""
-
 
 def style_ghost_node(node):
     """Apply default style to a ghost node."""
@@ -41,7 +43,35 @@ def style_ghost_node(node):
 
 
 class CylcDotViewerCommon(xdot.DotWindow):
+
+    def __init__(self, suite, suiterc, template_vars, orientation="TB",
+                 should_hide=False, start_point_string=None,
+                 stop_point_string=None, interactive=True):
+        self.suite = suite
+        self.suiterc = None
+        self.template_vars = template_vars
+        self.orientation = orientation
+        self.should_hide = should_hide
+        self.start_point_string = start_point_string
+        self.stop_point_string = stop_point_string
+        self.interactive = interactive
+
+        self.outfile = None
+        self.disable_output_image = False
+        self.file = suiterc
+        self.filter_recs = []
+
+        util.setup_icons()
+        gtk.Window.__init__(self)
+        self.graph = xdot.Graph()
+        self.set_icon(util.get_icon())
+        self.set_default_size(512, 512)
+        self.vbox = gtk.VBox()
+        self.add(self.vbox)
+        self.widget = xdot.DotWidget()
+
     def load_config(self):
+        """Load the suite config."""
         if self.suiterc:
             is_reload = True
             collapsed = self.suiterc.closed_families
@@ -55,12 +85,27 @@ class CylcDotViewerCommon(xdot.DotWindow):
                 cli_initial_point_string=self.start_point_string,
                 vis_start_string=self.start_point_string,
                 vis_stop_string=self.stop_point_string)
-        except Exception, x:
-            ERR.error("Failed - parsing error?\n" + str(x))
-            return False
+        except Exception as exc:
+            msg = "Failed - parsing error?\n\n" + str(exc)
+            ERR.error(msg)
+            if self.interactive:
+                dia = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,
+                                        buttons=gtk.BUTTONS_OK,
+                                        message_format=msg)
+                dia.run()
+                dia.destroy()
+                return False
+            sys.exit(1)
         self.inherit = self.suiterc.get_parent_lists()
         return True
 
+    def on_refresh(self, w):
+        """Re-load the suite config and refresh the graph."""
+        if self.load_config():
+            self.get_graph()
+        else:
+            self.set_dotcode('graph {}')
+
     def set_filter_graph_patterns(self, filter_patterns):
         """Set some regular expressions to filter out graph nodes."""
         self.filter_recs = [re.compile(_) for _ in filter_patterns]
@@ -97,47 +142,21 @@ class MyDotWindow2(CylcDotViewerCommon):
     </ui>
     '''
 
-    def __init__(self, suite, suiterc, template_vars, orientation="TB",
-                 should_hide=False):
-        self.outfile = None
-        self.disable_output_image = False
-        self.suite = suite
-        self.file = suiterc
-        self.suiterc = None
-        self.orientation = orientation
-        self.template_vars = template_vars
-        self.start_point_string = None
-        self.stop_point_string = None
-        self.filter_recs = []
+    def __init__(self, *args, **kwargs):
+        CylcDotViewerCommon.__init__(self, *args, **kwargs)
 
-        util.setup_icons()
-
-        gtk.Window.__init__(self)
-
-        self.graph = xdot.Graph()
-
-        window = self
-
-        window.set_title('Cylc Suite Runtime Inheritance Graph Viewer')
-        window.set_default_size(512, 512)
-        window.set_icon(util.get_icon())
-
-        vbox = gtk.VBox()
-        window.add(vbox)
-
-        self.widget = xdot.DotWidget()
+        self.set_title('Cylc Suite Runtime Inheritance Graph Viewer')
 
         # Create a UIManager instance
         uimanager = self.uimanager = gtk.UIManager()
 
         # Add the accelerator group to the toplevel window
         accelgroup = uimanager.get_accel_group()
-        window.add_accel_group(accelgroup)
+        self.add_accel_group(accelgroup)
 
         # Create an ActionGroup
         actiongroup = gtk.ActionGroup('Actions')
         self.actiongroup = actiongroup
-
         actiongroup.add_actions((
             ('ZoomIn', gtk.STOCK_ZOOM_IN, None, None, 'Zoom In',
                 self.widget.on_zoom_in),
@@ -169,12 +188,12 @@ class MyDotWindow2(CylcDotViewerCommon):
         # Create a Toolbar
 
         toolbar = uimanager.get_widget('/ToolBar')
-        vbox.pack_start(toolbar, False)
-        vbox.pack_start(self.widget)
+        self.vbox.pack_start(toolbar, False)
+        self.vbox.pack_start(self.widget)
 
         self.set_focus(self.widget)
 
-        if not should_hide:
+        if not self.should_hide:
             self.show_all()
         self.load_config()
 
@@ -238,11 +257,6 @@ class MyDotWindow2(CylcDotViewerCommon):
         self.orientation = orientation
         self.get_graph()
 
-    def on_refresh(self, w):
-        self.load_config()
-        self.get_graph()
-        return True
-
 
 class MyDotWindow(CylcDotViewerCommon):
     """Override xdot to get rid of some buttons + parse graph from suite.rc"""
@@ -268,48 +282,28 @@ class MyDotWindow(CylcDotViewerCommon):
     </ui>
     '''
 
-    def __init__(self, suite, suiterc, start_point_string, stop_point_string,
-                 template_vars, orientation="TB",
-                 subgraphs_on=False, ignore_suicide=True, should_hide=False):
-        self.outfile = None
-        self.disable_output_image = False
-        self.suite = suite
-        self.file = suiterc
-        self.suiterc = None
-        self.orientation = orientation
-        self.subgraphs_on = subgraphs_on
-        self.template_vars = template_vars
-        self.ignore_suicide = ignore_suicide
-        self.start_point_string = start_point_string
-        self.stop_point_string = stop_point_string
-        self.filter_recs = []
+    def __init__(self, suite, suiterc, template_vars,
+                 start_point_string, stop_point_string, **kwargs):
+        self.subgraphs_on = kwargs.get('subgraphs_on', False)
+        self.ignore_suicide = kwargs.get('ignore_suicide', True)
+        for kwarg in ['subgraphs_on', 'ignore_suicide']:
+            try:
+                del kwargs[kwarg]
+            except KeyError:
+                pass
+        CylcDotViewerCommon.__init__(
+            self, suite, suiterc, template_vars,
+            start_point_string=start_point_string,
+            stop_point_string=stop_point_string, **kwargs)
 
-        util.setup_icons()
-
-        gtk.Window.__init__(self)
-
-        self.graph = xdot.Graph()
-
-        window = self
-
-        window.set_title('Cylc Suite Dependency Graph Viewer')
-        window.set_default_size(512, 512)
-        window.set_icon(util.get_icon())
-        vbox = gtk.VBox()
-        window.add(vbox)
-
-        self.widget = xdot.DotWidget()
+        self.set_title('Cylc Suite Dependency Graph Viewer')
 
         # Create a UIManager instance
         uimanager = self.uimanager = gtk.UIManager()
 
         # Add the accelerator group to the toplevel window
         accelgroup = uimanager.get_accel_group()
-        window.add_accel_group(accelgroup)
-
-        # Create an ActionGroup
-        actiongroup = gtk.ActionGroup('Actions')
-        self.actiongroup = actiongroup
+        self.add_accel_group(accelgroup)
 
         # create new stock icons for group and ungroup actions
         imagedir = os.environ['CYLC_DIR'] + '/images/icons'
@@ -320,6 +314,9 @@ class MyDotWindow(CylcDotViewerCommon):
             factory.add(i, iconset)
         factory.add_default()
 
+        # Create an ActionGroup
+        actiongroup = gtk.ActionGroup('Actions')
+        self.actiongroup = actiongroup
         actiongroup.add_actions((
             ('ZoomIn', gtk.STOCK_ZOOM_IN, None, None, 'Zoom In',
                 self.widget.on_zoom_in),
@@ -370,17 +367,17 @@ class MyDotWindow(CylcDotViewerCommon):
         # Create a Toolbar
 
         toolbar = uimanager.get_widget('/ToolBar')
-        vbox.pack_start(toolbar, False)
-        vbox.pack_start(self.widget)
+        self.vbox.pack_start(toolbar, False)
+        self.vbox.pack_start(self.widget)
 
         eb = gtk.EventBox()
         eb.add(gtk.Label("right-click on nodes to control family grouping"))
         eb.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('#8be'))
-        vbox.pack_start(eb, False)
+        self.vbox.pack_start(eb, False)
 
         self.set_focus(self.widget)
 
-        if not should_hide:
+        if not self.should_hide:
             self.show_all()
         self.load_config()
 
@@ -495,11 +492,6 @@ class MyDotWindow(CylcDotViewerCommon):
         self.orientation = orientation
         self.get_graph()
 
-    def on_refresh(self, w):
-        self.load_config()
-        self.get_graph()
-        return True
-
 
 class DotTipWidget(xdot.DotWidget):
 
@@ -550,8 +542,8 @@ class xdot_widgets(object):
     def get(self):
         return self.vbox
 
-    def set_filter(self, filter):
-        self.widget.set_filter(filter)
+    def set_filter(self, filter_):
+        self.widget.set_filter(filter_)
 
     def set_dotcode(self, dotcode, filename='<stdin>', no_zoom=False):
         if no_zoom:
@@ -598,4 +590,4 @@ def get_reference_from_plain_format(plain_text):
                 pass
         indexed_lines.append((line_items, line))
     indexed_lines.sort()
-    return "".join([l[1] for l in indexed_lines])
+    return "".join(l[1] for l in indexed_lines)
