@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,13 +26,13 @@ from cylc.suite_logging import SuiteLog
 
 SUITE_SCAN_INFO_TMPL = r"""
 
-To view suite daemon contact information:
+To view suite server program contact information:
  $ cylc get-suite-contact %(suite)s
 
 Other ways to see if the suite is still running:
  $ cylc scan -n '\b%(suite)s\b' %(host)s
  $ cylc ping -v --host=%(host)s %(suite)s
- $ ssh %(host)s "pgrep -a -P 1 -fu $USER 'cylc-r.* \b%(suite)s\b'"
+ $ ps %(ps_opts)s %(pid)s  # on %(host)s
 
 """
 
@@ -70,26 +70,33 @@ def daemonize(server):
                     suite_pid is None or suite_port is None):
                 sleep(0.1)
                 try:
+                    # Line 1 (or 2 in debug mode) of suite log should contain
+                    # start up message, host name and port number. Format is:
+                    #  LOG-PREFIX Suite starting: server=HOST:PORT, pid=PID
+                    # Otherwise, something has gone wrong, print the suite log
+                    # and exit with an error.
                     log_stat = os.stat(log_fname)
                     if (log_stat.st_mtime == old_log_mtime or
                             log_stat.st_size == 0):
                         continue
-                    # Line 1 of suite log should contain start up message, host
-                    # name and port number. Format is:
-                    # LOG-PREIFX Suite starting: server=HOST:PORT, pid=PID
-                    # Otherwise, something has gone wrong, print the suite log
-                    # and exit with an error.
-                    log_line1 = open(log_fname).readline()
-                    if server.START_MESSAGE_PREFIX in log_line1:
-                        server_str, pid_str = log_line1.rsplit()[-2:]
-                        suite_pid = pid_str.rsplit("=", 1)[-1]
-                        suite_port = server_str.rsplit(":", 1)[-1]
-                    else:
+                    with open(log_fname) as log_f:
+                        try:
+                            first_two_lines = next(log_f), next(log_f)
+                        except StopIteration:
+                            continue
+                    ok = False
+                    for log_line in first_two_lines:
+                        if server.START_MESSAGE_PREFIX in log_line:
+                            ok = True
+                            server_str, pid_str = log_line.rsplit()[-2:]
+                            suite_pid = pid_str.rsplit("=", 1)[-1]
+                            suite_port = server_str.rsplit(":", 1)[-1]
+                    if not ok:
                         try:
                             sys.stderr.write(open(log_fname).read())
                             sys.exit(1)
                         except IOError:
-                            sys.exit("Suite daemon exited")
+                            sys.exit("Suite server program exited")
                 except (IOError, OSError, ValueError):
                     pass
             if suite_pid is None or suite_port is None:
@@ -99,6 +106,7 @@ def daemonize(server):
                 "suite": server.suite,
                 "host": server.host,
                 "port": suite_port,
+                "ps_opts": server.suite_srv_files_mgr.PS_OPTS,
                 "pid": suite_pid,
                 "logd": logd,
             })
