@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,12 +18,6 @@
 
 """Some global flags used in cylc"""
 
-# Set pflag = True to stimulate task dependency negotiation whenever a
-# task changes state in such a way that others could be affected. The
-# flag should only be turned off again after use in scheduler.py, to
-# ensure that dependency negotation occurs when required.
-pflag = False
-
 # Set iflag = True to simulate an update of the suite state summary
 # structure accessed by gcylc and commands.
 iflag = False
