@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,14 +15,13 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+"""Module for parsing cylc graph strings."""
 
 import re
 import unittest
-from cylc.param_expand import GraphExpander
+from cylc.param_expand import GraphExpander, ParamExpandError
 from cylc.task_id import TaskID
 
-"""Module for parsing cylc graph strings."""
-
 
 ARROW = '=>'
 
@@ -102,14 +101,14 @@ class GraphParser(object):
     _RE_TRIG = r':[\w\-]+'
 
     # Match fully qualified parameterized single nodes.
-    REC_NODE_FULL = re.compile(r'''
-        (
-        (?:''' + _RE_NODE + r''')    # node name
-        (?:''' + _RE_PARAMS + r''')? # optional parameter list
+    REC_NODE_FULL = re.compile(
+        r'''(
+        (?:''' +
+        _RE_NODE + r'|' + _RE_PARAMS + r'|' + _RE_NODE + _RE_PARAMS +
+        ''')                         # node name
         (?:''' + _RE_OFFSET + r''')? # optional cycle point offset
         (?:''' + _RE_TRIG + r''')?   # optional trigger type
-        )
-    ''', re.X)           # end of string
+        )''', re.X)           # end of string
 
     # Extract node info from a left-side expression, after parameter expansion.
     REC_NODES = re.compile(r'''
@@ -246,8 +245,11 @@ class GraphParser(object):
             if not self.__class__.REC_PARAMS.search(line):
                 line_set.add(line)
                 continue
-            for l in graph_expander.expand(line):
-                line_set.add(l)
+            try:
+                for l in graph_expander.expand(line):
+                    line_set.add(l)
+            except ParamExpandError as exc:
+                raise GraphParseError(str(exc))
 
         # Process chains of dependencies as pairs: left => right.
         # Parameterization can duplicate some dependencies, so use a set.
@@ -337,9 +339,10 @@ class GraphParser(object):
                 if not trig:
                     trig = self.__class__.TRIG_SUCCEED
                     if offset:
-                        this = r'\b%s\b%s(?!:)' % (name, re.escape(offset))
+                        this = r'\b%s\b%s(?!:)' % (
+                            re.escape(name), re.escape(offset))
                     else:
-                        this = r'\b%s\b(?![\[:])' % name
+                        this = r'\b%s\b(?![\[:])' % re.escape(name)
 
                     that = name + offset + trig
                     expr = re.sub(this, that, expr)
@@ -671,23 +674,22 @@ class TestGraphParser(unittest.TestCase):
         The correct format is:
           NAME(<PARAMS>)([CYCLE-POINT-OFFSET])(:TRIGGER-TYPE)")
         """
-        graphs = [
-            "foo[-P1Y]<m,n> => bar",
-            "foo[-P1Y]<m,n> => bar",
-            "foo:fail[-P1Y] => bar",
-            "foo[-P1Y]:fail<m,n> => bar",
-            "foo[-P1Y]<m,n>:fail => bar",
-            "foo<m,n>:fail[-P1Y] => bar",
-            "foo:fail<m,n>[-P1Y] => bar"
-        ]
-        detected = 0
-        for graph in graphs:
-            gp = GraphParser()
-            try:
-                gp.parse_graph(graph)
-            except GraphParseError:
-                detected += 1
-        self.assertEqual(detected, len(graphs))
+        gp = GraphParser()
+        self.assertRaises(
+            GraphParseError, gp.parse_graph, "foo[-P1Y]<m,n> => bar")
+        self.assertRaises(
+            GraphParseError, gp.parse_graph, "foo[-P1Y]<m,n> => bar")
+        self.assertRaises(
+            GraphParseError, gp.parse_graph, "foo:fail[-P1Y] => bar")
+        self.assertRaises(
+            GraphParseError, gp.parse_graph, "foo[-P1Y]:fail<m,n> => bar")
+        self.assertRaises(
+            GraphParseError, gp.parse_graph, "foo[-P1Y]<m,n>:fail => bar")
+        self.assertRaises(
+            GraphParseError, gp.parse_graph, "foo<m,n>:fail[-P1Y] => bar")
+        self.assertRaises(
+            GraphParseError, gp.parse_graph, "foo:fail<m,n>[-P1Y] => bar")
+
 
 if __name__ == "__main__":
     unittest.main()
