@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,11 +39,13 @@ def cat_state(suite, host=None, owner=None):
         stderr = PIPE
     cmd.append(suite)
     try:
-        proc = Popen(cmd, stderr=stderr, stdout=PIPE, preexec_fn=os.setpgrp)
+        proc = Popen(
+            cmd, stdin=open(os.devnull), stderr=stderr, stdout=PIPE,
+            preexec_fn=os.setpgrp)
     except OSError:
         return []
     else:
-        out, err = proc.communicate()
+        out = proc.communicate()[0]
         if proc.wait():  # non-zero return code
             return []
         return out.splitlines()
