@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,8 +17,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-class rotator(object):
-    def __init__(self, colors=['#fcc', '#cfc', '#bbf', '#ffb']):
+class ColorRotator(object):
+
+    COLORS = ('#fcc', '#cfc', '#bbf', '#ffb')
+
+    def __init__(self, colors=COLORS):
         self.colors = colors
         self.current_color = 0
 
