@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,10 +25,11 @@ import threading
 
 from cylc.gui.warning_dialog import warning_dialog, info_dialog
 from cylc.gui.util import get_icon, EntryTempText, EntryDialog
-from cylc.network.port_scan import scan_all
+from cylc.network.port_scan import scan_many, get_scan_items_from_fs
 from cylc.suite_srv_files_mgr import SuiteSrvFilesManager
 from cylc.run_get_stdout import run_get_stdout
-from cylc.suite_host import is_remote_host, is_remote_user
+from cylc.hostuserutil import is_remote_host, is_remote_user
+from cylc.suite_status import KEY_NAME, KEY_OWNER
 
 
 class db_updater(threading.Thread):
@@ -62,12 +63,7 @@ class db_updater(threading.Thread):
 
         for suite, auth in self.running_choices:
             if suite in regd_choices:
-                if is_remote_host(auth.split(':', 1)[0]):
-                    descr, suite_dir = (None, None)
-                else:
-                    # local suite
-                    suite_dir, descr = regd_choices[suite][1:3]
-                    del regd_choices[suite]
+                suite_dir, descr = regd_choices.pop(suite)[1:3]
             nest2 = self.newtree
             regp = suite.split(SuiteSrvFilesManager.DELIM)
             for key in regp[:-1]:
@@ -109,12 +105,13 @@ class db_updater(threading.Thread):
 
         # Scan for running suites
         choices = []
-        for host, port, suite_identity in scan_all(timeout=self.timeout):
-            name = suite_identity['name']
-            owner = suite_identity['owner']
+        for host, port, suite_identity in scan_many(
+                get_scan_items_from_fs(), timeout=self.timeout):
+            name = suite_identity[KEY_NAME]
+            owner = suite_identity[KEY_OWNER]
             if is_remote_user(owner):
                 continue  # current user only
-            auth = "%s:%d" % (host, port)
+            auth = "%s:%s" % (host, port)
             choices.append((name, auth))
         choices.sort()
         self.next_scan_time = time() + self.SCAN_INTERVAL
@@ -217,7 +214,7 @@ class db_updater(threading.Thread):
                         piter, [item] + [None, None, None, None, None, None])
                     self.build_treestore(new[key], xiter)
                 else:
-                    state, descr = new[key][0:2]
+                    state = new[key][0]
                     ts.append(
                         piter, [item] + new[key] + list(self.statecol(state)))
             else:
@@ -236,15 +233,17 @@ class db_updater(threading.Thread):
             gobject.idle_add(self.update)
             sleep(0.1)
 
-    def statecol(self, state):
-        bg = '#19ae0a'
-        fg = '#030'
+    @staticmethod
+    def statecol(state):
+        bg_ = '#19ae0a'
+        fg_ = '#030'
         if state == '-':
             return (None, None, None)
         else:
-            return (fg, bg, bg)
+            return (fg_, bg_, bg_)
 
-    def search_level(self, model, iter_, func, data):
+    @staticmethod
+    def search_level(model, iter_, func, data):
         while iter_:
             if func(model, iter_, data):
                 return iter_
@@ -262,7 +261,8 @@ class db_updater(threading.Thread):
             iter_ = model.iter_next(iter_)
         return None
 
-    def match_func(self, model, iter_, data):
+    @staticmethod
+    def match_func(model, iter_, data):
         column, key = data
         value = model.get_value(iter_, column)
         return value == key
@@ -416,7 +416,7 @@ class dbchooser(object):
         filtr = filtr_e.get_text()
         try:
             re.compile(filtr)
-        except:
+        except re.error:
             warning_dialog(
                 "Bad Regular Expression: " + filtr, self.window).warn()
             filtr_e.set_text("")
