@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,14 +16,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from cylc.gui.tailer import Tailer
-import gtk
 import gobject
+import gtk
+import os
 import pango
+from subprocess import Popen, STDOUT
 import tempfile
-from warning_dialog import warning_dialog, info_dialog
-from util import get_icon
-import subprocess
+
+from cylc.gui.tailer import Tailer
+from cylc.gui.util import get_icon
+from cylc.gui.warning_dialog import warning_dialog, info_dialog
 
 # unit test: see the command $CYLC_DIR/bin/gcapture
 
@@ -45,6 +47,8 @@ are displayed in red.
         self.command = command
         self.ignore_command = ignore_command
         self.stdout = stdoutfile
+        self.stdout_updater = None
+        self.proc = None
         self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
         self.window.set_border_width(5)
         if title is None:
@@ -141,9 +145,9 @@ are displayed in red.
     def run(self):
         proc = None
         if not self.ignore_command:
-            proc = subprocess.Popen(
-                self.command, stdout=self.stdout, stderr=subprocess.STDOUT,
-                shell=True)
+            proc = Popen(
+                self.command, stdin=open(os.devnull), stdout=self.stdout,
+                stderr=STDOUT, shell=True)
             self.proc = proc
             gobject.timeout_add(40, self.pulse_proc_progress)
         self.stdout_updater = Tailer(
@@ -185,10 +189,10 @@ are displayed in red.
                 gtk.RESPONSE_CANCEL,
                 gtk.STOCK_SAVE,
                 gtk.RESPONSE_OK))
-        filter = gtk.FileFilter()
-        filter.set_name("any")
-        filter.add_pattern("*")
-        dialog.add_filter(filter)
+        filter_ = gtk.FileFilter()
+        filter_.set_name("any")
+        filter_.add_pattern("*")
+        dialog.add_filter(filter_)
 
         response = dialog.run()
 
@@ -251,15 +255,17 @@ are displayed in red.
             s = tb.get_end_iter()
             tv.scroll_to_iter(s, 0)
         try:
-            f, l = s.backward_search(needle, gtk.TEXT_SEARCH_VISIBLE_ONLY)
-        except:
+            start, end = s.backward_search(needle,
+                                           gtk.TEXT_SEARCH_VISIBLE_ONLY)
+        except TypeError:
+            # Search pattern not found.
             warning_dialog(
                 '"' + needle + '"' + " not found", self.window).warn()
         else:
-            tb.apply_tag(self.ftag, f, l)
-            self.find_current_iter = f
+            tb.apply_tag(self.ftag, start, end)
+            self.find_current_iter = start
             self.find_current = needle
-            tv.scroll_to_iter(f, 0)
+            tv.scroll_to_iter(start, 0)
 
     def _handle_proc_completed(self):
         self.progress_bar.hide()
