@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
 
 import gtk
 
-from gcapture import gcapture_tmpfile
-from warning_dialog import warning_dialog
+from cylc.gui.gcapture import gcapture_tmpfile
+from cylc.gui.warning_dialog import warning_dialog
 
 
 def graph_suite_popup(reg, cmd_help, defstartc, defstopc, graph_opts,
@@ -28,8 +28,8 @@ def graph_suite_popup(reg, cmd_help, defstartc, defstopc, graph_opts,
     """Popup a dialog to allow a user to configure their suite graphing."""
     try:
         import xdot
-    except Exception, x:
-        warning_dialog(str(x) + "\nGraphing disabled.", parent_window).warn()
+    except ImportError as exc:
+        warning_dialog(str(exc) + "\nGraphing disabled.", parent_window).warn()
         return False
 
     window = gtk.Window()
