@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@ from isodatetime.data import (
     get_timepoint_from_seconds_since_unix_epoch as timepoint_from_epoch)
 
 from cylc.cfgspec.gcylc import gcfg
-from cylc.cfgspec.globalcfg import GLOBAL_CFG
 from cylc.cfgspec.gscan import gsfg
 from cylc.gui.legend import ThemeLegendWindow
 from cylc.gui.dot_maker import DotMaker
@@ -36,9 +35,9 @@ from cylc.gui.scanutil import (
     KEY_PORT, get_scan_menu, launch_gcylc, update_suites_info,
     launch_hosts_dialog, launch_about_dialog)
 from cylc.gui.util import get_icon, setup_icons, set_exception_hook_dialog
-from cylc.network import (
-    KEY_GROUP, KEY_STATES, KEY_TASKS_BY_STATE, KEY_TITLE, KEY_UPDATE_TIME)
-from cylc.suite_host import get_user
+from cylc.suite_status import (
+    KEY_GROUP, KEY_META, KEY_STATES, KEY_TASKS_BY_STATE, KEY_TITLE,
+    KEY_UPDATE_TIME)
 from cylc.task_state import (
     TASK_STATUSES_ORDERED, TASK_STATUS_RUNAHEAD, TASK_STATUS_FAILED,
     TASK_STATUS_SUBMIT_FAILED)
@@ -63,22 +62,17 @@ class ScanApp(object):
 
     def __init__(
             self, hosts=None, patterns_name=None, patterns_owner=None,
-            comms_timeout=None, poll_interval=None):
+            comms_timeout=None, interval=None):
         gobject.threads_init()
         set_exception_hook_dialog("cylc gscan")
         setup_icons()
-        if not hosts:
-            hosts = GLOBAL_CFG.get(["suite host scanning", "hosts"])
-        self.hosts = hosts
 
         self.window = gtk.Window()
         title = "cylc gscan"
-        for opt, items, skip in [
-                ("-n", patterns_name, None),
-                ("-o", patterns_owner, get_user())]:
+        for opt, items in [("-n", patterns_name), ("-o", patterns_owner)]:
             if items:
                 for pattern in items:
-                    if pattern != skip:
+                    if pattern is not None:
                         title += " %s %s" % (opt, pattern)
         self.window.set_title(title)
         self.window.set_icon(get_icon())
@@ -106,15 +100,10 @@ class ScanApp(object):
 
         # Visibility of columns
         vis_cols = gsfg.get(["columns"])
+        hide_main_menu_bar = gsfg.get(["hide main menubar"])
         # Doesn't make any sense without suite name column
         if gsfg.COL_SUITE not in vis_cols:
             vis_cols.append(gsfg.COL_SUITE.lower())
-        # In multiple host environment, add host column by default
-        if hosts != ['localhost']:
-            vis_cols.append(gsfg.COL_HOST.lower())
-        # In multiple owner environment, add owner column by default
-        if patterns_owner != [get_user()]:
-            vis_cols.append(gsfg.COL_OWNER.lower())
         # Construct the group, host, owner, suite, title, update time column.
         for col_title, col_id, col_cell_text_setter in [
                 (gsfg.COL_GROUP, self.GROUP_COLUMN, self._set_cell_text_group),
@@ -191,16 +180,18 @@ class ScanApp(object):
                     raise ValueError("Invalid %s pattern: %s" % (label, items))
 
         self.updater = ScanAppUpdater(
-            self.window, self.hosts, suite_treemodel, self.treeview,
-            comms_timeout=comms_timeout, poll_interval=poll_interval,
+            self.window, hosts, suite_treemodel, self.treeview,
+            comms_timeout=comms_timeout, interval=interval,
             group_column_id=self.GROUP_COLUMN,
             name_pattern=patterns["name"], owner_pattern=patterns["owner"])
 
         self.updater.start()
 
         self.dot_size = gcfg.get(['dot icon size'])
+        self.dots = None
         self._set_dots()
 
+        self.menu_bar = None
         self.create_menubar()
 
         accelgroup = gtk.AccelGroup()
@@ -209,13 +200,15 @@ class ScanApp(object):
         accelgroup.connect_group(
             key, modifier, gtk.ACCEL_VISIBLE, self._toggle_hide_menu_bar)
 
+        self.tool_bar = None
         self.create_tool_bar()
 
         self.menu_hbox = gtk.HBox()
         self.menu_hbox.pack_start(self.menu_bar, expand=True, fill=True)
         self.menu_hbox.pack_start(self.tool_bar, expand=True, fill=True)
         self.menu_hbox.show_all()
-        self.menu_hbox.hide_all()
+        if hide_main_menu_bar:
+            self.menu_hbox.hide_all()
 
         scrolled_window = gtk.ScrolledWindow()
         scrolled_window.set_policy(gtk.POLICY_AUTOMATIC,
@@ -346,7 +339,7 @@ class ScanApp(object):
         hosts_item.connect(
             "activate",
             lambda w: launch_hosts_dialog(
-                self.hosts, self.updater.set_hosts))
+                self.updater.hosts, self.updater.set_hosts))
         view_menu.append(hosts_item)
 
         sep_item = gtk.SeparatorMenuItem()
@@ -366,8 +359,8 @@ class ScanApp(object):
         info_item.set_image(img)
         info_item.show()
         info_item.connect(
-            "button-press-event",
-            lambda b, e: launch_about_dialog("cylc gscan", self.hosts)
+            "activate",
+            lambda w: launch_about_dialog("cylc gscan", self.updater.hosts)
         )
         help_menu.append(info_item)
 
@@ -383,12 +376,11 @@ class ScanApp(object):
         update_now_button = gtk.ToolButton(
             icon_widget=gtk.image_new_from_stock(
                 gtk.STOCK_REFRESH, gtk.ICON_SIZE_SMALL_TOOLBAR))
-        update_now_button.set_label("Update")
+        update_now_button.set_label("Update Listing")
         tooltip = gtk.Tooltips()
         tooltip.enable()
-        tooltip.set_tip(update_now_button, "Update now")
-        update_now_button.connect("clicked",
-                                  self.updater.update_now)
+        tooltip.set_tip(update_now_button, "Update Suite Listing Now")
+        update_now_button.connect("clicked", self.updater.set_update_listing)
 
         clear_stopped_button = gtk.ToolButton(
             icon_widget=gtk.image_new_from_stock(
@@ -407,13 +399,12 @@ class ScanApp(object):
         tooltip = gtk.Tooltips()
         tooltip.enable()
         tooltip.set_tip(expand_button, "Expand all rows")
-        expand_button.connect(
-            "clicked", lambda e: self.treeview.expand_all())
+        expand_button.connect("clicked", lambda e: self.treeview.expand_all())
 
         collapse_button = gtk.ToolButton(
             icon_widget=gtk.image_new_from_stock(
                 gtk.STOCK_REMOVE, gtk.ICON_SIZE_SMALL_TOOLBAR))
-        collapse_button.set_label("Expand all")
+        collapse_button.set_label("Collapse all")
         tooltip = gtk.Tooltips()
         tooltip.enable()
         tooltip.set_tip(collapse_button, "Collapse all rows")
@@ -487,7 +478,7 @@ class ScanApp(object):
                         self.HOST_COLUMN, self.OWNER_COLUMN, self.SUITE_COLUMN)
                     suite_keys.append((host, owner, suite))
 
-                except:
+                except TypeError:
                     # Now iterate over the children instead.
                     # We need to iterate over the children as there can be more
                     # than one suite in a group of suites.
@@ -563,12 +554,11 @@ class ScanApp(object):
             return True
         if column.get_title() == gsfg.COL_UPDATED:
             suite_update_point = timepoint_from_epoch(suite_update_time)
-            if (self.updater.last_update_time is not None and
-                    suite_update_time != int(self.updater.last_update_time)):
-                retrieval_point = timepoint_from_epoch(
-                    int(self.updater.last_update_time))
+            if (self.updater.prev_norm_update is not None and
+                    suite_update_time != int(self.updater.prev_norm_update)):
                 text = "Last changed at %s\n" % suite_update_point
-                text += "Last scanned at %s" % retrieval_point
+                text += "Last scanned at %s" % timepoint_from_epoch(
+                    int(self.updater.prev_norm_update))
             else:
                 # An older suite (or before any updates are made?)
                 text = "Last scanned at %s" % suite_update_point
@@ -765,20 +755,24 @@ class ScanAppUpdater(threading.Thread):
 
     """Update the scan app."""
 
-    POLL_INTERVAL = 60
+    UNGROUPED = "(ungrouped)"
 
     def __init__(self, window, hosts, suite_treemodel, suite_treeview,
-                 comms_timeout=None, poll_interval=None, group_column_id=0,
+                 comms_timeout=None, interval=None, group_column_id=0,
                  name_pattern=None, owner_pattern=None):
         self.window = window
-        self.hosts = hosts
+        if hosts:
+            self.hosts = hosts
+        else:
+            self.hosts = []
         self.comms_timeout = comms_timeout
-        if poll_interval is None:
-            poll_interval = self.POLL_INTERVAL
-        self.poll_interval = poll_interval
+        if interval is None:
+            interval = gsfg.get(['suite listing update interval'])
+        self.interval_full = interval
+        self.interval_part = gsfg.get(['suite status update interval'])
         self.suite_info_map = {}
-        self.last_update_time = None
-        self._should_force_update = False
+        self.prev_full_update = None
+        self.prev_norm_update = None
         self.quit = False
         self.suite_treemodel = suite_treemodel
         self.treeview = suite_treeview
@@ -826,8 +820,7 @@ class ScanAppUpdater(threading.Thread):
         failed_tasks = []
         for state in (TASK_STATUS_FAILED, TASK_STATUS_SUBMIT_FAILED):
             failed_tasks += self.tasks_by_state[key].get(state, [])
-        warnings = [warn for warn in failed_tasks if warn[0] > warn_time]
-        warnings.sort()
+        warnings = sorted(warn for warn in failed_tasks if warn[0] > warn_time)
         return warnings[-5:]
 
     def clear_stopped_suites(self, _=None):
@@ -875,36 +868,40 @@ class ScanAppUpdater(threading.Thread):
 
     def has_stopped_suites(self):
         """Return True if we have any stopped suite information."""
-        for result in self.suite_info_map.copy().values():
-            if KEY_PORT not in result:
-                return True
-        return False
+        return any(
+            KEY_PORT not in result for result in self.suite_info_map.values())
 
     def run(self):
         """Execute the main loop of the thread."""
         while not self.quit:
-            time_for_update = (
-                self.last_update_time is None or
-                time() >= self.last_update_time + self.poll_interval
-            )
-            if not self._should_force_update and not time_for_update:
-                sleep(1)
-                continue
-            if self._should_force_update:
-                self._should_force_update = False
-            title = self.window.get_title()
-            gobject.idle_add(self.window.set_title, title + " (updating)")
-            self.suite_info_map = update_suites_info(self)
-            self.last_update_time = time()
-            gobject.idle_add(self.window.set_title, title)
-            gobject.idle_add(self.update)
+            now = time()
+            full_mode = (
+                self.prev_full_update is None or
+                now >= self.prev_full_update + self.interval_full)
+            if (full_mode or
+                    self.prev_norm_update is None or
+                    now >= self.prev_norm_update + self.interval_part):
+                title = self.window.get_title()
+                if full_mode and self.hosts:
+                    active_title = "%s (scanning + updating)" % title
+                elif full_mode:
+                    active_title = "%s (listing + updating)" % title
+                else:
+                    active_title = "%s (updating)" % title
+                gobject.idle_add(self.window.set_title, active_title)
+                self.suite_info_map = update_suites_info(self, full_mode)
+                self.prev_norm_update = time()
+                if full_mode:
+                    self.prev_full_update = self.prev_norm_update
+                gobject.idle_add(self.window.set_title, title)
+                gobject.idle_add(self.update)
             sleep(1)
 
     def set_hosts(self, new_hosts):
         """Set new hosts."""
         del self.hosts[:]
         self.hosts.extend(new_hosts)
-        self.update_now()
+        self.set_update_listing()
 
     def update(self):
         """Update the Applet."""
@@ -914,13 +911,26 @@ class ScanAppUpdater(threading.Thread):
         group_counts = self._update_group_counts()
         self.suite_treemodel.clear()
         group_iters = {}
+        hosts = set()
+        owners = set()
         for key, suite_info in sorted(self.suite_info_map.items()):
             host, owner, suite = key
+            hosts.add(host)
+            owners.add(owner)
             suite_updated_time = suite_info.get(KEY_UPDATE_TIME)
             if suite_updated_time is None:
                 suite_updated_time = int(time())
-            title = suite_info.get(KEY_TITLE)
-            group = suite_info.get(KEY_GROUP)
+            try:
+                title = suite_info[KEY_META].get(KEY_TITLE)
+                group = suite_info[KEY_META].get(KEY_GROUP)
+            except KeyError:
+                # Compat:<=7.5.0
+                title = suite_info.get(KEY_TITLE)
+                group = suite_info.get(KEY_GROUP)
+            # For the purpose of this method, it is OK to handle both
+            # witheld (None) and unset (empty string) together
+            if not group:
+                group = self.UNGROUPED
 
             try:
                 self.tasks_by_state[key] = suite_info[KEY_TASKS_BY_STATE]
@@ -944,7 +954,7 @@ class ScanAppUpdater(threading.Thread):
 
             tasks = sorted(self._get_warnings(key), reverse=True)
             warning_text = '\n'.join(
-                [warn[1] + '.' + warn[2] for warn in tasks[0:6]])
+                (warn[1] + '.' + warn[2] for warn in tasks[0:6]))
 
             is_stopped = KEY_PORT not in suite_info
             if KEY_STATES in suite_info:
@@ -964,7 +974,7 @@ class ScanAppUpdater(threading.Thread):
                         continue
                     self.suite_treemodel.append(
                         parent_iter, [
-                            None, None, None, None, is_stopped, title,
+                            None, None, None, None, is_stopped, None,
                             suite_updated_time, str(point),
                             states_text, warning_text])
             else:
@@ -974,13 +984,25 @@ class ScanAppUpdater(threading.Thread):
                     suite_updated_time, None, None, warning_text])
 
         self.suite_treemodel.foreach(self._expand_row, row_ids)
+        if len(hosts) > 1:
+            self.treeview.get_column(ScanApp.HOST_COLUMN).set_visible(True)
+        if len(owners) > 1:
+            self.treeview.get_column(ScanApp.OWNER_COLUMN).set_visible(True)
         return False
 
     def _update_group_counts(self):
         """Helper for self.update."""
         group_counts = {"": {'total': 0}}
         for suite_info in self.suite_info_map.values():
-            group_id = suite_info.get(KEY_GROUP)
+            try:
+                group_id = suite_info[KEY_META].get(KEY_GROUP)
+            except KeyError:
+                # Compat:<=7.5.0
+                group_id = suite_info.get(KEY_GROUP)
+            # For the purpose of this method, it is OK to handle both
+            # witheld (None) and unset (empty string) together
+            if not group_id:
+                group_id = self.UNGROUPED
 
             if group_id in group_counts:
                 group_counts[group_id]['total'] += 1
@@ -1004,6 +1026,7 @@ class ScanAppUpdater(threading.Thread):
                 states_text += '%s %d ' % (state, number)
         return states_text.rstrip()
 
-    def update_now(self, _=None):
-        """Force an update as soon as possible."""
-        self._should_force_update = True
+    def set_update_listing(self, _=None):
+        """Force an update of suite listing as soon as possible."""
+        self.prev_full_update = None
+        self.prev_norm_update = None
