@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 """Scan utilities for "cylc gscan" and "cylc gpanel"."""
 
 import os
+import re
 import signal
 from subprocess import Popen, PIPE, STDOUT
 import sys
@@ -29,15 +30,18 @@ from cylc.cfgspec.gcylc import gcfg
 import cylc.flags
 from cylc.gui.legend import ThemeLegendWindow
 from cylc.gui.util import get_icon
-from cylc.network import (
+from cylc.hostuserutil import get_user
+from cylc.network.port_scan import (
+    get_scan_items_from_fs, scan_many, DEBUG_DELIM)
+from cylc.suite_status import (
     KEY_NAME, KEY_OWNER, KEY_STATES, KEY_UPDATE_TIME)
-from cylc.network.port_scan import scan_all
 from cylc.version import CYLC_VERSION
 from cylc.wallclock import get_unix_time_from_time_string as timestr_to_seconds
 
 
 DURATION_EXPIRE_STOPPED = 600.0
 KEY_PORT = "port"
+DEBUG_DELIM = '\n' + ' ' * 4
 
 
 def get_gpanel_scan_menu(
@@ -374,7 +378,7 @@ def launch_about_dialog(program_name, hosts):
         comments_text = program_name + "\n" + hosts_text
 
     about.set_version(CYLC_VERSION)
-    about.set_copyright("Copyright (C) 2008-2017 NIWA")
+    about.set_copyright("Copyright (C) 2008-2018 NIWA")
     about.set_comments(comments_text)
     about.set_icon(get_icon())
     about.run()
@@ -409,21 +413,19 @@ def launch_hosts_dialog(existing_hosts, change_hosts_func):
     dialog.vbox.pack_start(entry, expand=False, fill=False, padding=5)
     response = dialog.run()
     if response == gtk.RESPONSE_OK:
-        new_hosts = [h.strip() for h in entry.get_text().split(",")]
-        change_hosts_func(new_hosts)
+        change_hosts_func([h.strip() for h in entry.get_text().split(",")])
     dialog.destroy()
 
 
 def get_suite_version(args):
     """Gets the suite version given the host, owner, and suite arguments"""
-    f_null = open(os.devnull, "w")
     if cylc.flags.debug:
         stderr = sys.stderr
         args = ["--debug"] + args
     else:
-        stderr = f_null
+        stderr = open(os.devnull, "w")
     command = ["cylc", "get-suite-version"] + args
-    proc = Popen(command, stdout=PIPE, stderr=stderr)
+    proc = Popen(command, stdin=open(os.devnull), stdout=PIPE, stderr=stderr)
     suite_version = proc.communicate()[0].strip()
     proc.wait()
 
@@ -436,7 +438,6 @@ def launch_gcylc(key):
     args = ["--host=" + host, "--user=" + owner, suite]
 
     # Get version of suite - now separate method get_suite_version()
-    f_null = open(os.devnull, "w")
     suite_version = get_suite_version(args)
 
     # Run correct version of "cylc gui", provided that "admin/cylc-wrapper" is
@@ -446,14 +447,15 @@ def launch_gcylc(key):
         env = dict(os.environ)
         env["CYLC_VERSION"] = suite_version
     command = ["cylc", "gui"] + args
+    stdin = open(os.devnull)
     if cylc.flags.debug:
         stdout = sys.stdout
         stderr = sys.stderr
-        Popen(command, env=env, stdout=stdout, stderr=stderr)
     else:
-        stdout = f_null
+        command = ["nohup"] + command
+        stdout = open(os.devnull, "w")
         stderr = STDOUT
-        Popen(["nohup"] + command, env=env, stdout=stdout, stderr=stderr)
+    Popen(command, env=env, stdin=stdin, stdout=stdout, stderr=stderr)
 
 
 def call_cylc_command(keys, command_id):
@@ -483,7 +485,6 @@ def call_cylc_command(keys, command_id):
         args = ["--host=" + host, "--user=" + owner, suite]
 
         # Get version of suite
-        f_null = open(os.devnull, "w")
         suite_version = get_suite_version(args)
 
         env = None
@@ -491,52 +492,81 @@ def call_cylc_command(keys, command_id):
             env = dict(os.environ)
             env["CYLC_VERSION"] = suite_version
         command = ["cylc"] + command_id.split() + args
-
+        stdin = open(os.devnull)
         if cylc.flags.debug:
             stdout = sys.stdout
             stderr = sys.stderr
-            Popen(command, env=env, stdout=stdout, stderr=stderr)
         else:
-            stdout = f_null
+            command = ["nohup"] + command
+            stdout = open(os.devnull, "w")
             stderr = stdout
-            Popen(["nohup"] + command, env=env, stdout=stdout, stderr=stderr)
+        Popen(command, env=env, stdin=stdin, stdout=stdout, stderr=stderr)
 
 
-def update_suites_info(updater):
+def update_suites_info(updater, full_mode=False):
     """Return mapping of suite info by host, owner and suite name.
 
-    updater - gscan or gpanel updater.
+    Args:
+        updater (object): gscan or gpanel updater:
+            Compulsory attributes from updater:
+                hosts: hosts to scan
+                owner_pattern: re to filter results by owners
+                suite_info_map: previous results returned by this function
+            Optional attributes from updater:
+                timeout: communication timeout
+        full_mode (boolean): update in full mode?
 
-    Return a dict of the form: {(host, owner, name): suite_info, ...}
-
-    where each "suite_info" is a dict with keys:
-        KEY_GROUP - group name of suite
-        KEY_OWNER - suite owner name
-        KEY_PORT - suite port, for running suites only
-        KEY_STATES - suite state
-        KEY_STATES:cycle - states by cycle
-        KEY_TASKS_BY_STATE - tasks by state
-        KEY_TITLE - suite title
-        KEY_UPDATE_TIME - last update time of suite
+    Return:
+        dict: {(host, owner, name): suite_info, ...}
+        where each "suite_info" is a dict with keys:
+            KEY_GROUP: group name of suite
+            KEY_META: suite metadata (new in 7.6)
+            KEY_OWNER: suite owner name
+            KEY_PORT: suite port, for running suites only
+            KEY_STATES: suite state
+            KEY_TASKS_BY_STATE: tasks by state
+            KEY_TITLE: suite title
+            KEY_UPDATE_TIME: last update time of suite
     """
     # Compulsory attributes from updater
     # hosts - hosts to scan, or the default set in the site/user global.rc
     # owner_pattern - return only suites with owners matching this compiled re
-    # prev_results - previous results returned by this function
-    hosts = updater.hosts
-    owner_pattern = updater.owner_pattern
-    prev_results = updater.suite_info_map
+    # suite_info_map - previous results returned by this function
     # Optional attributes from updater
     # timeout - communication timeout
+    owner_pattern = updater.owner_pattern
     timeout = getattr(updater, "comms_timeout", None)
     # name_pattern - return only suites with names matching this compiled re
     name_pattern = getattr(updater, "name_pattern", None)
-    # Scan
+    # Determine items to scan
     results = {}
-    for host, port, result in scan_all(
-            hosts=hosts, timeout=timeout, updater=updater):
+    items = []
+    if full_mode and updater.hosts:
+        # Scan full port range on all hosts
+        items.extend(updater.hosts)
+        if owner_pattern is None:
+            owner_pattern = re.compile(r"\A" + get_user() + r"\Z")
+    elif full_mode:
+        # Get (host, port) list from file system
+        items.extend(get_scan_items_from_fs(owner_pattern, updater))
+    else:
+        # Scan suites in previous results only
+        for (host, owner, name), prev_result in updater.suite_info_map.items():
+            port = prev_result.get(KEY_PORT)
+            if port:
+                items.append((host, port))
+            else:
+                results[(host, owner, name)] = prev_result
+    if not items:
+        return results
+    if cylc.flags.debug:
+        sys.stderr.write('Scan items:%s%s\n' % (
+            DEBUG_DELIM, DEBUG_DELIM.join(str(item) for item in items)))
+    # Scan
+    for host, port, result in scan_many(
+            items, timeout=timeout, updater=updater):
         if updater.quit:
-            break
+            return
         if (name_pattern and not name_pattern.match(result[KEY_NAME]) or
                 owner_pattern and not owner_pattern.match(result[KEY_OWNER])):
             continue
@@ -547,25 +577,25 @@ def update_suites_info(updater):
         except (KeyError, TypeError, ValueError):
             pass
     expire_threshold = time() - DURATION_EXPIRE_STOPPED
-    for (host, owner, name), prev_result in prev_results.items():
+    for (host, owner, name), prev_result in updater.suite_info_map.items():
         if updater.quit:
-            break
+            return
         if ((host, owner, name) in results or
-                host not in hosts or
                 owner_pattern and not owner_pattern.match(owner) or
                 name_pattern and not name_pattern.match(name)):
             # OK if suite already in current results set.
             # Don't bother if:
-            # * previous host not in current hosts list
             # * previous owner does not match current owner pattern
             # * previous suite name does not match current name pattern
             continue
         if prev_result.get(KEY_PORT):
             # A previously running suite is no longer running.
-            # Get suite info with "cat-state", if possible, and include in the
-            # results set.
+            # Get suite info with "ls-checkpoints", if possible, and include in
+            # the results set.
             try:
-                prev_result = _update_stopped_suite_info((host, owner, name))
+                prev_result.update(
+                    _update_stopped_suite_info((host, owner, name)))
+                del prev_result[KEY_PORT]
             except (IndexError, TypeError, ValueError):
                 continue
         if prev_result.get(KEY_UPDATE_TIME, 0) > expire_threshold:
@@ -574,7 +604,7 @@ def update_suites_info(updater):
 
 
 def _update_stopped_suite_info(key):
-    """Return a map like cylc scan --raw for states and last update time."""
+    """Use "cylc ls-checkpoints" to obtain info of stopped suite."""
     host, owner, suite = key
     cmd = ["cylc", "ls-checkpoints"]
     if host:
@@ -589,7 +619,9 @@ def _update_stopped_suite_info(key):
     cmd += [suite, "0"]  # checkpoint 0 is latest checkpoint
     result = {}
     try:
-        proc = Popen(cmd, stderr=stderr, stdout=PIPE, preexec_fn=os.setpgrp)
+        proc = Popen(
+            cmd, stdin=open(os.devnull), stderr=stderr, stdout=PIPE,
+            preexec_fn=os.setpgrp)
     except OSError:
         return result
     else:
