@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -56,7 +56,10 @@ class Tailer(threading.Thread):
         self.filename = filename
         self.cmd_tmpl = cmd_tmpl
         self.pollable = pollable
-        self.filters = filters
+        if filters:
+            self.filters = [re.compile(f) for f in filters]
+        else:
+            self.filters = None
 
         self.logbuffer = logview.get_buffer()
         self.quit = False
@@ -73,6 +76,9 @@ class Tailer(threading.Thread):
     def run(self):
         """Invoke the tailer."""
         command = []
+        cmd_tmpl = None
+        if self.cmd_tmpl:
+            cmd_tmpl = self.cmd_tmpl
         if ":" in self.filename:  # remote
             user_at_host, filename = self.filename.split(':')
             if "@" in user_at_host:
@@ -81,23 +87,25 @@ class Tailer(threading.Thread):
                 owner, host = (None, user_at_host)
             ssh = str(GLOBAL_CFG.get_host_item("ssh command", host, owner))
             command = shlex.split(ssh) + ["-n", user_at_host]
-            cmd_tmpl = str(GLOBAL_CFG.get_host_item(
-                "remote tail command template", host, owner))
+            if not cmd_tmpl:
+                cmd_tmpl = str(GLOBAL_CFG.get_host_item(
+                    "remote tail command template", host, owner))
+            command.append(cmd_tmpl % {"filename": filename})
         else:
             filename = self.filename
-            cmd_tmpl = str(GLOBAL_CFG.get_host_item(
-                "local tail command template"))
+            if not cmd_tmpl:
+                cmd_tmpl = str(GLOBAL_CFG.get_host_item(
+                    "local tail command template"))
+            command += shlex.split(cmd_tmpl % {"filename": filename})
 
-        if self.cmd_tmpl:
-            cmd_tmpl = self.cmd_tmpl
-        command += shlex.split(cmd_tmpl % {"filename": filename})
         try:
             self.proc = Popen(
-                command, stdout=PIPE, stderr=STDOUT, preexec_fn=os.setpgrp)
+                command, stdin=open(os.devnull), stdout=PIPE, stderr=STDOUT,
+                preexec_fn=os.setpgrp)
         except OSError as exc:
             # E.g. ssh command not found
             dialog = warning_dialog("%s: %s" % (
-                exc, " ".join([quote(item) for item in command])))
+                exc, " ".join(quote(item) for item in command)))
             gobject.idle_add(dialog.warn)
             return
         poller = select.poll()
@@ -119,7 +127,7 @@ class Tailer(threading.Thread):
                 data = os.read(self.proc.stdout.fileno(), self.READ_SIZE)
             except (IOError, OSError) as exc:
                 dialog = warning_dialog("%s: %s" % (
-                    exc, " ".join([quote(item) for item in command])))
+                    exc, " ".join(quote(item) for item in command)))
                 gobject.idle_add(dialog.warn)
                 break
             if data:
@@ -133,7 +141,7 @@ class Tailer(threading.Thread):
                         line = buf + line
                         buf = ""
                     if (not self.filters or
-                            all([re.search(f, line) for f in self.filters])):
+                            all(f.search(line) for f in self.filters)):
                         gobject.idle_add(self.update_gui, line)
             sleep(0.01)
         self.stop()
