@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,20 +16,20 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from copy import deepcopy
 import gobject
 import gtk
 import threading
 from time import sleep
 
-from cylc.task_id import TaskID
-from cylc.gui.dot_maker import DotMaker
-from cylc.network.suite_state_client import get_id_summary
-from copy import deepcopy
-
 import warnings
 warnings.filterwarnings('ignore', '^.*was not found when attempting to ' +
                         'remove it', Warning)
 
+from cylc.task_id import TaskID
+from cylc.gui.dot_maker import DotMaker
+from cylc.gui.util import get_id_summary
+
 
 class DotUpdater(threading.Thread):
 
@@ -84,20 +84,23 @@ class DotUpdater(threading.Thread):
         dotm = DotMaker(theme, size=dot_size)
         self.dots = dotm.get_dots()
 
-    def _set_tooltip(self, widget, tip_text):
+    @staticmethod
+    def _set_tooltip(widget, tip_text):
         tip = gtk.Tooltips()
         tip.enable()
         tip.set_tip(widget, tip_text)
 
-    def clear_list(self):
+    def clear_gui(self):
+        """Clear the LED tree store."""
         self.led_treestore.clear()
         # gtk idle functions must return false or will be called multiple times
         return False
 
     def update(self):
+        """Update data using data from self.updater."""
         if not self.updater.connected:
             if not self.cleared:
-                gobject.idle_add(self.clear_list)
+                gobject.idle_add(self.clear_gui)
                 self.cleared = True
             return False
         self.cleared = False
@@ -108,14 +111,14 @@ class DotUpdater(threading.Thread):
             return False
 
         self.last_update_time = self.updater.last_update_time
-        self.updater.set_update(False)
+        self.updater.no_update_event.set()
 
         self.state_summary = deepcopy(self.updater.state_summary)
         self.fam_state_summary = deepcopy(self.updater.fam_state_summary)
         self.ancestors_pruned = deepcopy(self.updater.ancestors_pruned)
         self.descendants = deepcopy(self.updater.descendants)
 
-        self.updater.set_update(True)
+        self.updater.no_update_event.clear()
 
         self.point_strings = []
         for id_ in self.state_summary:
@@ -161,7 +164,7 @@ class DotUpdater(threading.Thread):
 
             for heading, tasks in self.family_tree.iteritems():
                 # Place associated tasks after headers.
-                ind = min([self.task_list.index(task) for task in tasks])
+                ind = min(self.task_list.index(task) for task in tasks)
                 for task in tasks:
                     if task in self.task_list:
                         self.task_list.remove(task)
@@ -179,10 +182,10 @@ class DotUpdater(threading.Thread):
         self.led_headings = new_headings
 
         # Get a list of tasks belonging to families.
-        if len(self.family_tree) > 0:
+        if self.family_tree:
             all_sub_tasks = reduce(
                 lambda x, y: x + y,
-                [self.family_tree[key] for key in self.family_tree]
+                self.family_tree.values()
             )
         else:
             all_sub_tasks = []
@@ -470,8 +473,9 @@ class DotUpdater(threading.Thread):
         if col_index == 0:
             tooltip.set_text(task_id)
             return True
-        text = get_id_summary(task_id, self.state_summary,
-                              self.fam_state_summary, self.descendants)
+        text = get_id_summary(
+            task_id, self.state_summary, self.fam_state_summary,
+            self.descendants)
         if text == task_id:
             return False
         tooltip.set_text(text)
@@ -596,5 +600,3 @@ class DotUpdater(threading.Thread):
             if self.update() or self.action_required:
                 gobject.idle_add(self.update_gui)
             sleep(0.2)
-        else:
-            pass
