@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,19 +16,22 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from cylc.graphing import CGraphPlain
-from cylc.mkdir_p import mkdir_p
-from cylc.network.suite_state_client import get_id_summary
-from cylc.task_id import TaskID
-from cylc.cfgspec.globalcfg import GLOBAL_CFG
-from cylc.gui.warning_dialog import warning_dialog
 from copy import deepcopy
 import gobject
 import os
 import re
-import sys
 import threading
 from time import sleep
+import traceback
+
+from cylc.cfgspec.globalcfg import GLOBAL_CFG
+import cylc.flags
+from cylc.graphing import CGraphPlain
+from cylc.gui.warning_dialog import warning_dialog
+from cylc.gui.util import get_id_summary
+from cylc.mkdir_p import mkdir_p
+from cylc.network.httpclient import ClientError
+from cylc.task_id import TaskID
 from cylc.task_state import TASK_STATUS_RUNAHEAD
 
 
@@ -56,6 +59,10 @@ def compare_dict_of_dict(one, two):
 
 
 class GraphUpdater(threading.Thread):
+    """Thread for updating "cylc gui" graph view."""
+
+    PREFIX_BASE = 'base:'
+
     def __init__(self, cfg, updater, theme, info_bar, xdot):
         super(GraphUpdater, self).__init__()
 
@@ -132,7 +139,8 @@ class GraphUpdater(threading.Thread):
                 ).warn)
                 self.write_dot_frames = False
 
-    def clear_graph(self):
+    def clear_gui(self):
+        """Clear the graph GUI."""
         self.prev_graph_id = ()
         self.graphw = CGraphPlain(self.cfg.suite)
         self.normal_fit = True
@@ -141,13 +149,16 @@ class GraphUpdater(threading.Thread):
         return False
 
     def get_summary(self, task_id):
-        return get_id_summary(task_id, self.state_summary,
-                              self.fam_state_summary, self.descendants)
+        """Return some state information about a task or family."""
+        return get_id_summary(
+            task_id, self.state_summary, self.fam_state_summary,
+            self.descendants)
 
     def update(self):
+        """Update data using data from self.updater."""
         if not self.updater.connected:
             if not self.cleared:
-                gobject.idle_add(self.clear_graph)
+                gobject.idle_add(self.clear_gui)
                 self.cleared = True
             return False
         self.cleared = False
@@ -158,14 +169,14 @@ class GraphUpdater(threading.Thread):
                 return True
             return False
 
-        self.updater.set_update(False)
+        self.updater.no_update_event.set()
         states_full = deepcopy(self.updater.state_summary)
         fam_states_full = deepcopy(self.updater.fam_state_summary)
         self.ancestors = deepcopy(self.updater.ancestors)
         self.descendants = deepcopy(self.updater.descendants)
         self.all_families = deepcopy(self.updater.all_families)
         self.global_summary = deepcopy(self.updater.global_summary)
-        self.updater.set_update(True)
+        self.updater.no_update_event.clear()
 
         self.first_update = (self.last_update_time is None)
         self.last_update_time = self.updater.last_update_time
@@ -218,18 +229,11 @@ class GraphUpdater(threading.Thread):
     def run(self):
         while not self.quit:
             if self.update():
-                if self.global_summary:
-                    needed_no_redraw = self.update_graph()
-                # DO NOT USE gobject.idle_add() HERE - IT DRASTICALLY
-                # AFFECTS PERFORMANCE FOR LARGE SUITES? appears to
-                # be unnecessary anyway (due to xdot internals?)
-                    self.update_xdot(no_zoom=needed_no_redraw)
+                self.update_gui()
             sleep(0.2)
-        else:
-            pass
 
     def update_xdot(self, no_zoom=False):
-        self.xdot.set_dotcode(self.graphw.to_string(), no_zoom=True)
+        self.xdot.set_dotcode(self.graphw.to_string(), no_zoom=no_zoom)
         if self.first_update:
             self.xdot.widget.zoom_to_fit()
             self.first_update = False
@@ -240,13 +244,13 @@ class GraphUpdater(threading.Thread):
             self.xdot.widget.zoom_image(1.0, center=True)
             self.normal_fit = False
 
-    def set_live_node_attr(self, node, id, shape=None):
+    def set_live_node_attr(self, node, id_, shape=None):
         # override base graph URL to distinguish live tasks
-        node.attr['URL'] = id
-        if id in self.state_summary:
-            state = self.state_summary[id]['state']
+        node.attr['URL'] = id_
+        if id_ in self.state_summary:
+            state = self.state_summary[id_]['state']
         else:
-            state = self.fam_state_summary[id]['state']
+            state = self.fam_state_summary[id_]['state']
 
         try:
             node.attr['style'] = 'bold,' + self.theme[state]['style']
@@ -262,11 +266,13 @@ class GraphUpdater(threading.Thread):
         if shape:
             node.attr['shape'] = shape
 
-    def update_graph(self):
+    def update_gui(self):
         # TODO - check edges against resolved ones
         # (adding new ones, and nodes, if necessary)
-
         self.action_required = False
+        if not self.global_summary:
+            return
+
         self.oldest_point_string = (
             self.global_summary['oldest cycle point string'])
         self.newest_point_string = (
@@ -293,7 +299,7 @@ class GraphUpdater(threading.Thread):
             ungroup_for_server = None
 
         try:
-            res = self.updater.suite_info_client.get_info(
+            res = self.updater.client.get_info(
                 'get_graph_raw', start_point_string=oldest,
                 stop_point_string=newest,
                 group_nodes=group_for_server,
@@ -302,8 +308,13 @@ class GraphUpdater(threading.Thread):
                 group_all=self.group_all,
                 ungroup_all=self.ungroup_all
             )
-        except Exception as exc:
-            print >> sys.stderr, str(exc)
+        except ClientError:
+            if cylc.flags.debug:
+                try:
+                    traceback.print_exc()
+                except IOError:
+                    pass  # Cannot print to terminal (session may be closed).
+
             return False
 
         self.have_leaves_and_feet = True
@@ -311,9 +322,7 @@ class GraphUpdater(threading.Thread):
         gr_edges = [tuple(edge) for edge in gr_edges]
 
         current_id = self.get_graph_id(gr_edges)
-        needs_redraw = current_id != self.prev_graph_id
-
-        if needs_redraw:
+        if current_id != self.prev_graph_id:
             self.graphw = CGraphPlain(
                 self.cfg.suite, suite_polling_tasks)
             self.graphw.add_edges(
@@ -325,14 +334,14 @@ class GraphUpdater(threading.Thread):
             if (self.updater.filter_name_string or
                     self.updater.filter_states_excl):
                 for node in self.graphw.nodes():
-                    id = node.get_name()
+                    id_ = node.get_name()
                     # Don't need to guard against special nodes here (yet).
-                    name, point_string = TaskID.split(id)
+                    name, point_string = TaskID.split(id_)
                     if name not in self.all_families:
                         # This node is a task, not a family.
-                        if id in self.updater.filt_task_ids:
+                        if id_ in self.updater.filt_task_ids:
                             nodes_to_remove.add(node)
-                        elif id not in self.updater.kept_task_ids:
+                        elif id_ not in self.updater.kept_task_ids:
                             # A base node - these only appear in the graph.
                             filter_string = self.updater.filter_name_string
                             if (filter_string and
@@ -340,7 +349,7 @@ class GraphUpdater(threading.Thread):
                                     not re.search(filter_string, name)):
                                 # A base node that fails the name filter.
                                 nodes_to_remove.add(node)
-                    elif id in self.fam_state_summary:
+                    elif id_ in self.fam_state_summary:
                         # Remove family nodes if all members filtered out.
                         remove = True
                         for mem in self.descendants[name]:
@@ -350,7 +359,7 @@ class GraphUpdater(threading.Thread):
                                 break
                         if remove:
                             nodes_to_remove.add(node)
-                    elif id in self.updater.full_fam_state_summary:
+                    elif id_ in self.updater.full_fam_state_summary:
                         # An updater-filtered-out family.
                         nodes_to_remove.add(node)
 
@@ -401,20 +410,23 @@ class GraphUpdater(threading.Thread):
             node.attr['color'] = '#888888'
             node.attr['fillcolor'] = 'white'
             node.attr['fontcolor'] = '#888888'
+            if not node.attr['URL'].startswith(self.PREFIX_BASE):
+                node.attr['URL'] = self.PREFIX_BASE + node.attr['URL']
 
-        for id in self.state_summary:
+        for id_ in self.state_summary:
             try:
-                node = self.graphw.get_node(id)
+                node = self.graphw.get_node(id_)
             except KeyError:
                 continue
-            self.set_live_node_attr(node, id)
+            self.set_live_node_attr(node, id_)
 
-        for id in self.fam_state_summary:
+        for id_ in self.fam_state_summary:
             try:
-                node = self.graphw.get_node(id)
-            except:
+                node = self.graphw.get_node(id_)
+            except KeyError:
+                # Node not in graph.
                 continue
-            self.set_live_node_attr(node, id)
+            self.set_live_node_attr(node, id_)
 
         self.graphw.graph_attr['rankdir'] = self.orientation
 
@@ -425,8 +437,8 @@ class GraphUpdater(threading.Thread):
             self.graphw.write(arg)
             self.graph_frame_count += 1
 
+        self.update_xdot(no_zoom=(current_id == self.prev_graph_id))
         self.prev_graph_id = current_id
-        return not needs_redraw
 
     def get_graph_id(self, edges):
         """If any of these quantities change, the graph should be redrawn."""
