@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,23 +17,16 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from copy import deepcopy
-import time
 import gobject
 import itertools
 import threading
-from time import sleep
+from time import time, sleep
 
-from cylc.task_id import TaskID
 from cylc.gui.dot_maker import DotMaker
-from cylc.network.suite_state_client import get_id_summary
-from cylc.wallclock import get_time_string_from_unix_time
+from cylc.gui.util import get_id_summary
+from cylc.task_id import TaskID
 from cylc.task_state import TASK_STATUSES_AUTO_EXPAND
-
-
-def _time_trim(time_value):
-    if time_value is not None:
-        return time_value.rsplit(".", 1)[0]
-    return time_value
+from cylc.wallclock import get_time_string_from_unix_time
 
 
 class TreeUpdater(threading.Thread):
@@ -57,6 +50,7 @@ class TreeUpdater(threading.Thread):
         self.ancestors = {}
         self.descendants = []
         self.fam_state_summary = {}
+        self.state_summary = {}
         self._prev_id_named_paths = {}
         self._prev_data = {}
         self._prev_fam_data = {}
@@ -85,15 +79,17 @@ class TreeUpdater(threading.Thread):
         dotm = DotMaker(theme, size=dot_size)
         self.dots = dotm.get_dots()
 
-    def clear_tree(self):
+    def clear_gui(self):
+        """Clear the tree view tree store."""
         self.ttreestore.clear()
         # gtk idle functions must return false or will be called multiple times
         return False
 
     def update(self):
+        """Update data using data from self.updater."""
         if not self.updater.connected:
             if not self.cleared:
-                gobject.idle_add(self.clear_tree)
+                gobject.idle_add(self.clear_gui)
                 self.cleared = True
             return False
         self.cleared = False
@@ -105,35 +101,37 @@ class TreeUpdater(threading.Thread):
 
         self.last_update_time = self.updater.last_update_time
 
-        self.updater.set_update(False)
+        self.updater.no_update_event.set()
         self.state_summary = deepcopy(self.updater.state_summary)
         self.fam_state_summary = deepcopy(self.updater.fam_state_summary)
         self.ancestors = deepcopy(self.updater.ancestors)
         self.descendants = deepcopy(self.updater.descendants)
-        self.updater.set_update(True)
+        self.updater.no_update_event.clear()
         return True
 
-    def search_level(self, model, iter, func, data):
-        while iter:
-            if func(model, iter, data):
-                return iter
-            iter = model.iter_next(iter)
+    @staticmethod
+    def search_level(model, iter_, func, data):
+        while iter_:
+            if func(model, iter_, data):
+                return iter_
+            iter_ = model.iter_next(iter_)
         return None
 
-    def search_treemodel(self, model, iter, func, data):
-        while iter:
-            if func(model, iter, data):
-                return iter
+    def search_treemodel(self, model, iter_, func, data):
+        while iter_:
+            if func(model, iter_, data):
+                return iter_
             result = self.search_treemodel(
-                model, model.iter_children(iter), func, data)
+                model, model.iter_children(iter_), func, data)
             if result:
                 return result
-            iter = model.iter_next(iter)
+            iter_ = model.iter_next(iter_)
         return None
 
-    def match_func(self, model, iter, data):
+    @staticmethod
+    def match_func(model, iter_, data):
         column, key = data
-        value = model.get_value(iter, column)
+        value = model.get_value(iter_, column)
         return value == key
 
     def on_query_tooltip(self, widget, x, y, kbd_ctx, tooltip):
@@ -143,7 +141,7 @@ class TreeUpdater(threading.Thread):
             self._prev_tooltip_task_id = None
             return False
         x, y = self.ttreeview.convert_widget_to_bin_window_coords(x, y)
-        path, column, cell_x, cell_y = self.ttreeview.get_path_at_pos(x, y)
+        path = self.ttreeview.get_path_at_pos(x, y)[0]
         if not path:
             return False
         model = self.ttreeview.get_model()
@@ -160,8 +158,9 @@ class TreeUpdater(threading.Thread):
             self._prev_tooltip_task_id = task_id
             tooltip.set_text(None)
             return False
-        text = get_id_summary(task_id, self.state_summary,
-                              self.fam_state_summary, self.descendants)
+        text = get_id_summary(
+            task_id, self.state_summary, self.fam_state_summary,
+            self.descendants)
         if text == task_id:
             return False
         tooltip.set_text(text)
@@ -186,8 +185,11 @@ class TreeUpdater(threading.Thread):
         # Retrieve any user-expanded rows so that we can expand them later.
         # This is only really necessary for edge cases in tree reconstruction.
         expand_me = self._get_user_expanded_row_ids()
-        daemon_time_zone_info = self.updater.global_summary.get(
-            "daemon time zone info")
+        try:
+            time_zone_info = self.updater.global_summary.get("time zone info")
+        except KeyError:  # Back compat <= 7.5.0
+            time_zone_info = self.updater.global_summary.get(
+                "daemon time zone info")
 
         # Store the state, times, messages, etc for tasks and families.
         new_data = {}
@@ -217,11 +219,11 @@ class TreeUpdater(threading.Thread):
                 (self.updater.fam_state_summary, new_fam_data,
                  self._prev_fam_data, True)]:
             # Populate new_data and new_fam_data.
-            for id in summary:
-                name, point_string = TaskID.split(id)
+            for id_ in summary:
+                name, point_string = TaskID.split(id_)
                 if point_string not in dest:
                     dest[point_string] = {}
-                state = summary[id].get('state')
+                state = summary[id_].get('state')
 
                 # Populate task timing slots.
                 t_info = {}
@@ -235,26 +237,29 @@ class TreeUpdater(threading.Thread):
                         t_info['mean_elapsed_time_string'] = ""
                         t_info['progress'] = 0
                 else:
-                    meant = summary[id].get('mean_elapsed_time')
-                    tstart = summary[id].get('started_time')
+                    meant = summary[id_].get('mean_elapsed_time')
+                    tstart = summary[id_].get('started_time')
                     tetc_string = None
 
                     for dt in tkeys:
-                        t_info[dt] = summary[id][dt]
+                        t_info[dt] = summary[id_][dt]
 
                     # Compute percent progress.
+                    t_info['progress'] = 0
                     if (isinstance(tstart, float) and (
                             isinstance(meant, float) or
                             isinstance(meant, int))):
                         tetc_unix = tstart + meant
-                        tnow = time.time()
-                        if tnow > tetc_unix:
+                        tnow = time()
+                        if tstart > tnow:
+                            # Reportably possible via interraction with
+                            # cylc reset.
+                            t_info['progress'] = 0
+                        elif tnow > tetc_unix:
                             t_info['progress'] = 100
-                        else:
+                        elif meant != 0:
                             t_info['progress'] = int(
-                                100 * (tnow - tstart) / (tetc_unix - tstart))
-                    else:
-                        t_info['progress'] = 0
+                                100 * (tnow - tstart) / (meant))
 
                     if (t_info['finished_time_string'] is None and
                             isinstance(tstart, float) and
@@ -263,13 +268,13 @@ class TreeUpdater(threading.Thread):
                         # Task not finished, but has started and has a meant;
                         # so we can compute an expected time of completion.
                         tetc_string = (
-                            self._id_tetc_cache.get(id, {}).get(tetc_unix))
+                            self._id_tetc_cache.get(id_, {}).get(tetc_unix))
                         if tetc_string is None:
                             # We have to calculate it.
                             tetc_string = get_time_string_from_unix_time(
                                 tetc_unix,
-                                custom_time_zone_info=daemon_time_zone_info)
-                            self._id_tetc_cache[id] = {tetc_unix: tetc_string}
+                                custom_time_zone_info=time_zone_info)
+                            self._id_tetc_cache[id_] = {tetc_unix: tetc_string}
                         t_info['finished_time_string'] = tetc_string
                         estimated_t_finish = True
                     else:
@@ -302,15 +307,15 @@ class TreeUpdater(threading.Thread):
                             t_info['finished_time_string'])
 
                 # Use "*" (or "" for family rows) until slot is populated.
-                job_id = summary[id].get('submit_method_id')
-                batch_sys_name = summary[id].get('batch_sys_name')
-                host = summary[id].get('host')
-                message = summary[id].get('latest_message')
+                job_id = summary[id_].get('submit_method_id')
+                batch_sys_name = summary[id_].get('batch_sys_name')
+                host = summary[id_].get('host')
+                message = summary[id_].get('latest_message')
                 if message is not None:
                     if last_update_date is not None:
                         message = message.replace(
                             last_update_date + "T", "", 1)
-                    submit_num = summary[id].get('submit_num')
+                    submit_num = summary[id_].get('submit_num')
                     if submit_num:
                         message = "job(%02d) " % submit_num + message
                 if is_fam:
@@ -374,9 +379,9 @@ class TreeUpdater(threading.Thread):
             # Some previous task ids need deleting, so rebuild the tree.
             should_rebuild_tree = True
 
-        for id in tetc_cached_ids_left:
+        for id_ in tetc_cached_ids_left:
             # These ids were not present in the summary - so clear them.
-            self._id_tetc_cache.pop(id)
+            self._id_tetc_cache.pop(id_)
 
         # Cache the current row point-string and names.
         row_id_iters_left = {}
@@ -396,7 +401,7 @@ class TreeUpdater(threading.Thread):
                 try:
                     p_data = new_fam_data[point_string]["root"]
                 except KeyError:
-                    p_data = [None] * 7
+                    p_data = [None] * 11
                 p_path = (i,)
                 p_row_id = (point_string, point_string)
                 p_data = list(p_row_id) + p_data
@@ -504,7 +509,7 @@ class TreeUpdater(threading.Thread):
                         data = new_fam_data[point_string][name]
                     else:
                         data = new_data[point_string][name]
-                    iter_, path = row_id_iters_left[(point_string, name)]
+                    iter_ = row_id_iters_left[(point_string, name)][0]
                 except KeyError:
                     if not is_fam:
                         raise
@@ -593,7 +598,8 @@ class TreeUpdater(threading.Thread):
         iter_ = model.append(parent_iter, data)
         return iter_
 
-    def _get_row_id(self, model, rpath):
+    @staticmethod
+    def _get_row_id(model, rpath):
         """Record a row's first two values."""
         riter = model.get_iter(rpath)
         point_string = model.get_value(riter, 0)
@@ -688,5 +694,3 @@ class TreeUpdater(threading.Thread):
             if self.update():
                 gobject.idle_add(self.update_gui)
             sleep(0.2)
-        else:
-            pass
