@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,12 +16,15 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import glib
 import os
 import sys
 import traceback
 
 import gtk
 
+from cylc.task_id import TaskID
+
 
 class EntryTempText(gtk.Entry):
     """Class to add temporary text to an entry that vanishes on focus."""
@@ -104,16 +107,70 @@ def get_image_dir():
 
 def get_icon():
     """Return the gcylc icon as a gtk.gdk.Pixbuf."""
+    icon_path = os.path.join(get_image_dir(), "icon.svg")
     try:
-        icon_path = os.path.join(get_image_dir(), "icon.svg")
         icon = gtk.gdk.pixbuf_new_from_file(icon_path)
-    except:
+    except glib.GError:
         # SVG error? Try loading it the old way.
         icon_path = os.path.join(get_image_dir(), "icon.png")
         icon = gtk.gdk.pixbuf_new_from_file(icon_path)
     return icon
 
 
+def get_id_summary(id_, task_state_summary, fam_state_summary, id_family_map):
+    """Return some state information about a task or family id."""
+    prefix_text = ""
+    meta_text = ""
+    sub_text = ""
+    sub_states = {}
+    stack = [(id_, 0)]
+    done_ids = []
+    for summary in [task_state_summary, fam_state_summary]:
+        if id_ in summary:
+            title = summary[id_].get('title')
+            if title:
+                meta_text += "\n" + title.strip()
+            description = summary[id_].get('description')
+            if description:
+                meta_text += "\n" + description.strip()
+    while stack:
+        this_id, depth = stack.pop(0)
+        if this_id in done_ids:  # family dive down will give duplicates
+            continue
+        done_ids.append(this_id)
+        prefix = "\n" + " " * 4 * depth + this_id
+        if this_id in task_state_summary:
+            submit_num = task_state_summary[this_id].get('submit_num')
+            if submit_num:
+                prefix += "(%02d)" % submit_num
+            state = task_state_summary[this_id]['state']
+            sub_text += prefix + " " + state
+            sub_states.setdefault(state, 0)
+            sub_states[state] += 1
+        elif this_id in fam_state_summary:
+            name, point_string = TaskID.split(this_id)
+            sub_text += prefix + " " + fam_state_summary[this_id]['state']
+            for child in reversed(sorted(id_family_map[name])):
+                child_id = TaskID.get(child, point_string)
+                stack.insert(0, (child_id, depth + 1))
+        if not prefix_text:
+            prefix_text = sub_text.strip()
+            sub_text = ""
+    if len(sub_text.splitlines()) > 10:
+        state_items = sub_states.items()
+        state_items.sort()
+        state_items.sort(lambda x, y: cmp(y[1], x[1]))
+        sub_text = ""
+        for state, number in state_items:
+            sub_text += "\n    {0} tasks {1}".format(number, state)
+    if sub_text and meta_text:
+        sub_text = "\n" + sub_text
+    text = prefix_text + meta_text + sub_text
+    if not text:
+        return id_
+    return text
+
+
 def get_logo():
     """Return the gcylc logo as a gtk.gdk.Pixbuf."""
     logo_path = os.path.join(get_image_dir(), "logo.png")
@@ -121,7 +178,7 @@ def get_logo():
 
 
 def _launch_exception_hook_dialog(e_type, e_value, e_traceback,
-                                  old_hook=None, program_name=None):
+                                  old_hook, program_name):
     if program_name is None:
         program_name = "This program"
     exc_lines = traceback.format_exception(e_type, e_value, e_traceback)
@@ -140,8 +197,9 @@ def _launch_exception_hook_dialog(e_type, e_value, e_traceback,
 def set_exception_hook_dialog(program_name=None):
     """Set a custom uncaught exception hook for launching an error dialog."""
     old_hook = sys.excepthook
-    sys.excepthook = lambda *a: _launch_exception_hook_dialog(
-        *a, old_hook=old_hook, program_name=program_name)
+    sys.excepthook = lambda e_type, e_value, e_traceback: (
+        _launch_exception_hook_dialog(
+            e_type, e_value, e_traceback, old_hook, program_name))
 
 
 def setup_icons():
