@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,11 +16,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import re
 import gtk
 import gobject
-from updater_graph import GraphUpdater
 
+from cylc.gui.updater_graph import GraphUpdater
 from cylc.cylc_xdot import xdot_widgets
 from cylc.task_id import TaskID
 
@@ -46,6 +45,9 @@ Dependency graph suite control interface.
         self.log_colors = log_colors
         self.insert_task_popup = insert_task_popup
 
+        self.t = None
+        self.menu_subgraphs_item = None
+
         self.gcapture_windows = []
 
         self.xdot = xdot_widgets()
@@ -74,19 +76,6 @@ Dependency graph suite control interface.
     def graph_update(self, w):
         self.t.action_required = True
 
-    def on_url_clicked(self, widget, url, event):
-        if event.button != 3:
-            return False
-
-        m = re.match('base:(.*)', url)
-        if m:
-            # base graph node
-            task_id = m.groups()[0]
-            self.right_click_menu(event, task_id, type='base graph task')
-            return
-
-        self.right_click_menu(event, url, type='live task')
-
     def on_motion_notify(self, widget, event):
         """Add a new tooltip when the cursor moves in the graph."""
         url = self.xdot.widget.get_url(event.x, event.y)
@@ -100,21 +89,26 @@ Dependency graph suite control interface.
         if url is None:
             self.xdot.widget.set_tooltip_text(None)
             return False
-        url = unicode(url.url)
-        m = re.match('base:(.*)', url)
-        if m:
-            task_id = m.groups()[0]
-            self.xdot.widget.set_tooltip_text(self.t.get_summary(task_id))
-            return False
-
-        # URL is task ID
-        self.xdot.widget.set_tooltip_text(self.t.get_summary(url))
+        task_id = unicode(url.url)
+        if task_id.startswith(self.t.PREFIX_BASE):
+            # base graph node
+            task_id = task_id[len(self.t.PREFIX_BASE):]
+        self.xdot.widget.set_tooltip_text(self.t.get_summary(task_id))
         return False
 
     def stop(self):
         self.t.quit = True
 
-    def right_click_menu(self, event, task_id, type='live task'):
+    def on_url_clicked(self, _, task_id, event):
+        """Callback on clicking the right hand mouse button."""
+        if event.button != 3:
+            return False
+
+        is_base = task_id.startswith(self.t.PREFIX_BASE)
+        if is_base:
+            # base graph node
+            task_id = task_id[len(self.t.PREFIX_BASE):]
+
         name, point_string = TaskID.split(task_id)
 
         menu = gtk.Menu()
@@ -155,9 +149,7 @@ Dependency graph suite control interface.
                                        name not in self.t.leaves)
         ungroup_rec_item.connect('activate', self.grouping, name, False, True)
 
-        menu.append(gtk.SeparatorMenuItem())
-
-        if type is not 'live task':
+        if is_base:
             insert_item = gtk.ImageMenuItem('Insert ...')
             img = gtk.image_new_from_stock(gtk.STOCK_DIALOG_INFO,
                                            gtk.ICON_SIZE_MENU)
@@ -170,7 +162,7 @@ Dependency graph suite control interface.
                     name=name, point_string=point_string
                 )
             )
-            menu.append(gtk.SeparatorMenuItem())
+        menu.append(gtk.SeparatorMenuItem())
 
         menu.append(timezoom_item_direct)
         menu.append(timezoom_reset_item)
@@ -180,7 +172,7 @@ Dependency graph suite control interface.
         menu.append(ungroup_item)
         menu.append(ungroup_rec_item)
 
-        if type == 'live task':
+        if not is_base:
             is_fam = (name in self.t.descendants)
             if is_fam:
                 if task_id not in self.t.fam_state_summary:
@@ -226,20 +218,6 @@ Dependency graph suite control interface.
         self.t.action_required = True
         self.t.best_fit = True
 
-    def rearrange(self, col, n):
-        cols = self.ttreeview.get_columns()
-        for i_n in range(len(cols)):
-            if i_n == n:
-                cols[i_n].set_sort_indicator(True)
-            else:
-                cols[i_n].set_sort_indicator(False)
-        # col is cols[n]
-        if col.get_sort_order() == gtk.SORT_ASCENDING:
-            col.set_sort_order(gtk.SORT_DESCENDING)
-        else:
-            col.set_sort_order(gtk.SORT_ASCENDING)
-        self.ttreestore.set_sort_column_id(n, col.get_sort_order())
-
     def refresh(self):
         self.t.update()
         self.t.best_fit = True
@@ -295,7 +273,8 @@ Dependency graph suite control interface.
 
         return items
 
-    def _set_tooltip(self, widget, tip_text):
+    @staticmethod
+    def _set_tooltip(widget, tip_text):
         tip = gtk.Tooltips()
         tip.enable()
         tip.set_tip(widget, tip_text)
@@ -567,28 +546,3 @@ Dependency graph suite control interface.
 #        return
 #        self.t.best_fit = True
 #        self.t.action_required = True
-
-
-class StandaloneControlGraphApp(ControlGraph):
-    # For a ControlApp not launched by the gcylc main app:
-    # 1/ call gobject.threads_init() on startup
-    # 2/ call gtk.main_quit() on exit
-
-    def __init__(self, suite, owner, host, port):
-        gobject.threads_init()
-        ControlGraph.__init__(self, suite, owner, host, port)
-
-    def quit_gcapture(self):
-        for gwindow in self.gcapture_windows:
-            if not gwindow.quit_already:
-                gwindow.quit(None, None)
-
-    def delete_event(self, widget, event, data=None):
-        self.quit_gcapture()
-        ControlGraph.delete_event(self, widget, event, data)
-        gtk.main_quit()
-
-    def click_exit(self, foo):
-        self.quit_gcapture()
-        ControlGraph.click_exit(self, foo)
-        gtk.main_quit()
