@@ -1,7 +1,7 @@
 #!/bin/sh
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,8 +39,7 @@ cylc__job__main() {
     typeset file_name=
     for file_name in \
         "${HOME}/.cylc/job-init-env.sh" \
-        "${CYLC_DIR}/conf/job-init-env.sh" \
-        "${CYLC_DIR}/conf/job-init-env-default.sh"
+        "${CYLC_DIR}/conf/job-init-env.sh"
     do
         if [[ -f "${file_name}" ]]; then
             if "${CYLC_DEBUG:-false}"; then
@@ -120,7 +119,7 @@ __OUT__
     # Env-Script
     cylc__job__run_inst_func 'env_script'
     # Send task started message
-    cylc task message 'started' &
+    cylc message 'started' &
     CYLC_TASK_MESSAGE_STARTED_PID=$!
     # Access to the suite bin directory
     if [[ -n "${CYLC_SUITE_DEF_PATH:-}" && -d "${CYLC_SUITE_DEF_PATH}/bin" ]]
@@ -142,7 +141,7 @@ __OUT__
     rmdir "${CYLC_TASK_WORK_DIR}" 2>'/dev/null' || true
     # Send task succeeded message
     wait "${CYLC_TASK_MESSAGE_STARTED_PID}" 2>'/dev/null' || true
-    cylc task message 'succeeded' || true
+    cylc message 'succeeded' || true
     trap '' EXIT
     exit 0
 }
@@ -169,14 +168,15 @@ cylc__job__run_inst_func() {
 #   CYLC_VACATION_SIGNALS
 # Arguments:
 #   signal - trapped or given signal
-#   priority - message priority
+#   severity - message severity
 #   run_err_script - boolean, run job err script or not
-#   messages - (remaining arguments) messages to send back to the suite daemon
+#   messages - (remaining arguments)
+#              messages to send back to the suite server program
 # Returns:
 #   exit 1
 cylc__job_finish() {
     typeset signal="$1"
-    typeset priority="$2"
+    typeset severity="$2"
     typeset run_err_script="$3"
     shift 3
     typeset signal_name=
@@ -186,7 +186,7 @@ cylc__job_finish() {
     if [[ -n "${CYLC_TASK_MESSAGE_STARTED_PID:-}" ]]; then
         wait "${CYLC_TASK_MESSAGE_STARTED_PID}" 2>'/dev/null' || true
     fi
-    cylc task message -p "${priority}" "$@" || true
+    cylc message -s "${severity}" "$@" || true
     if $run_err_script; then
         cylc__job__run_inst_func 'err_script' "${signal}" >&2
     fi
