@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ class JobFileWriter(object):
         self.suite_env.clear()
         self.suite_env.update(suite_env)
 
-    def write(self, local_job_file_path, job_conf):
+    def write(self, local_job_file_path, job_conf, check_syntax=True):
         """Write each job script section in turn."""
 
         # ########### !!!!!!!! WARNING !!!!!!!!!!! #####################
@@ -76,24 +76,27 @@ class JobFileWriter(object):
                 pass
             raise exc
         # check syntax
-        try:
-            proc = Popen([job_conf['shell'], '-n', tmp_name], stderr=PIPE)
-        except OSError as exc:
-            # Popen has a bad habit of not telling you anything if it fails
-            # to run the executable.
-            if exc.filename is None:
-                exc.filename = job_conf['shell']
-            # Remove temporary file
+        if check_syntax:
             try:
-                os.unlink(tmp_name)
-            except OSError:
-                pass
-            raise exc
-        else:
-            if proc.wait():
-                # This will leave behind the temporary file,
-                # which is useful for debugging syntax errors, etc.
-                raise RuntimeError(proc.communicate()[1])
+                proc = Popen(
+                    [job_conf['shell'], '-n', tmp_name],
+                    stderr=PIPE, stdin=open(os.devnull))
+            except OSError as exc:
+                # Popen has a bad habit of not telling you anything if it fails
+                # to run the executable.
+                if exc.filename is None:
+                    exc.filename = job_conf['shell']
+                # Remove temporary file
+                try:
+                    os.unlink(tmp_name)
+                except OSError:
+                    pass
+                raise exc
+            else:
+                if proc.wait():
+                    # This will leave behind the temporary file,
+                    # which is useful for debugging syntax errors, etc.
+                    raise RuntimeError(proc.communicate()[1])
         # Make job file executable
         mode = (
             os.stat(tmp_name).st_mode |
@@ -125,8 +128,8 @@ class JobFileWriter(object):
     @staticmethod
     def _write_header(handle, job_conf):
         """Write job script header."""
-        handle.write("#!" + job_conf['shell'])
-        handle.write("\n#\n# ++++ THIS IS A CYLC TASK JOB SCRIPT ++++")
+        handle.write("#!%s -l\n" % job_conf['shell'])
+        handle.write("#\n# ++++ THIS IS A CYLC TASK JOB SCRIPT ++++")
         for prefix, value in [
                 ("# Suite: ", job_conf['suite_name']),
                 ("# Task: ", job_conf['task_id']),
@@ -153,7 +156,7 @@ class JobFileWriter(object):
         # Environment variables for prelude
         handle.write("\nexport CYLC_DIR='%s'" % (os.environ['CYLC_DIR']))
         if cylc.flags.debug:
-            handle.write("\nexport CYLC_DEBUG='true'")
+            handle.write("\nexport CYLC_DEBUG=true")
         for key in ['CYLC_VERSION'] + self._get_host_item(
                 job_conf, 'copyable environment variables'):
             if key in os.environ:
@@ -206,9 +209,13 @@ class JobFileWriter(object):
             ' '.join(job_conf['namespace_hierarchy']))
         handle.write(
             '\n    export CYLC_TASK_TRY_NUMBER=%s' % job_conf['try_num'])
-        if job_conf['param_var']:
-            for var, val in job_conf['param_var'].items():
-                handle.write('\n    export %s="%s"' % (var, val))
+        # Custom parameter environment variables
+        for var, tmpl in job_conf['param_env_tmpl'].items():
+            handle.write('\n    export %s="%s"' % (
+                var, tmpl % job_conf['param_var']))
+        # Standard parameter environment variables
+        for var, val in job_conf['param_var'].items():
+            handle.write('\n    export CYLC_TASK_PARAM_%s="%s"' % (var, val))
         if job_conf['work_d']:
             # Note: not an environment variable, but used by job.sh
             handle.write(
@@ -227,12 +234,29 @@ class JobFileWriter(object):
                 '\n    export CYLC_TASK_SSH_LOGIN_SHELL="%s"' %
                 self._get_host_item(job_conf, 'use login shell'))
 
-    def _write_environment_2(self, handle, job_conf):
+    @staticmethod
+    def _write_environment_2(handle, job_conf):
         """Run time environment part 2."""
         if job_conf['environment']:
             handle.write("\n\ncylc__job__inst__user_env() {")
             # Generate variable assignment expressions
             handle.write("\n    # TASK RUNTIME ENVIRONMENT:")
+
+            # NOTE: the reason for separate export of user-specified
+            # variables is this: inline export does not activate the
+            # error trap if sub-expressions fail, e.g. (note typo in
+            # 'echo' command name):
+            #   export FOO=$( ecko foo )  # error not trapped!
+            #   FOO=$( ecko foo )  # error trapped
+            # The export is done before variable definition to enable
+            # use of already defiend variables by command substitutions
+            # in later definitions:
+            #   FOO='foo'
+            #   BAR=$(script_using_FOO)
+            handle.write("\n    export")
+            for var in job_conf['environment']:
+                handle.write(" " + var)
+
             for var, val in job_conf['environment'].items():
                 value = str(val)  # (needed?)
                 match = re.match(r"^(~[^/\s]*/)(.*)$", value)
@@ -261,15 +285,6 @@ class JobFileWriter(object):
                 # | ~foo
                 # | ~
 
-            # NOTE: the reason for separate export of user-specified
-            # variables is this: inline export does not activate the
-            # error trap if sub-expressions fail, e.g. (note typo in
-            # 'echo' command name):
-            # export FOO=$( ecko foo )  # error not trapped!
-            # FOO=$( ecko foo )  # error trapped
-            handle.write("\n    export")
-            for var in job_conf['environment']:
-                handle.write(" " + var)
             handle.write("\n}")
 
     @classmethod
