@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 from optparse import OptionParser, OptionConflictError
 import os
 import cylc.flags
-from cylc.suite_host import get_user
 
 
 class CylcOptionParser(OptionParser):
@@ -28,8 +27,7 @@ class CylcOptionParser(OptionParser):
     """Common options for all cylc CLI commands."""
 
     MULTITASK_USAGE = """
-A TASKID is an identifier for matching individual task proxies and/or families
-of them. It can be written in these syntaxes:
+TASKID is a pattern to match task proxies or task families, or groups of them:
 * [CYCLE-POINT-GLOB/]TASK-NAME-GLOB[:TASK-STATE]
 * [CYCLE-POINT-GLOB/]FAMILY-NAME-GLOB[:TASK-STATE]
 * TASK-NAME-GLOB[.CYCLE-POINT-GLOB][:TASK-STATE]
@@ -116,8 +114,7 @@ Arguments:"""
                 "Other user account name. This results in "
                 "command reinvocation on the remote account."
             ),
-            metavar="USER", default=get_user(),
-            action="store", dest="owner")
+            metavar="USER", action="store", dest="owner")
         self.add_std_option(
             "--host",
             help="Other host name. This results in "
@@ -175,8 +172,8 @@ Arguments:"""
                 "--print-uuid",
                 help=(
                     "Print the client UUID to stderr. "
-                    "This can be matched "
-                    "to information logged by the receiving suite daemon."
+                    "This can be matched to information logged "
+                    "by the receiving suite server program."
                 ),
                 action="store_true", default=False, dest="print_uuid")
             self.add_std_option(
@@ -250,17 +247,18 @@ Arguments:"""
                     "Set initial cycle point. "
                     "Required if not defined in suite.rc."))
 
-    def parse_args(self, remove_opts=[]):
+    def parse_args(self, remove_opts=None):
         """Parse options and arguments, overrides OptionParser.parse_args."""
         if self.auto_add:
             # Add common options after command-specific options.
             self.add_std_options()
 
-        for opt in remove_opts:
-            try:
-                self.remove_option(opt)
-            except:
-                pass
+        if remove_opts:
+            for opt in remove_opts:
+                try:
+                    self.remove_option(opt)
+                except ValueError:
+                    pass
 
         (options, args) = OptionParser.parse_args(self)
 
@@ -295,7 +293,7 @@ Arguments:"""
 
         """
         if (options.multitask_compat and len(mtask_args) == 2 and
-                all(["/" not in mtask_arg for mtask_arg in mtask_args]) and
+                not any("/" in mtask_arg for mtask_arg in mtask_args) and
                 "." not in mtask_args[1]):
             # For backward compat, argument list should have 2 elements.
             # Element 1 may be a regular expression, so it may contain "." but
