@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -56,6 +56,8 @@ class Profiler(object):
         """Print a message to standard out with the current memory usage."""
         if not self.enabled:
             return
-        proc = Popen(["ps", "h", "-orss", str(os.getpid())], stdout=PIPE)
+        proc = Popen(
+            ["ps", "h", "-orss", str(os.getpid())],
+            stdin=open(os.devnull), stdout=PIPE)
         memory = int(proc.communicate()[0])
         print "PROFILE: Memory: %d KiB: %s" % (memory, message)
