@@ -1,5 +1,5 @@
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -212,7 +212,7 @@ def process_out_file(file_name, suite_start_time, validate=False):
                                     loop_mem_entries[-1][0])
 
         # Maximum memory usage.
-        ret['mxmem'] = max([entry[2] for entry in ret['memory']])
+        ret['mxmem'] = max(entry[2] for entry in ret['memory'])
 
         # Startup time (time from running cmd to reaching the end of the first
         # loop).
@@ -293,7 +293,7 @@ def make_table(results, versions, experiment, quick_analysis=False):
             data = results[version['id']][experiment['id']]
             run_names = data.keys()
             try:
-                run_names.sort(key=lambda x: int(x))
+                run_names.sort(key=int)
             except ValueError:
                 run_names.sort()
             for run_name in run_names:
@@ -327,14 +327,14 @@ def print_table(table, transpose=False):
     col_widths = []
     for col_no in range(len(table[0])):
         col_widths.append(
-            max([len(table[row_no][col_no]) for row_no in range(len(table))]))
+            max(len(table[row_no][col_no]) for row_no in range(len(table))))
 
     for row_no in range(len(table)):
         for col_no in range(len(table[row_no])):
             if col_no != 0:
                 sys.stdout.write('  ')
             cell = table[row_no][col_no]
-            if type(cell) is list:
+            if isinstance(cell, list):
                 sys.stdout.write('-' * col_widths[col_no])
             else:
                 sys.stdout.write(cell + ' ' * (col_widths[col_no] - len(cell)))
@@ -358,8 +358,7 @@ def plot_single(results, run_names, versions, metric, experiment,
                  color=colours[bar_no])
 
     axis.set_xticks(ind + ((width * n_bars) / 2.))
-    axis.set_xticklabels([version['name'] for
-                          version in versions])
+    axis.set_xticklabels([version['name'] for version in versions])
     axis.set_xlabel('Cylc Version')
     axis.set_xlim([0, (1. * n_groups) - spacing])
     if len(run_names) > 1:
