@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,8 @@ import sys
 from cylc.cfgspec.globalcfg import GLOBAL_CFG
 
 
-def prompt(question, force=False, gui=False, no_force=False, no_abort=False):
+def prompt(question, force=False, gui=False, no_force=False, no_abort=False,
+           keep_above=True):
     """Interactive Yes/No prompt for cylc CLI scripts.
 
     For convenience, on No we just exit rather than return.
@@ -38,6 +39,7 @@ def prompt(question, force=False, gui=False, no_force=False, no_abort=False):
             gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,
             question
         )
+        dialog.set_keep_above(keep_above)
         gui_response = dialog.run()
         response_no = (gui_response != gtk.RESPONSE_YES)
     else:
