@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@ from cylc.suite_logging import LOG, ERR
 class CylcSuiteDAOTableColumn(object):
     """Represent a column in a table."""
 
+    __slots__ = ('name', 'datatype', 'is_primary_key')
+
     def __init__(self, name, datatype, is_primary_key):
         self.name = name
         self.datatype = datatype
@@ -43,6 +45,9 @@ class CylcSuiteDAOTable(object):
     FMT_INSERT = "INSERT OR REPLACE INTO %(name)s VALUES(%(values_str)s)"
     FMT_UPDATE = "UPDATE %(name)s SET %(set_str)s%(where_str)s"
 
+    __slots__ = ('name', 'columns', 'delete_queues', 'insert_queue',
+                 'update_queues')
+
     def __init__(self, name, column_items):
         self.name = name
         self.columns = []
