@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,8 +23,8 @@ from pipes import quote
 
 from cylc.cfgspec.globalcfg import GLOBAL_CFG
 from cylc.mp_pool import SuiteProcContext
-from cylc.suite_host import get_suite_host, get_user
-from cylc.suite_logging import ERR, LOG
+from cylc.hostuserutil import get_host, get_user
+from cylc.suite_logging import LOG
 
 
 class SuiteEventError(Exception):
@@ -106,7 +106,7 @@ class SuiteEventHandler(object):
                     '-s', subject,
                     '-r', self.get_events_conf(
                         config,
-                        'mail from', 'notifications@' + get_suite_host()),
+                        'mail from', 'notifications@' + get_host()),
                     self.get_events_conf(config, 'mail to', get_user()),
                 ],
                 env=env,
@@ -172,12 +172,13 @@ class SuiteEventHandler(object):
                 self.proc_pool.put_command(
                     proc_ctx, self._run_event_handlers_callback)
 
-    def _run_event_handlers_callback(self, proc_ctx, abort_on_error=False):
+    @staticmethod
+    def _run_event_handlers_callback(proc_ctx, abort_on_error=False):
         """Callback on completion of a suite event handler."""
         if proc_ctx.ret_code:
             msg = '%s EVENT HANDLER FAILED' % proc_ctx.cmd_key[1]
             LOG.error(str(proc_ctx))
-            ERR.error(msg)
+            LOG.error(msg)
             if abort_on_error:
                 raise SuiteEventError(msg)
         else:
