@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,14 +25,16 @@ import logging
 import logging.handlers
 import os
 import sys
-import time
+from time import time
+
 
 try:
     # Permits use of module by standalone utilities that aren't part of a
     # suite.
     from cylc.cfgspec.globalcfg import GLOBAL_CFG
-except:
+except ImportError:
     pass
+import cylc.flags
 from cylc.wallclock import (get_time_string_from_unix_time,
                             get_current_time_string)
 
@@ -94,12 +96,14 @@ class RollingFileHandler(logging.handlers.BaseRotatingHandler):
         self.maxBytes = maxBytes
         self.archive_length = archive_length
         self.syncronised_group = None
+        self.file_stamp_fcn = file_stamp_fcn
 
-        if file_stamp_fcn:
-            self._gen_file_stamp = file_stamp_fcn
+    def _gen_file_stamp(self):
+        """Use time or self.file_stamp_fcn to generate file name."""
+        if callable(self.file_stamp_fcn):
+            return self.file_stamp_fcn()
         else:
-            self._gen_file_stamp = lambda: ('%f' % time.time()).replace('.',
-                                                                        '')
+            return ('%f' % time()).replace('.', '')
 
     def notifyRollover(self):
         """Notify this RollingFilehandler that its log file has been rolled.
@@ -282,13 +286,6 @@ class SuiteLog(object):
         # File streams
         self.streams = []
 
-        # Filename stamp functions.
-        if self.is_test:
-            self.stamp = lambda: get_current_time_string(True, True, True
-                                                         ).replace('.', '-')
-        else:
-            self.stamp = lambda: get_current_time_string(False, True, True)
-
         SuiteLog.__INSTANCE = self
 
     @classmethod
@@ -308,6 +305,29 @@ class SuiteLog(object):
         suite logging."""
         return GLOBAL_CFG.get_derived_host_item(suite, 'suite log directory')
 
+    def get_lines(self, log, prev_size, max_lines=10):
+        """Read content from log file up to max_lines from prev_size."""
+        if prev_size is None:
+            prev_size = 0
+        else:
+            prev_size = int(prev_size)
+        path = self.get_log_path(log)
+        try:
+            size = os.path.getsize(path)
+        except (IOError, OSError):
+            size = 0
+        if size == prev_size:
+            return "", prev_size
+        try:
+            handle = open(path, "r")
+            handle.seek(prev_size)
+            new_content = handle.read()
+            handle.close()
+        except (IOError, OSError):
+            return "", prev_size
+        new_content_lines = new_content.splitlines()[-int(max_lines):]
+        return "\n".join(new_content_lines), size
+
     def get_log(self, log):
         """Return the requested logger."""
         if log in self.loggers:
@@ -326,7 +346,14 @@ class SuiteLog(object):
         if log in self.loggers:
             return self.log_paths[log]
 
-    def pimp(self, detach=False, log_logger_level=logging.INFO):
+    def get_stamp(self):
+        """Returns the current time stamp for logging."""
+        if self.is_test:
+            return get_current_time_string(True, True, True).replace('.', '-')
+        else:
+            return get_current_time_string(False, True, True)
+
+    def pimp(self, detach=False, log_logger_level=None):
         """Initiate the suite logs."""
         if not self.loggers[self.LOG]:
             # Don't initiate logs if they exist already.
@@ -336,7 +363,7 @@ class SuiteLog(object):
         elif self.roll_at_startup:
             self._group.roll_all()
 
-    def _create_logs(self, detach, log_logger_level=logging.INFO):
+    def _create_logs(self, detach, log_logger_level=None):
         """Sets up the log files and their file handlers."""
         # Logging formatters.
         # plain_formatter = logging.Formatter('%(message)s')
@@ -351,6 +378,14 @@ class SuiteLog(object):
         # Multi-line filters.
         multi_line_indented_filter = MultiLineFilter('\n\t')
 
+        # Level
+        if log_logger_level is None:
+            if cylc.flags.debug:
+                log_logger_level = logging.DEBUG
+            elif cylc.flags.verbose:
+                log_logger_level = logging.DEBUG
+            else:
+                log_logger_level = logging.INFO
         # --- Create the 'log' logger. ---
         log = logging.getLogger(self.LOG)
         self.loggers[self.LOG] = log
@@ -362,7 +397,7 @@ class SuiteLog(object):
                                     mode='a',
                                     maxBytes=self.max_bytes,
                                     archive_length=self.archive_length,
-                                    file_stamp_fcn=self.stamp)
+                                    file_stamp_fcn=self.get_stamp)
         log_fh.setLevel(log_logger_level)
         log_fh.setFormatter(iso8601_formatter)
         log.addHandler(log_fh)
@@ -372,7 +407,7 @@ class SuiteLog(object):
                                         mode='a',
                                         maxBytes=self.max_bytes,
                                         archive_length=self.archive_length,
-                                        file_stamp_fcn=self.stamp)
+                                        file_stamp_fcn=self.get_stamp)
         log_err_fh.setLevel(logging.WARNING)
         log_err_fh.setFormatter(iso8601_formatter)
         log.addHandler(log_err_fh)
@@ -380,30 +415,30 @@ class SuiteLog(object):
         # --- Create the 'out' logger. ---
         out = logging.getLogger(self.OUT)
         self.loggers[self.OUT] = out
-        out.setLevel(logging.INFO)
+        out.setLevel(log_logger_level)
 
         # Output to the 'out' file.
         out_fh = RollingFileHandler(self.log_paths[self.OUT],
                                     mode='a',
                                     maxBytes=self.max_bytes,
                                     archive_length=self.archive_length,
-                                    file_stamp_fcn=self.stamp)
-        out_fh.setLevel(logging.INFO)
+                                    file_stamp_fcn=self.get_stamp)
+        out_fh.setLevel(log_logger_level)
         out_fh.setFormatter(iso8601_formatter)
         out.addHandler(out_fh)
 
         # --- Create the 'err' logger. ---
         err = logging.getLogger(self.ERR)
         self.loggers[self.ERR] = err
-        err.setLevel(logging.WARNING)
+        err.setLevel(log_logger_level)
         err.addFilter(multi_line_indented_filter)
 
         # Output to the 'err' file.
         err_fh = RollingFileHandler(self.log_paths[self.ERR],
                                     mode='a',
                                     maxBytes=self.max_bytes,
-                                    file_stamp_fcn=self.stamp)
-        err_fh.setLevel(logging.WARNING)
+                                    file_stamp_fcn=self.get_stamp)
+        err_fh.setLevel(log_logger_level)
         err_fh.setFormatter(iso8601_formatter)
         err.addHandler(err_fh)
 
@@ -419,24 +454,24 @@ class SuiteLog(object):
 
             # LOG: warnings or higher -> stderr
             log_stderr_fh = logging.StreamHandler(sys.stderr)
-            log_stderr_fh.setLevel(logging.WARNING)
+            log_stderr_fh.setLevel(log_logger_level)
             log_stderr_fh.setFormatter(iso8601_formatter)
             log.addHandler(log_stderr_fh)
             # OUT: info or higher -> stdout
             out_stdout_fh = logging.StreamHandler(sys.stdout)
-            out_stdout_fh.setLevel(logging.INFO)
+            out_stdout_fh.setLevel(log_logger_level)
             out_stdout_fh.setFormatter(iso8601_formatter)
             out.addHandler(out_stdout_fh)
             # ERR: warnings or higher -> stderr
             err_stderr_fh = logging.StreamHandler(sys.stderr)
-            err_stderr_fh.setLevel(logging.WARNING)
+            err_stderr_fh.setLevel(log_logger_level)
             err_stderr_fh.setFormatter(iso8601_formatter)
             err.addHandler(err_stderr_fh)
 
     def _register_syncronised_logs(self):
         """Establishes synchronisation between the logs."""
         self._group = RollingFileHandlerGroup()
-        for log in [self.loggers[log_name] for log_name in self.ALL_LOGS]:
+        for log in (self.loggers[log_name] for log_name in self.ALL_LOGS):
             self._group.add(log)
             for handler in log.handlers:
                 if isinstance(handler, RollingFileHandler):
@@ -479,6 +514,7 @@ class STDLogger(object):
             raise Exception('Unknown logger provided "{0}"'.format(log))
         self.log_ = log
         self.logger = logging.getLogger(log)
+        self.update_time = time()
 
     def log(self, level, *args, **kwargs):
         try:
@@ -491,6 +527,7 @@ class STDLogger(object):
             except AttributeError:
                 args = ("[%s] -%s" % (itask, args[0]),) + args[1:]
             args = tuple(args)
+        self.update_time = time()
         if self.logger.handlers:
             # If this logger has file handlers write out to it.
             self.logger.log(level, *args, **kwargs)
