@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,8 +26,8 @@ import sys
 
 import cylc.flags
 from cylc.mkdir_p import mkdir_p
-from cylc.suite_host import (
-    get_local_ip_address, get_suite_host, get_user, is_remote, is_remote_host)
+from cylc.hostuserutil import (
+    get_local_ip_address, get_host, get_user, is_remote, is_remote_host)
 
 
 class SuiteServiceFileError(Exception):
@@ -39,14 +39,16 @@ class SuiteSrvFilesManager(object):
     """Suite service files management."""
 
     DELIM = "/"
-    DIR_BASE_AUTH = 'auth'
+    DIR_BASE_AUTH = "auth"
     DIR_BASE_SRV = ".service"
     FILE_BASE_CONTACT = "contact"
-    FILE_BASE_PASSPHRASE = 'passphrase'
+    FILE_BASE_PASSPHRASE = "passphrase"
     FILE_BASE_SOURCE = "source"
-    FILE_BASE_SSL_CERT = 'ssl.cert'
-    FILE_BASE_SSL_PEM = 'ssl.pem'
+    FILE_BASE_SSL_CERT = "ssl.cert"
+    FILE_BASE_SSL_PEM = "ssl.pem"
     FILE_BASE_SUITE_RC = "suite.rc"
+    KEY_API = "CYLC_API"
+    KEY_COMMS_PROTOCOL = "CYLC_COMMS_PROTOCOL"  # default (or none?)
     KEY_DIR_ON_SUITE_HOST = "CYLC_DIR_ON_SUITE_HOST"
     KEY_HOST = "CYLC_SUITE_HOST"
     KEY_NAME = "CYLC_SUITE_NAME"
@@ -58,9 +60,11 @@ class SuiteSrvFilesManager(object):
     KEY_TASK_MSG_RETRY_INTVL = "CYLC_TASK_MSG_RETRY_INTVL"
     KEY_TASK_MSG_TIMEOUT = "CYLC_TASK_MSG_TIMEOUT"
     KEY_VERSION = "CYLC_VERSION"
+    NO_TITLE = "No title provided"
     PASSPHRASE_CHARSET = ascii_letters + digits
     PASSPHRASE_LEN = 20
-    KEY_COMMS_PROTOCOL = "CYLC_COMMS_PROTOCOL"  # default (or none?)
+    PS_OPTS = '-opid,args'
+    REC_TITLE = re.compile(r"^\s*title\s*=\s*(.*)\s*$")
 
     def __init__(self):
         self.local_passphrases = set()
@@ -78,7 +82,7 @@ class SuiteSrvFilesManager(object):
         if owner is None:
             owner = get_user()
         if host is None:
-            host = get_suite_host()
+            host = get_host()
         path = self._get_cache_dir(reg, owner, host)
         self.cache[self.FILE_BASE_PASSPHRASE][(reg, owner, host)] = value
         # Dump to a file only for remote suites loaded via SSH.
@@ -92,11 +96,26 @@ class SuiteSrvFilesManager(object):
                     import traceback
                     traceback.print_exc()
 
-    def detect_old_contact_file(self, reg):
+    def detect_old_contact_file(self, reg, check_host_port=None):
         """Detect old suite contact file.
 
-        Raise SuiteServiceFileError if old contact file exists, and there is
-        evidence that the old suite is still running.
+        If old contact file does not exist, do nothing. If old contact file
+        exists, but suite process is definitely not alive, remove old contact
+        file. If old contact file exists and suite process still alive, raise
+        SuiteServiceFileError. If check_host_port is specified and does not
+        match the (host, port) value in the old contact file, raise
+        AssertionError.
+
+        Args:
+            reg (str): suite name
+            check_host_port (tuple): (host, port) to check against
+
+        Raise:
+            AssertionError:
+                If old contact file exists but does not have matching
+                (host, port) with value of check_host_port.
+            SuiteServiceFileError:
+                If old contact file exists and the suite process still alive.
         """
         # An old suite of the same name may be running if a contact file exists
         # and can be loaded.
@@ -108,11 +127,15 @@ class SuiteSrvFilesManager(object):
         except (IOError, ValueError, SuiteServiceFileError):
             # Contact file does not exist or corrupted, should be OK to proceed
             return
+        if check_host_port and check_host_port != (old_host, int(old_port)):
+            raise AssertionError("%s != (%s, %s)" % (
+                check_host_port, old_host, old_port))
         # Run the "ps" command to see if the process is still running or not.
         # If the old suite process is still running, it should show up with the
         # same command line as before.
+        # Terminate command after 10 seconds to prevent hanging, etc.
         old_pid_str = old_proc_str.split(None, 1)[0].strip()
-        cmd = ["ps", "-opid,args", str(old_pid_str)]
+        cmd = ["timeout", "10", "ps", self.PS_OPTS, str(old_pid_str)]
         if is_remote_host(old_host):
             import shlex
             from cylc.cfgspec.globalcfg import GLOBAL_CFG
@@ -120,7 +143,7 @@ class SuiteSrvFilesManager(object):
             cmd = shlex.split(ssh_str) + ["-n", old_host] + cmd
         from subprocess import Popen, PIPE
         from time import sleep, time
-        proc = Popen(cmd, stdout=PIPE, stderr=PIPE)
+        proc = Popen(cmd, stdin=open(os.devnull), stdout=PIPE, stderr=PIPE)
         # Terminate command after 10 seconds to prevent hanging SSH, etc.
         timeout = time() + 10.0
         while proc.poll() is None:
@@ -128,8 +151,12 @@ class SuiteSrvFilesManager(object):
                 proc.terminate()
             sleep(0.1)
         fname = self.get_contact_file(reg)
-        proc.wait()
-        for line in reversed(proc.communicate()[0].splitlines()):
+        ret_code = proc.wait()
+        out, err = proc.communicate()
+        if cylc.flags.debug and ret_code:
+            sys.stderr.write(
+                "%s  # return %d\n%s\n" % (' '.join(cmd), ret_code, err))
+        for line in reversed(out.splitlines()):
             if line.strip() == old_proc_str:
                 # Suite definitely still running
                 break
@@ -142,18 +169,18 @@ class SuiteSrvFilesManager(object):
                 except OSError:
                     break
 
-        sys.stderr.write(
+        raise SuiteServiceFileError(
             (
                 r"""ERROR, suite contact file exists: %(fname)s
 
-If %(suite)s is not running, delete the suite contact file and try again.
-If it is running but unresponsive, kill any left over suite processes too.
-
-To see if %(suite)s is running on '%(host)s:%(port)s':
- * cylc scan -n '\b%(suite)s\b' '%(host)s'
- * cylc ping -v --host='%(host)s' '%(suite)s'
- * ssh -n '%(host)s' 'ps -o pid,args %(pid)s'
+Suite "%(suite)s" is already running, and listening at "%(host)s:%(port)s".
 
+To start a new run, stop the old one first with one or more of these:
+* cylc stop %(suite)s              # wait for active tasks/event handlers
+* cylc stop --kill %(suite)s       # kill active tasks and wait
+* cylc stop --now %(suite)s        # don't wait for active tasks
+* cylc stop --now --now %(suite)s  # don't wait
+* ssh -n "%(host)s" kill %(pid)s   # final brute force!
 """
             ) % {
                 "host": old_host,
@@ -163,14 +190,13 @@ To see if %(suite)s is running on '%(host)s:%(port)s':
                 "suite": reg,
             }
         )
-        raise SuiteServiceFileError(
-            "ERROR, suite contact file exists: %s" % fname)
 
     def dump_contact_file(self, reg, data):
         """Create contact file. Data should be a key=value dict."""
         with open(self.get_contact_file(reg), "wb") as handle:
             for key, value in sorted(data.items()):
                 handle.write("%s=%s\n" % (key, value))
+            os.fsync(handle.fileno())
 
     def get_contact_file(self, reg):
         """Return name of contact file."""
@@ -219,7 +245,7 @@ To see if %(suite)s is running on '%(host)s:%(port)s':
                 env_keys.append('CYLC_SUITE_RUN_DIR')
             elif self.KEY_SUITE_RUN_DIR_ON_SUITE_HOST in os.environ:
                 # 1(b)/ Task messaging call via ssh messaging.
-                env_keys.append(self.SUITE_RUN_DIR_ON_SUITE_HOST)
+                env_keys.append(self.KEY_SUITE_RUN_DIR_ON_SUITE_HOST)
             for key in env_keys:
                 path = os.path.join(os.environ[key], self.DIR_BASE_SRV)
                 if content:
@@ -235,7 +261,7 @@ To see if %(suite)s is running on '%(host)s:%(port)s':
             if my_owner is None:
                 my_owner = get_user()
             if my_host is None:
-                my_host = get_suite_host()
+                my_host = get_host()
             try:
                 return self.cache[item][(reg, my_owner, my_host)]
             except KeyError:
@@ -337,7 +363,7 @@ To see if %(suite)s is running on '%(host)s:%(port)s':
                 results.append([
                     reg,
                     self.get_suite_source_dir(reg),
-                    self._get_suite_title(reg)])
+                    self.get_suite_title(reg)])
             except (IOError, SuiteServiceFileError) as exc:
                 print >> sys.stderr, str(exc)
         return results
@@ -459,7 +485,7 @@ To see if %(suite)s is running on '%(host)s:%(port)s':
             # OpenSSL not installed, so we can't use HTTPS anyway.
             return
         # Use suite host as the 'common name', but no more than 64 chars.
-        host = get_suite_host()
+        host = get_host()
         common_name = host
         if len(common_name) > 64:
             common_name = common_name[:61] + "..."
@@ -484,7 +510,7 @@ To see if %(suite)s is running on '%(host)s:%(port)s':
                     return  # certificate good for the same suite and host
         # Generate a new certificate
         cert_obj = crypto.X509()
-        cert_obj.get_subject().O = "Cylc"
+        # cert_obj.get_subject().O = "Cylc"
         cert_obj.get_subject().CN = common_name
         cert_obj.gmtime_adj_notBefore(0)
         cert_obj.gmtime_adj_notAfter(10 * 365 * 24 * 60 * 60)  # 10 years.
@@ -497,7 +523,8 @@ To see if %(suite)s is running on '%(host)s:%(port)s':
             path, self.FILE_BASE_SSL_CERT,
             crypto.dump_certificate(crypto.FILETYPE_PEM, cert_obj))
 
-    def _dump_item(self, path, item, value):
+    @staticmethod
+    def _dump_item(path, item, value):
         """Dump "value" to a file called "item" in the directory "path".
 
         1. File permission should already be user-read-write-only on
@@ -522,14 +549,14 @@ To see if %(suite)s is running on '%(host)s:%(port)s':
             os.path.expanduser("~"), ".cylc", self.DIR_BASE_AUTH,
             "%s@%s" % (owner, host), reg)
 
-    def _get_suite_title(self, reg):
+    def get_suite_title(self, reg):
         """Return the the suite title without a full file parse
 
         Limitations:
         * 1st line of title only.
         * Assume title is not in an include-file.
         """
-        title = "No title provided"
+        title = self.NO_TITLE
         for line in open(self.get_suite_rc(reg), 'rb'):
             if line.lstrip().startswith("[meta]"):
                 # continue : title comes inside [meta] section
@@ -537,7 +564,7 @@ To see if %(suite)s is running on '%(host)s:%(port)s':
             elif line.lstrip().startswith("["):
                 # abort: title comes before first [section]
                 break
-            match = re.match('^\s*title\s*=\s*(.*)\s*$', line)
+            match = self.REC_TITLE.match(line)
             if match:
                 title = match.groups()[0].strip('"\'')
         return title
@@ -566,7 +593,7 @@ To see if %(suite)s is running on '%(host)s:%(port)s':
                     if owner is None:
                         owner = get_user()
                     if host is None:
-                        host = get_suite_host()
+                        host = get_host()
                     host_value = data.get(self.KEY_HOST, "")
                     self.can_use_load_auths[(reg, owner, host)] = (
                         reg == data.get(self.KEY_NAME) and
@@ -601,7 +628,7 @@ To see if %(suite)s is running on '%(host)s:%(port)s':
             # Attempt to read suite contact file via the local filesystem.
             path = r'%(run_d)s/%(srv_base)s' % {
                 'run_d': GLOBAL_CFG.get_derived_host_item(
-                    reg, 'suite run directory', host, owner,
+                    reg, 'suite run directory', 'localhost', owner,
                     replace_home=False),
                 'srv_base': self.DIR_BASE_SRV,
             }
@@ -628,7 +655,8 @@ To see if %(suite)s is running on '%(host)s:%(port)s':
         command += ['-n', owner + '@' + host, script]
         from subprocess import Popen, PIPE
         try:
-            proc = Popen(command, stdout=PIPE, stderr=PIPE)
+            proc = Popen(
+                command, stdin=open(os.devnull), stdout=PIPE, stderr=PIPE)
         except OSError:
             if cylc.flags.debug:
                 import traceback
