@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,26 +38,23 @@ import traceback
 
 from parsec.config import ItemNotFoundError
 
+from cylc.broadcast_mgr import BroadcastMgr
 from cylc.cfgspec.globalcfg import GLOBAL_CFG
 import cylc.flags
 from cylc.mp_pool import SuiteProcContext
-from cylc.network.suite_broadcast_server import BroadcastServer
 from cylc.suite_logging import ERR, LOG
-from cylc.suite_host import get_suite_host, get_user
+from cylc.hostuserutil import get_host, get_user
 from cylc.task_action_timer import TaskActionTimer
 from cylc.task_message import TaskMessage
 from cylc.task_state import (
-    TASK_STATUSES_ACTIVE, TASK_STATUS_READY, TASK_STATUS_SUBMITTED,
-    TASK_STATUS_SUBMIT_RETRYING, TASK_STATUS_SUBMIT_FAILED,
-    TASK_STATUS_RUNNING, TASK_STATUS_RETRYING, TASK_STATUS_FAILED,
-    TASK_STATUS_SUCCEEDED)
+    TASK_STATUS_READY, TASK_STATUS_SUBMITTED, TASK_STATUS_SUBMIT_RETRYING,
+    TASK_STATUS_SUBMIT_FAILED, TASK_STATUS_RUNNING, TASK_STATUS_RETRYING,
+    TASK_STATUS_FAILED, TASK_STATUS_SUCCEEDED)
 from cylc.task_outputs import (
     TASK_OUTPUT_SUBMITTED, TASK_OUTPUT_STARTED, TASK_OUTPUT_SUCCEEDED,
     TASK_OUTPUT_FAILED)
 from cylc.wallclock import (
-    get_current_time_string,
-    get_unix_time_from_time_string,
-    RE_DATE_TIME_FORMAT_EXTENDED)
+    get_current_time_string, RE_DATE_TIME_FORMAT_EXTENDED)
 
 
 CustomTaskEventHandlerContext = namedtuple(
@@ -106,23 +103,31 @@ class TaskEventsManager(object):
     NN = "NN"
     POLLED_INDICATOR = "(polled)"
     RE_MESSAGE_TIME = re.compile(
-        '\A(.+) at (' + RE_DATE_TIME_FORMAT_EXTENDED + ')\Z')
+        '\A(.+) at (' + RE_DATE_TIME_FORMAT_EXTENDED + ')\Z', re.DOTALL)
 
-    def __init__(self, suite, proc_pool, suite_db_mgr):
+    def __init__(self, suite, proc_pool, suite_db_mgr, broadcast_mgr=None):
         self.suite = suite
         self.suite_url = None
-        self.suite_cfg = []
+        self.suite_cfg = {}
         self.proc_pool = proc_pool
         self.suite_db_mgr = suite_db_mgr
+        if broadcast_mgr is None:
+            broadcast_mgr = BroadcastMgr(self.suite_db_mgr)
+        self.broadcast_mgr = broadcast_mgr
         self.mail_interval = 0.0
         self.mail_footer = None
         self.next_mail_time = None
         self.event_timers = {}
+        # Set pflag = True to stimulate task dependency negotiation whenever a
+        # task changes state in such a way that others could be affected. The
+        # flag should only be turned off again after use in
+        # Scheduler.process_tasks, to ensure that dependency negotation occurs
+        # when required.
+        self.pflag = False
 
-    @staticmethod
-    def get_host_conf(itask, key, default=None, skey="remote"):
+    def get_host_conf(self, itask, key, default=None, skey="remote"):
         """Return a host setting from suite then global configuration."""
-        overrides = BroadcastServer.get_inst().get(itask.identity)
+        overrides = self.broadcast_mgr.get_broadcast(itask.identity)
         if skey in overrides and overrides[skey].get(key) is not None:
             return overrides[skey][key]
         elif itask.tdef.rtconfig[skey].get(key) is not None:
@@ -243,16 +248,36 @@ class TaskEventsManager(object):
             elif ctx.ctx_type == self.HANDLER_JOB_LOGS_RETRIEVE:
                 self._process_job_logs_retrieval(schd_ctx, ctx, id_keys)
 
-    def process_message(self, itask, priority, message, poll_event_time=None,
-                        is_incoming=False):
+    def _poll_to_confirm(self, itask, status_gt, poll_func):
+        """Poll itask to confirm an apparent state reversal."""
+        if (itask.state.is_greater_than(status_gt) and not
+                itask.state.confirming_with_poll):
+            poll_func(self.suite, [itask],
+                      msg="polling %s to confirm state" % itask.identity)
+            itask.state.confirming_with_poll = True
+            return True
+        else:
+            itask.state.confirming_with_poll = False
+            return False
+
+    def process_message(self, itask, severity, message, poll_func,
+                        poll_event_time=None, is_incoming=False):
         """Parse an incoming task message and update task state.
 
-        Incoming is e.g. "succeeded at <TIME>".
+        Incoming, e.g. "succeeded at <TIME>", may be from task job or polling.
 
-        Correctly handle late (out of order) message which would otherwise set
-        the state backward in the natural order of events.
+        It is possible for my current state to be inconsistent with an incoming
+        message (whether normal or polled) e.g. due to a late poll result, or a
+        network outage, or manual state reset. To handle this, if a message
+        would take the task state backward, issue a poll to confirm instead of
+        changing state - then always believe the next message. Note that the
+        next message might not be the result of this confirmation poll, in the
+        unlikely event that a job emits a succession of messages very quickly,
+        but this is the best we can do without somehow uniquely associating
+        each poll with its result message.
 
         """
+
         is_polled = poll_event_time is not None
         # Log incoming messages with '>' to distinguish non-message log entries
         message_flag = ""
@@ -262,7 +287,7 @@ class TaskEventsManager(object):
             itask.state.status, message_flag, message)
         if poll_event_time is not None:
             log_message += ' %s' % self.POLLED_INDICATOR
-        LOG.log(self.LEVELS.get(priority, INFO), log_message, itask=itask)
+        LOG.log(self.LEVELS.get(severity, INFO), log_message, itask=itask)
 
         # Strip the "at TIME" suffix.
         event_time = poll_event_time
@@ -279,78 +304,51 @@ class TaskEventsManager(object):
             itask.summary['latest_message'] += " %s" % self.POLLED_INDICATOR
         cylc.flags.iflag = True
 
-        # Failed tasks do not send messages unless declared resurrectable
-        if itask.state.status == TASK_STATUS_FAILED:
-            if itask.tdef.rtconfig['enable resurrection']:
-                LOG.warning(
-                    'message received while in the failed state:' +
-                    ' I am returning from the dead!',
-                    itask=itask)
-            else:
-                LOG.warning(
-                    'message rejected while in the failed state:\n  %s' %
-                    message,
-                    itask=itask)
+        # Satisfy my output, if possible, and record the result.
+        an_output_was_satisfied = itask.state.outputs.set_msg_trg_completion(
+            message=message, is_completed=True)
+
+        if message == TASK_OUTPUT_STARTED:
+            if self._poll_to_confirm(itask, TASK_STATUS_RUNNING, poll_func):
                 return
-
-        if is_polled and itask.state.status not in TASK_STATUSES_ACTIVE:
-            # A poll result can come in after a task finishes.
-            LOG.warning(
-                "Ignoring late poll result: task is not active", itask=itask)
-            return
-
-        # Check registered outputs
-        output_has_changed = itask.state.outputs.set_completed(message)
-        if output_has_changed is False:  # Not True or None
-            # This output has already been reported complete. Not an error
-            # condition - maybe the network was down for a bit. Ok for
-            # polling as multiple polls *should* produce the same result.
-            if not is_polled:
-                LOG.debug(
-                    "ignore - output already completed:\n  %s" % message,
-                    itask=itask)
-            return
-
-        if (message == TASK_OUTPUT_STARTED and
-                itask.state.status in [
-                    TASK_STATUS_READY, TASK_STATUS_SUBMITTED,
-                    TASK_STATUS_SUBMIT_FAILED]):
             self._process_message_started(itask, event_time)
-        elif (message == TASK_OUTPUT_SUCCEEDED and
-                itask.state.status in [
-                    TASK_STATUS_READY, TASK_STATUS_SUBMITTED,
-                    TASK_STATUS_SUBMIT_FAILED, TASK_STATUS_RUNNING,
-                    TASK_STATUS_FAILED]):
+        elif message == TASK_OUTPUT_SUCCEEDED:
+            if self._poll_to_confirm(itask, TASK_STATUS_SUCCEEDED, poll_func):
+                return
             self._process_message_succeeded(itask, event_time)
-        elif (message == TASK_OUTPUT_FAILED and
-                itask.state.status in [
-                    TASK_STATUS_READY, TASK_STATUS_SUBMITTED,
-                    TASK_STATUS_SUBMIT_FAILED, TASK_STATUS_RUNNING]):
-            # Generic fail message, via polling.
-            # (submit- states in case of very fast submission and execution).
+        elif message == TASK_OUTPUT_FAILED:
+            if self._poll_to_confirm(itask, TASK_STATUS_FAILED, poll_func):
+                return
             self._process_message_failed(itask, event_time, self.JOB_FAILED)
         elif message == self.EVENT_SUBMIT_FAILED:
+            if self._poll_to_confirm(itask,
+                                     TASK_STATUS_SUBMIT_FAILED, poll_func):
+                return
             self._process_message_submit_failed(itask, event_time)
         elif message == TASK_OUTPUT_SUBMITTED:
+            if self._poll_to_confirm(itask, TASK_STATUS_SUBMITTED, poll_func):
+                return
             self._process_message_submitted(itask, event_time)
         elif message.startswith(TaskMessage.FAIL_MESSAGE_PREFIX):
-            # Task failed message
+            # Task received signal.
             signal = message[len(TaskMessage.FAIL_MESSAGE_PREFIX):]
             self._db_events_insert(itask, "signaled", signal)
             self.suite_db_mgr.put_update_task_jobs(
                 itask, {"run_signal": signal})
+            if self._poll_to_confirm(itask, TASK_STATUS_FAILED, poll_func):
+                return
             self._process_message_failed(itask, event_time, self.JOB_FAILED)
         elif message.startswith(TaskMessage.ABORT_MESSAGE_PREFIX):
             # Task aborted with message
-            signal = message[len(TaskMessage.ABORT_MESSAGE_PREFIX):]
+            aborted_with = message[len(TaskMessage.ABORT_MESSAGE_PREFIX):]
             self._db_events_insert(itask, "aborted", message)
             self.suite_db_mgr.put_update_task_jobs(
-                itask, {"run_signal": signal})
-            self._process_message_failed(itask, event_time, signal)
+                itask, {"run_signal": aborted_with})
+            if self._poll_to_confirm(itask, TASK_STATUS_FAILED, poll_func):
+                return
+            self._process_message_failed(itask, event_time, aborted_with)
         elif message.startswith(TaskMessage.VACATION_MESSAGE_PREFIX):
             # Task job pre-empted into a vacation state
-            cylc.flags.pflag = True
-            itask.state.reset_state(TASK_STATUS_SUBMITTED)
             self._db_events_insert(itask, "vacated", message)
             itask.set_event_time('started')  # reset
             if TASK_STATUS_SUBMIT_RETRYING in itask.try_timers:
@@ -362,25 +360,31 @@ class TaskEventsManager(object):
                     float(self._get_events_conf(itask, 'submission timeout')))
             except (TypeError, ValueError):
                 itask.timeout_timers[TASK_STATUS_SUBMITTED] = None
-        elif output_has_changed:
-            # Message of a custom task output
-            cylc.flags.pflag = True
+            # Believe this and change state without polling (could poll?).
+            self.pflag = True
+            itask.state.reset_state(TASK_STATUS_SUBMITTED)
+        elif an_output_was_satisfied:
+            # Message of an as-yet unreported custom task output.
+            # No state change.
+            self.pflag = True
             self.suite_db_mgr.put_update_task_outputs(itask)
         else:
             # Unhandled messages. These include:
             #  * general non-output/progress messages
             #  * poll messages that repeat previous results
             # Note that all messages are logged already at the top.
+            # No state change.
             LOG.debug(
                 '(current: %s) unhandled: %s' % (itask.state.status, message),
                 itask=itask)
-            if priority in [CRITICAL, ERROR, WARNING, INFO, DEBUG]:
-                priority = getLevelName(priority)
+            if severity in [CRITICAL, ERROR, WARNING, INFO, DEBUG]:
+                severity = getLevelName(severity)
             self._db_events_insert(
-                itask, ("message %s" % str(priority).lower()), message)
-
-        if priority in [TaskMessage.WARNING, TaskMessage.CRITICAL]:
-            self.setup_event_handlers(itask, priority.lower(), message)
+                itask, ("message %s" % str(severity).lower()), message)
+        if severity == "CUSTOM":
+            self.setup_event_handlers(itask, "custom", message)
+        elif severity in [TaskMessage.WARNING, TaskMessage.CRITICAL]:
+            self.setup_event_handlers(itask, severity.lower(), message)
 
     def setup_event_handlers(self, itask, event, message):
         """Set up handlers for a task event."""
@@ -394,7 +398,8 @@ class TaskEventsManager(object):
         self._setup_event_mail(itask, event)
         self._setup_custom_event_handlers(itask, event, message)
 
-    def set_poll_time(self, itask, now=None):
+    @staticmethod
+    def set_poll_time(itask, now=None):
         """Set the next task execution/submission poll time.
 
         If now is set, set the timer only if the previous delay is done.
@@ -441,7 +446,7 @@ class TaskEventsManager(object):
             subject = "[%s/%s/%02d %s] %s" % (
                 point, name, submit_num, event, schd_ctx.suite)
         else:
-            event_set = set([id_key[0][1] for id_key in id_keys])
+            event_set = set(id_key[0][1] for id_key in id_keys)
             if len(event_set) == 1:
                 # 1 event from n tasks
                 subject = "[%d tasks %s] %s" % (
@@ -503,11 +508,10 @@ class TaskEventsManager(object):
                 if cylc.flags.debug:
                     ERR.debug(traceback.format_exc())
 
-    @staticmethod
-    def _get_events_conf(itask, key, default=None):
+    def _get_events_conf(self, itask, key, default=None):
         """Return an events setting from suite then global configuration."""
         for getter in [
-                BroadcastServer.get_inst().get(itask.identity).get("events"),
+                self.broadcast_mgr.get_broadcast(itask.identity).get("events"),
                 itask.tdef.rtconfig["events"],
                 GLOBAL_CFG.get()["task events"]]:
             try:
@@ -600,7 +604,7 @@ class TaskEventsManager(object):
         if (TASK_STATUS_RETRYING not in itask.try_timers or
                 itask.try_timers[TASK_STATUS_RETRYING].next() is None):
             # No retry lined up: definitive failure.
-            cylc.flags.pflag = True
+            self.pflag = True
             itask.state.reset_state(TASK_STATUS_FAILED)
             self.setup_event_handlers(itask, "failed", message)
             LOG.critical("job(%02d) %s" % (
@@ -623,7 +627,7 @@ class TaskEventsManager(object):
         if itask.job_vacated:
             itask.job_vacated = False
             LOG.warning("Vacated job restarted", itask=itask)
-        cylc.flags.pflag = True
+        self.pflag = True
         itask.state.reset_state(TASK_STATUS_RUNNING)
         itask.set_event_time('started', event_time)
         self.suite_db_mgr.put_update_task_jobs(itask, {
@@ -647,7 +651,7 @@ class TaskEventsManager(object):
 
     def _process_message_succeeded(self, itask, event_time):
         """Helper for process_message, handle a succeeded message."""
-        cylc.flags.pflag = True
+        self.pflag = True
         itask.set_event_time('finished', event_time)
         self.suite_db_mgr.put_update_task_jobs(itask, {
             "run_status": 0,
@@ -675,15 +679,12 @@ class TaskEventsManager(object):
             "time_submit_exit": get_current_time_string(),
             "submit_status": 1,
         })
-        try:
-            del itask.summary['submit_method_id']
-        except KeyError:
-            pass
+        itask.summary['submit_method_id'] = None
         if (TASK_STATUS_SUBMIT_RETRYING not in itask.try_timers or
                 itask.try_timers[TASK_STATUS_SUBMIT_RETRYING].next() is None):
             # No submission retry lined up: definitive failure.
             itask.set_event_time('finished', event_time)
-            cylc.flags.pflag = True
+            self.pflag = True
             # See github #476.
             self.setup_event_handlers(
                 itask, self.EVENT_SUBMIT_FAILED,
@@ -722,7 +723,7 @@ class TaskEventsManager(object):
             # Simulate job execution at this point.
             itask.set_event_time('started', event_time)
             itask.state.reset_state(TASK_STATUS_RUNNING)
-            itask.state.outputs.set_completed(TASK_OUTPUT_STARTED)
+            itask.state.outputs.set_completion(TASK_OUTPUT_STARTED, True)
             return
 
         itask.set_event_time('submitted', event_time)
@@ -732,7 +733,7 @@ class TaskEventsManager(object):
         self.setup_event_handlers(
             itask, TASK_OUTPUT_SUBMITTED, 'job submitted')
 
-        cylc.flags.pflag = True
+        self.pflag = True
         if itask.state.status == TASK_STATUS_READY:
             # In rare occassions, the submit command of a batch system has sent
             # the job to its server, and the server has started the job before
@@ -796,7 +797,7 @@ class TaskEventsManager(object):
                 self._get_events_conf(  # mail_from
                     itask,
                     "mail from",
-                    "notifications@" + get_suite_host(),
+                    "notifications@" + get_host(),
                 ),
                 self._get_events_conf(itask, "mail to", get_user()),  # mail_to
                 self._get_events_conf(itask, "mail smtp"),  # mail_smtp
@@ -826,6 +827,11 @@ class TaskEventsManager(object):
                 continue
             # Custom event handler can be a command template string
             # or a command that takes 4 arguments (classic interface)
+            # Note quote() fails on None, need str(None).
+            user_at_host = itask.summary['job_hosts'][itask.submit_num]
+            if '@' not in user_at_host:
+                # (only has 'user@' on the front if user is not suite owner).
+                user_at_host = '%s@%s' % (get_user(), user_at_host)
             try:
                 handler_data = {
                     "event": quote(event),
@@ -835,6 +841,17 @@ class TaskEventsManager(object):
                     "submit_num": itask.submit_num,
                     "id": quote(itask.identity),
                     "message": quote(message),
+                    "batch_sys_name": quote(
+                        str(itask.summary['batch_sys_name'])),
+                    "batch_sys_job_id": quote(
+                        str(itask.summary['submit_method_id'])),
+                    "submit_time": quote(
+                        str(itask.summary['submitted_time_string'])),
+                    "start_time": quote(
+                        str(itask.summary['started_time_string'])),
+                    "finish_time": quote(
+                        str(itask.summary['finished_time_string'])),
+                    "user@host": quote(user_at_host)
                 }
 
                 if self.suite_cfg:
