@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -46,17 +46,18 @@ class TaskMessage(object):
     NORMAL = "NORMAL"
     WARNING = "WARNING"
     CRITICAL = "CRITICAL"
-    PRIORITIES = (NORMAL, WARNING, CRITICAL)
+    CUSTOM = "CUSTOM"
+    SEVERITIES = (NORMAL, WARNING, CRITICAL, CUSTOM)
 
     MSG_RETRY_INTVL = 5.0
     MSG_MAX_TRIES = 7
     MSG_TIMEOUT = 30.0
 
-    def __init__(self, priority=NORMAL):
-        if priority in self.PRIORITIES:
-            self.priority = priority
+    def __init__(self, severity=NORMAL):
+        if severity in self.SEVERITIES:
+            self.severity = severity
         else:
-            raise ValueError('Illegal message priority ' + priority)
+            raise ValueError('Illegal message severity ' + severity)
 
         # load the environment
         self.env_map = dict(os.environ)
@@ -84,26 +85,26 @@ class TaskMessage(object):
                 import traceback
                 traceback.print_exc()
             return
-        if (self.env_map.get('CYLC_TASK_COMMS_METHOD') == 'ssh' and
-                self._send_by_ssh()):
-            return
-        self._send_by_remote_port(messages)
+        if self.env_map.get('CYLC_TASK_COMMS_METHOD') == 'ssh':
+            self._send_by_ssh()
+        else:
+            self._send_by_remote_port(messages)
 
     def _print_messages(self, messages):
         """Print message to send."""
-        if self.priority == self.NORMAL:
+        if self.severity in [self.NORMAL, self.CUSTOM]:
             handle = sys.stdout
         else:
             handle = sys.stderr
         for message in messages:
             handle.write('%s %s - %s\n' % (
-                self.true_event_time, self.priority, message))
+                self.true_event_time, self.severity, message))
         handle.flush()
 
     def _send_by_remote_port(self, messages):
         """Send message by talking to the daemon (remote?) port."""
-        from cylc.network import ConnectionError, ConnectionInfoError
-        from cylc.network.task_msg_client import TaskMessageClient
+        from cylc.network.httpclient import (
+            SuiteRuntimeServiceClient, ClientError, ClientInfoError)
 
         # Convert time/duration into appropriate units
         retry_intvl = float(self.env_map.get(
@@ -113,7 +114,7 @@ class TaskMessage(object):
             SuiteSrvFilesManager.KEY_TASK_MSG_MAX_TRIES,
             self.MSG_MAX_TRIES))
 
-        client = TaskMessageClient(
+        client = SuiteRuntimeServiceClient(
             self.suite,
             owner=self.env_map.get(SuiteSrvFilesManager.KEY_OWNER),
             host=self.env_map.get(SuiteSrvFilesManager.KEY_HOST),
@@ -125,17 +126,19 @@ class TaskMessage(object):
         for i in range(1, max_tries + 1):  # 1..max_tries inclusive
             try:
                 for message in messages:
-                    client.put(self.task_id, self.priority, message)
-            except ConnectionError as exc:
-                sys.stderr.write("Send message: try %s of %s failed: %s\n" % (
-                    i, max_tries, exc))
+                    client.put_message(self.task_id, self.severity, message)
+            except ClientError as exc:
+                sys.stderr.write(
+                    "%s WARNING - Send message: try %s of %s failed: %s\n" % (
+                        get_current_time_string(), i, max_tries, exc))
                 # Break if:
                 # * Exhausted number of tries.
                 # * Contact info file not found, suite probably not running.
                 #   Don't bother with retry, suite restart will poll any way.
-                if i >= max_tries or isinstance(exc, ConnectionInfoError):
+                if i >= max_tries or isinstance(exc, ClientInfoError):
                     # Issue a warning and let the task carry on
-                    sys.stderr.write("WARNING: MESSAGE SEND FAILED\n")
+                    sys.stderr.write("%s WARNING - MESSAGE SEND FAILED\n" % (
+                        get_current_time_string()))
                 else:
                     sys.stderr.write(
                         "   retry in %s seconds, timeout is %s\n" % (
@@ -150,8 +153,8 @@ class TaskMessage(object):
                     # Continue to write to STDERR, so users can easily see that
                     # it has recovered from previous failures.
                     sys.stderr.write(
-                        "Send message: try %s of %s succeeded\n" % (
-                            i, max_tries))
+                        "%s INFO - Send message: try %s of %s succeeded\n" % (
+                            get_current_time_string(), i, max_tries))
                 break
 
     def _send_by_ssh(self):
@@ -182,16 +185,17 @@ class TaskMessage(object):
         # this code block.
         env = {}
         for var in [
-                'CYLC_MODE', 'CYLC_TASK_ID', 'CYLC_VERBOSE',
-                'CYLC_SUITE_RUN_DIR',
-                'CYLC_SUITE_RUN_DIR_ON_SUITE_HOST',
-                'CYLC_SUITE_NAME', 'CYLC_SUITE_OWNER',
-                'CYLC_SUITE_HOST', 'CYLC_SUITE_PORT', 'CYLC_UTC',
-                'CYLC_TASK_MSG_MAX_TRIES', 'CYLC_TASK_MSG_TIMEOUT',
-                'CYLC_TASK_MSG_RETRY_INTVL']:
+                SuiteSrvFilesManager.KEY_NAME,
+                SuiteSrvFilesManager.KEY_DIR_ON_SUITE_HOST,
+                'CYLC_TASK_ID',
+                'CYLC_UTC',
+                'CYLC_VERBOSE']:
             # (no exception handling here as these variables should
             # always be present in the task execution environment)
-            env[var] = self.env_map.get(var, 'UNSET')
+            try:
+                env[var] = self.env_map[var]
+            except KeyError:
+                pass
 
         # The path to cylc/bin on the remote end may be required:
         path = os.path.join(self.env_map['CYLC_DIR_ON_SUITE_HOST'], 'bin')
@@ -200,10 +204,7 @@ class TaskMessage(object):
         # otherwise drop through to local messaging.
         # Note: do not sys.exit(0) here as the commands do, it
         # will cause messaging failures on the remote host.
-        try:
-            return remrun().execute(env=env, path=[path])
-        except SystemExit:
-            return
+        remrun(env=env, path=[path])
 
     def _update_job_status_file(self, messages):
         """Write messages to job status file."""
@@ -260,11 +261,11 @@ class TaskMessage(object):
                 for line in lines:
                     job_status_file.write(line)
                 job_status_file.write("%s=%s|%s|%s\n" % (
-                    self.CYLC_MESSAGE, self.true_event_time, self.priority,
+                    self.CYLC_MESSAGE, self.true_event_time, self.severity,
                     message))
             else:
                 job_status_file.write("%s=%s|%s|%s\n" % (
-                    self.CYLC_MESSAGE, self.true_event_time, self.priority,
+                    self.CYLC_MESSAGE, self.true_event_time, self.severity,
                     message))
         try:
             job_status_file.close()
