@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -57,7 +57,7 @@ class TaskOutputs(object):
     # Memory optimization - constrain possible attributes to this list.
     __slots__ = ["_by_message", "_by_trigger"]
 
-    def __init__(self, tdef, point):
+    def __init__(self, tdef):
         self._by_message = {}
         self._by_trigger = {}
         for trigger, message in tdef.outputs:
@@ -72,7 +72,7 @@ class TaskOutputs(object):
 
     def all_completed(self):
         """Return True if all all outputs completed."""
-        return all([val[_IS_COMPLETED] for val in self._by_message.values()])
+        return all(val[_IS_COMPLETED] for val in self._by_message.values())
 
     def exists(self, message=None, trigger=None):
         """Return True if message/trigger is identified as an output."""
@@ -143,18 +143,25 @@ class TaskOutputs(object):
         for value in self._by_message.values():
             value[_IS_COMPLETED] = False
 
-    def set_completed(self, message=None, trigger=None, is_completed=True):
-        """Set the output identified by message/trigger as completed.
+    def set_completion(self, message, is_completed):
+        """Set output message completion status to is_completed (bool)."""
+        if message in self._by_message:
+            self._by_message[message][_IS_COMPLETED] = is_completed
+
+    def set_msg_trg_completion(self, message=None, trigger=None,
+                               is_completed=True):
+        """Set the output identified by message/trigger to is_completed.
 
         Return True if completion flag is changed, False if completion is
         unchanged, or None if message/trigger is not found.
+
         """
         try:
             item = self._get_item(message, trigger)
             old_is_completed = item[_IS_COMPLETED]
             item[_IS_COMPLETED] = is_completed
         except KeyError:
-            pass
+            return None
         else:
             return bool(old_is_completed) != bool(is_completed)
 
