@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,12 +37,9 @@ from time import time
 import traceback
 
 from cylc.config import SuiteConfigError
-from cylc.cycling.loader import (
-    get_interval, get_interval_cls, get_point, ISO8601_CYCLING_TYPE,
-    standardise_point_string)
+from cylc.cycling.loader import get_point, standardise_point_string
 import cylc.flags
-from cylc.network.ext_trigger_server import ExtTriggerServer
-from cylc.suite_logging import ERR, LOG, OUT
+from cylc.suite_logging import ERR, LOG
 from cylc.task_action_timer import TaskActionTimer
 from cylc.task_id import TaskID
 from cylc.task_proxy import TaskProxy
@@ -237,9 +234,9 @@ class TaskPool(object):
 
         Return True if any tasks are released, else False.
         """
-
+        released = False
         if not self.runahead_pool:
-            return False
+            return released
 
         # Any finished tasks can be released immediately (this can happen at
         # restart when all tasks are initially loaded into the runahead pool).
@@ -249,7 +246,7 @@ class TaskPool(object):
                                           TASK_STATUS_SUCCEEDED,
                                           TASK_STATUS_EXPIRED]:
                     self.release_runahead_task(itask)
-                    self.rhpool_changed = True
+                    released = True
 
         limit = self.max_num_active_cycle_points
 
@@ -321,7 +318,6 @@ class TaskPool(object):
         if latest_allowed_point > self.stop_point:
             latest_allowed_point = self.stop_point
 
-        released = False
         for point, itask_id_map in self.runahead_pool.copy().items():
             if point <= latest_allowed_point:
                 for itask in itask_id_map.copy().values():
@@ -351,7 +347,7 @@ class TaskPool(object):
         are polled to determine what their true status is.
         """
         if row_idx == 0:
-            OUT.info("LOADING task proxies")
+            LOG.info("LOADING task proxies")
         (cycle, name, spawned, status, hold_swap, submit_num, _,
          user_at_host, time_submit, time_run, timeout,
          outputs_str) = row
@@ -413,23 +409,23 @@ class TaskPool(object):
                     TASK_STATUS_SUCCEEDED]:
                 try:
                     for output in outputs_str.splitlines():
-                        itask.state.outputs.set_completed(
-                            message=output.split("=", 1)[1])
+                        itask.state.outputs.set_completion(
+                            output.split("=", 1)[1], True)
                 except AttributeError:
                     pass
 
             if user_at_host:
                 itask.summary['job_hosts'][int(submit_num)] = user_at_host
             if hold_swap:
-                OUT.info("+ %s.%s %s (%s)" % (name, cycle, status, hold_swap))
+                LOG.info("+ %s.%s %s (%s)" % (name, cycle, status, hold_swap))
             else:
-                OUT.info("+ %s.%s %s" % (name, cycle, status))
+                LOG.info("+ %s.%s %s" % (name, cycle, status))
             self.add_to_runahead_pool(itask, is_restart=True)
 
     def load_db_task_action_timers(self, row_idx, row):
         """Load a task action timer, e.g. event handlers, retry states."""
         if row_idx == 0:
-            OUT.info("LOADING task action timers")
+            LOG.info("LOADING task action timers")
         (cycle, name, ctx_key_pickle, ctx_pickle, delays_pickle, num, delay,
          timeout) = row
         id_ = TaskID.get(name, cycle)
@@ -456,7 +452,7 @@ class TaskPool(object):
                 {"id": id_, "ctx_key": ctx_key})
             ERR.warning(traceback.format_exc())
             return
-        OUT.info("+ %s.%s %s" % (name, cycle, ctx_key))
+        LOG.info("+ %s.%s %s" % (name, cycle, ctx_key))
 
     def release_runahead_task(self, itask):
         """Release itask to the appropriate queue in the active pool."""
@@ -469,7 +465,6 @@ class TaskPool(object):
         self.pool.setdefault(itask.point, {})
         self.pool[itask.point][itask.identity] = itask
         self.pool_changed = True
-        cylc.flags.pflag = True
         LOG.debug("released to the task pool", itask=itask)
         del self.runahead_pool[itask.point][itask.identity]
         if not self.runahead_pool[itask.point]:
@@ -621,30 +616,11 @@ class TaskPool(object):
         """Check for future triggers extending beyond the final cycle."""
         if not self.stop_point:
             return False
-        for pct in set(itask.state.prerequisites_get_target_points()):
+        for pct in itask.state.prerequisites_get_target_points():
             if pct > self.stop_point:
                 return True
         return False
 
-    def set_runahead(self, interval=None):
-        """Set the runahead."""
-        if isinstance(interval, int) or isinstance(interval, basestring):
-            # The unit is assumed to be hours (backwards compatibility).
-            interval = str(interval)
-            interval_cls = get_interval_cls()
-            if interval_cls.TYPE == ISO8601_CYCLING_TYPE:
-                interval = get_interval("PT%sH" % interval)
-            else:
-                interval = get_interval(interval)
-        if interval is None:
-            # No limit
-            LOG.warning("setting NO custom runahead limit")
-            self.custom_runahead_limit = None
-        else:
-            LOG.info("setting custom runahead limit to %s" % interval)
-            self.custom_runahead_limit = interval
-        self.release_runahead_tasks()
-
     def get_min_point(self):
         """Return the minimum cycle point currently in the pool."""
         cycles = self.pool.keys()
@@ -679,7 +655,7 @@ class TaskPool(object):
                 max_offset = itask.tdef.max_future_prereq_offset
         self.max_future_offset = max_offset
 
-    def reconfigure(self, config, stop_point):
+    def set_do_reload(self, config, stop_point):
         """Set the task pool to reload mode."""
         self.config = config
         self.do_reload = True
@@ -720,7 +696,7 @@ class TaskPool(object):
         LOG.info("Reloading task definitions.")
         # Log tasks orphaned by a reload that were not in the task pool.
         for task in self.orphans:
-            if task not in [tsk.tdef.name for tsk in self.get_all_tasks()]:
+            if task not in (tsk.tdef.name for tsk in self.get_all_tasks()):
                 LOG.warning("Removed task: '%s'" % (task,))
         for itask in self.get_all_tasks():
             if itask.tdef.name in self.orphans:
@@ -804,10 +780,13 @@ class TaskPool(object):
         """Return True if the suite is stalled.
 
         A suite is stalled when:
+        * It is not held.
         * It has no active tasks.
         * It has waiting tasks with unmet prerequisites
           (ignoring clock triggers).
         """
+        if self.is_held:
+            return False
         can_be_stalled = False
         for itask in self.get_tasks():
             if itask.point > self.stop_point or itask.state.status in [
@@ -866,10 +845,6 @@ class TaskPool(object):
             for prereq in prereqs:
                 LOG.warning(" * %s" % prereq)
 
-    def get_hold_point(self):
-        """Return the point after which tasks must be held."""
-        return self.hold_point
-
     def set_hold_point(self, point):
         """Set the point after which tasks must be held."""
         self.hold_point = point
@@ -928,16 +903,16 @@ class TaskPool(object):
         outputs. Brokered negotiation is O(n) in number of tasks.
 
         """
-        all_outputs = {}   # all_outputs[message] = taskid
+        all_task_outputs = set()
         for itask in self.get_tasks():
-            for message in itask.state.outputs.get_completed():
-                all_outputs["%s %s" % (itask.identity, message)] = (
-                    itask.identity)
-        all_output_msgs = set(all_outputs)
+            for output in itask.state.outputs.get_completed():
+                all_task_outputs.add((itask.tdef.name,
+                                      str(itask.point),
+                                      output))
         for itask in self.get_tasks():
             # Try to satisfy itask if not already satisfied.
             if itask.state.prerequisites_are_not_all_satisfied():
-                itask.state.satisfy_me(all_output_msgs, all_outputs)
+                itask.state.satisfy_me(all_task_outputs)
 
     def force_spawn(self, itask):
         """Spawn successor of itask."""
@@ -988,7 +963,7 @@ class TaskPool(object):
         num_removed = 0
         for itask in self.get_tasks():
             if itask.state.suicide_prerequisites:
-                if itask.state.suicide_prerequisites_are_all_satisfied():
+                if itask.state.suicide_prerequisites_satisfied():
                     if itask.state.status in [TASK_STATUS_READY,
                                               TASK_STATUS_SUBMITTED,
                                               TASK_STATUS_RUNNING]:
@@ -1093,10 +1068,10 @@ class TaskPool(object):
                         itask.state.outputs.set_all_incomplete()
                         LOG.info("reset all output to incomplete", itask=itask)
                     else:
-                        ret = itask.state.outputs.set_completed(
+                        ret = itask.state.outputs.set_msg_trg_completion(
                             message=output, is_completed=is_completed)
                         if ret is None:
-                            ret = itask.state.outputs.set_completed(
+                            ret = itask.state.outputs.set_msg_trg_completion(
                                 trigger=output, is_completed=is_completed)
                         if ret is None:
                             LOG.warning(
@@ -1122,11 +1097,16 @@ class TaskPool(object):
             self.remove(itask, 'by request')
         return len(bad_items)
 
-    def trigger_tasks(self, items):
+    def trigger_tasks(self, items, back_out=False):
         """Trigger tasks."""
         itasks, bad_items = self.filter_task_proxies(items)
         n_warnings = len(bad_items)
         for itask in itasks:
+            if back_out:
+                # (Aborted edit-run, reset for next trigger attempt).
+                itask.submit_num -= 1
+                itask.local_job_file_path = None
+                continue
             if itask.state.status in TASK_STATUSES_ACTIVE:
                 LOG.warning('%s: already triggered' % itask.identity)
                 n_warnings += 1
@@ -1170,13 +1150,13 @@ class TaskPool(object):
                         (itask.get_try_num() == 1 or
                          not conf['fail try 1 only'])):
                     message_queue.put(
-                        itask.identity, 'CRITICAL', TASK_STATUS_FAILED)
+                        (itask.identity, 'CRITICAL', TASK_STATUS_FAILED))
                 else:
                     # Simulate message outputs.
                     for msg in itask.tdef.rtconfig['outputs'].values():
-                        message_queue.put(itask.identity, 'NORMAL', msg)
+                        message_queue.put((itask.identity, 'NORMAL', msg))
                     message_queue.put(
-                        itask.identity, 'NORMAL', TASK_STATUS_SUCCEEDED)
+                        (itask.identity, 'NORMAL', TASK_STATUS_SUCCEEDED))
                 sim_task_state_changed = True
         return sim_task_state_changed
 
@@ -1248,7 +1228,7 @@ class TaskPool(object):
         Result in a dict of a dict:
         {
             "task_id": {
-                "descriptions": {key: value, ...},
+                "meta": {key: value, ...},
                 "prerequisites": {key: value, ...},
                 "outputs": {key: value, ...},
                 "extras": {key: value, ...},
@@ -1283,19 +1263,12 @@ class TaskPool(object):
             for _, msg, is_completed in itask.state.outputs.get_all():
                 outputs.append(["%s %s" % (itask.identity, msg), is_completed])
             results[itask.identity] = {
-                "descriptions": itask.tdef.describe(),
+                "meta": itask.tdef.describe(),
                 "prerequisites": itask.state.prerequisites_dump(),
                 "outputs": outputs,
                 "extras": extras}
         return results, bad_items
 
-    def match_ext_triggers(self):
-        """See if any queued external event messages can trigger tasks."""
-        ets = ExtTriggerServer.get_inst()
-        for itask in self.get_tasks():
-            if itask.state.external_triggers:
-                ets.retrieve(itask)
-
     def filter_task_proxies(self, items):
         """Return task proxies that match names, points, states in items.
 
