@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -69,12 +69,10 @@ class TaskProxy(object):
             self.point = min(adjusted)
         else:
             self.point = start_point
-        self.cleanup_cutoff = self.tdef.get_cleanup_cutoff_point(
-            self.point, self.tdef.intercycle_offsets)
+        self.cleanup_cutoff = self.tdef.get_cleanup_cutoff_point(self.point)
         self.identity = TaskID.get(self.tdef.name, self.point)
 
         self.has_spawned = has_spawned
-
         self.point_as_seconds = None
 
         # Manually inserted tasks may have a final cycle point set.
@@ -98,6 +96,8 @@ class TaskProxy(object):
             'logfiles': [],
             'job_hosts': {},
             'execution_time_limit': None,
+            'batch_sys_name': None,
+            'submit_method_id': None
         }
 
         self.local_job_file_path = None
