@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -77,6 +77,9 @@ TASK_STATUSES_RESTRICTED = set([
 
 # Task statuses we can manually reset a task TO.
 TASK_STATUSES_CAN_RESET_TO = set([
+    TASK_STATUS_SUBMITTED,
+    TASK_STATUS_SUBMIT_FAILED,
+    TASK_STATUS_RUNNING,
     TASK_STATUS_WAITING,
     TASK_STATUS_HELD,
     TASK_STATUS_READY,
@@ -101,7 +104,7 @@ TASK_STATUSES_TO_BE_ACTIVE = set([
     TASK_STATUS_RETRYING,
 ])
 
-# Task statuses that are externally active, i.e. pollable and killable
+# Task statuses that are externally active
 TASK_STATUSES_ACTIVE = set([
     TASK_STATUS_SUBMITTED,
     TASK_STATUS_RUNNING,
@@ -160,7 +163,7 @@ class TaskState(object):
     __slots__ = ["identity", "status", "hold_swap",
                  "_is_satisfied", "_suicide_is_satisfied", "prerequisites",
                  "suicide_prerequisites", "external_triggers", "outputs",
-                 "kill_failed", "time_updated"]
+                 "kill_failed", "time_updated", "confirming_with_poll"]
 
     def __init__(self, tdef, point, status, hold_swap):
         self.identity = TaskID.get(tdef.name, str(point))
@@ -186,7 +189,7 @@ class TaskState(object):
             self.external_triggers[ext] = False
 
         # Message outputs.
-        self.outputs = TaskOutputs(tdef, point)
+        self.outputs = TaskOutputs(tdef)
 
         # Standard outputs.
         self.outputs.add(TASK_OUTPUT_SUBMITTED)
@@ -194,12 +197,13 @@ class TaskState(object):
         self.outputs.add(TASK_OUTPUT_SUCCEEDED)
 
         self.kill_failed = False
+        self.confirming_with_poll = False
 
-    def satisfy_me(self, task_output_msgs, task_outputs):
+    def satisfy_me(self, all_task_outputs):
         """Attempt to get my prerequisites satisfied."""
-        for preqs in [self.prerequisites, self.suicide_prerequisites]:
-            for preq in preqs:
-                if preq.satisfy_me(task_output_msgs, task_outputs):
+        for prereqs in [self.prerequisites, self.suicide_prerequisites]:
+            for prereq in prereqs:
+                if prereq.satisfy_me(all_task_outputs):
                     self._is_satisfied = None
                     self._suicide_is_satisfied = None
 
@@ -215,6 +219,12 @@ class TaskState(object):
         return (not self.prerequisites_are_all_satisfied() or
                 not self.suicide_prerequisites_are_all_satisfied())
 
+    def suicide_prerequisites_satisfied(self):
+        """Return True if any suicide prerequisites are satisfied."""
+        if self._suicide_is_satisfied is True:
+            return True
+        return any(preq.is_satisfied() for preq in self.suicide_prerequisites)
+
     def suicide_prerequisites_are_all_satisfied(self):
         """Return True if all suicide prerequisites are satisfied."""
         if self._suicide_is_satisfied is None:
@@ -224,10 +234,8 @@ class TaskState(object):
 
     def prerequisites_get_target_points(self):
         """Return a list of cycle points targeted by each prerequisite."""
-        points = []
-        for preq in self.prerequisites:
-            points += preq.get_target_points()
-        return points
+        return set(point for prerequisite in self.prerequisites for
+                   point in prerequisite.get_target_points())
 
     def prerequisites_eval_all(self):
         """Set all prerequisites to satisfied."""
@@ -250,26 +258,23 @@ class TaskState(object):
 
     def prerequisites_dump(self, list_prereqs=False):
         """Dump prerequisites."""
-        res = []
         if list_prereqs:
-            for prereq in self.prerequisites:
-                for task in sorted(prereq.messages):
-                    res.append(task)
+            return [Prerequisite.MESSAGE_TEMPLATE % msg for prereq in
+                    self.prerequisites for msg in sorted(prereq.satisfied)]
         else:
-            for preq in self.prerequisites:
-                res += preq.dump()
-        return res
+            return [x for prereq in self.prerequisites for x in prereq.dump()]
 
     def get_resolved_dependencies(self):
-        """Report who I triggered off."""
-        satby = {}
-        for req in self.prerequisites:
-            satby.update(req.satisfied_by)
-        dep = satby.values()
-        # order does not matter here; sort to allow comparison with
-        # reference run task with lots of near-simultaneous triggers.
-        dep.sort()
-        return dep
+        """Return a list of dependencies which have been met for this task.
+
+        E.G: ['foo.1', 'bar.2']
+
+        The returned list is sorted to allow comparison with reference run
+        task with lots of near-simultaneous triggers.
+
+        """
+        return list(sorted(dep for prereq in self.prerequisites for dep in
+                           prereq.get_resolved_dependencies()))
 
     def unset_special_outputs(self):
         """Remove special outputs added for triggering purposes.
@@ -325,11 +330,19 @@ class TaskState(object):
             self.outputs.set_all_incomplete()
         elif status == TASK_STATUS_SUBMITTED:
             self.set_prerequisites_all_satisfied()
-            self.outputs.set_completed(TASK_OUTPUT_SUBMITTED)
+            self.outputs.set_completion(TASK_OUTPUT_SUBMITTED, True)
+            # In case of manual reset, set final outputs incomplete (but assume
+            # completed message outputs remain completed).
+            self.outputs.set_completion(TASK_OUTPUT_SUCCEEDED, False)
+            self.outputs.set_completion(TASK_OUTPUT_FAILED, False)
         elif status == TASK_STATUS_RUNNING:
             self.set_prerequisites_all_satisfied()
-            self.outputs.set_completed(TASK_OUTPUT_SUBMITTED)
-            self.outputs.set_completed(TASK_OUTPUT_STARTED)
+            self.outputs.set_completion(TASK_OUTPUT_SUBMITTED, True)
+            self.outputs.set_completion(TASK_OUTPUT_STARTED, True)
+            # In case of manual reset, set final outputs incomplete (but assume
+            # completed message outputs remain completed).
+            self.outputs.set_completion(TASK_OUTPUT_SUCCEEDED, False)
+            self.outputs.set_completion(TASK_OUTPUT_FAILED, False)
         elif status == TASK_STATUS_SUBMIT_RETRYING:
             self.set_prerequisites_all_satisfied()
             self.outputs.remove(TASK_OUTPUT_SUBMITTED)
@@ -340,9 +353,9 @@ class TaskState(object):
         elif status == TASK_STATUS_SUCCEEDED:
             self.set_prerequisites_all_satisfied()
             self.unset_special_outputs()
-            self.outputs.set_completed(TASK_OUTPUT_SUBMITTED)
-            self.outputs.set_completed(TASK_OUTPUT_STARTED)
-            self.outputs.set_completed(TASK_OUTPUT_SUCCEEDED)
+            self.outputs.set_completion(TASK_OUTPUT_SUBMITTED, True)
+            self.outputs.set_completion(TASK_OUTPUT_STARTED, True)
+            self.outputs.set_completion(TASK_OUTPUT_SUCCEEDED, True)
         elif status == TASK_STATUS_RETRYING:
             self.set_prerequisites_all_satisfied()
             self.outputs.set_all_incomplete()
@@ -419,8 +432,7 @@ class TaskState(object):
             if adjusted:
                 p_prev = max(adjusted)
                 cpre = Prerequisite(point, tdef.start_point)
-                prereq = "%s %s" % (TaskID.get(tdef.name, p_prev),
-                                    TASK_STATUS_SUCCEEDED)
-                cpre.add(prereq, p_prev < tdef.start_point)
+                cpre.add(tdef.name, p_prev, TASK_STATUS_SUCCEEDED,
+                         p_prev < tdef.start_point)
                 cpre.set_condition(tdef.name)
                 self.prerequisites.append(cpre)
