@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@ class TaskDef(object):
     # Memory optimization - constrain possible attributes to this list.
     __slots__ = [
         "MAX_LEN_ELAPSED_TIMES", "run_mode", "rtconfig", "start_point",
-        "spawn_ahead", "sequences", "implicit_sequences",
+        "spawn_ahead", "sequences",
         "used_in_offset_trigger", "max_future_prereq_offset",
         "intercycle_offsets", "sequential", "is_coldstart",
         "suite_polling_cfg", "clocktrigger_offset", "expiration_offset",
@@ -62,7 +62,6 @@ class TaskDef(object):
         self.spawn_ahead = spawn_ahead
 
         self.sequences = []
-        self.implicit_sequences = []  # Implicit sequences are deprecated.
         self.used_in_offset_trigger = False
 
         # some defaults
@@ -95,19 +94,14 @@ class TaskDef(object):
             self.dependencies[sequence] = []
         self.dependencies[sequence].append(dependency)
 
-    def add_sequence(self, sequence, is_implicit=False):
+    def add_sequence(self, sequence):
         """Add a sequence."""
         if sequence not in self.sequences:
             self.sequences.append(sequence)
-            if is_implicit:
-                self.implicit_sequences.append(sequence)
 
     def describe(self):
         """Return title and description of the current task."""
-        info = {}
-        for item in 'title', 'description':
-            info[item] = self.rtconfig['meta'][item]
-        return info
+        return self.rtconfig['meta']
 
     def check_for_explicit_cycling(self):
         """Check for explicitly somewhere.
@@ -118,21 +112,19 @@ class TaskDef(object):
             raise TaskDefError(
                 "No cycling sequences defined for %s" % self.name)
 
-    @classmethod
-    def get_cleanup_cutoff_point(cls, my_point, offset_sequence_tuples):
+    def get_cleanup_cutoff_point(self, point):
         """Extract the max dependent cycle point for this point."""
-        if not offset_sequence_tuples:
+        if not self.intercycle_offsets:
             # This task does not have dependent tasks at other cycles.
-            return my_point
+            return point
         cutoff_points = []
-        for offset_string, sequence in offset_sequence_tuples:
+        for offset_string, sequence in self.intercycle_offsets:
             if offset_string is None:
                 # This indicates a dependency that lasts for the whole run.
                 return None
             if sequence is None:
                 # This indicates a simple offset interval such as [-PT6H].
-                cutoff_points.append(
-                    my_point - get_interval(offset_string))
+                cutoff_points.append(point - get_interval(offset_string))
                 continue
             if is_offset_absolute(offset_string):
                 stop_point = sequence.get_stop_point()
@@ -153,11 +145,11 @@ class TaskDef(object):
                 # TODO: Is it realistically possible to hang in this loop?
                 target_point = (
                     get_point_relative(offset_string, dependent_point))
-                if target_point > my_point:
+                if target_point > point:
                     # Assume monotonic (target_point can never jump back).
                     break
-                if target_point == my_point:
-                    # We have found a dependent_point for my_point.
+                if target_point == point:
+                    # We have found a dependent_point for point.
                     my_cutoff_point = dependent_point
                 dependent_point = sequence.get_next_point_on_sequence(
                     dependent_point)
@@ -166,9 +158,9 @@ class TaskDef(object):
                 cutoff_points.append(my_cutoff_point)
         if cutoff_points:
             max_cutoff_point = max(cutoff_points)
-            if max_cutoff_point < my_point:
-                # This is caused by future triggers - default to my_point.
-                return my_point
+            if max_cutoff_point < point:
+                # This is caused by future triggers - default to point.
+                return point
             return max_cutoff_point
-        # There aren't any dependent tasks in other cycles for my_point.
-        return my_point
+        # There aren't any dependent tasks in other cycles for point.
+        return point
