@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # ----------------------------------------------------------------------------
-# (C) British Crown Copyright 2013-2017 Met Office.
+# (C) British Crown Copyright 2013-2018 Met Office.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -22,7 +22,6 @@ import re
 import sre_constants
 
 from . import data
-from . import dumpers
 from . import parser_spec
 from . import timezone
 
@@ -196,7 +195,8 @@ class TimePointParser(object):
                 self._time_zone_regex_map[format_type].append(
                     [re.compile(time_zone_regex), time_zone_expr])
 
-    def get_expressions(self, text):
+    @staticmethod
+    def get_expressions(text):
         """Yield valid expressions from text."""
         for line in text.splitlines():
             line_text = line.strip()
@@ -207,24 +207,26 @@ class TimePointParser(object):
 
     def parse_date_expression_to_regex(self, expression):
         """Construct regular expressions for the date."""
-        for expr_regex, substitute, format_, name in (
+        for expr_regex, substitute, _, _ in (
                 parser_spec.get_date_translate_info(
                     self.expanded_year_digits)):
             expression = re.sub(expr_regex, substitute, expression)
         expression = "^" + expression + "$"
         return expression
 
-    def parse_time_expression_to_regex(self, expression):
+    @staticmethod
+    def parse_time_expression_to_regex(expression):
         """Construct regular expressions for the time."""
-        for expr_regex, substitute, format_, name in (
+        for expr_regex, substitute, _, _ in (
                 parser_spec.get_time_translate_info()):
             expression = re.sub(expr_regex, substitute, expression)
         expression = "^" + expression + "$"
         return expression
 
-    def parse_time_zone_expression_to_regex(self, expression):
+    @staticmethod
+    def parse_time_zone_expression_to_regex(expression):
         """Construct regular expressions for the time zone."""
-        for expr_regex, substitute, format_, name in (
+        for expr_regex, substitute, _, _ in (
                 parser_spec.get_time_zone_translate_info()):
             expression = re.sub(expr_regex, substitute, expression)
         expression = "^" + expression + "$"
@@ -323,15 +325,13 @@ class TimePointParser(object):
         regex = "^"
         for item in split_format:
             if parser_spec.REC_STRFTIME_DIRECTIVE_TOKEN.search(item):
-                item_regex, item_properties = (
-                    parser_spec.translate_strptime_token(item))
-                regex += item_regex
+                regex += parser_spec.translate_strptime_token(item)[0]
             else:
                 regex += re.escape(item)
         regex += "$"
         return self._parse_from_custom_regex(
             regex, strptime_data_string,
-            dump_format=None, source=strptime_format_string)
+            dump_format=dump_format, source=strptime_format_string)
 
     def _parse_from_custom_regex(self, regex, data_string, dump_format=None,
                                  source=None):
@@ -350,14 +350,12 @@ class TimePointParser(object):
                 translator = data.PARSE_PROPERTY_TRANSLATORS[property_]
                 info.update(translator(value))
         date_info_keys = []
-        for expr_regex, substitute, format_, name in (
-                parser_spec.get_date_translate_info(
-                    self.expanded_year_digits)):
-            date_info_keys.append(name)
+        for item in parser_spec.get_date_translate_info(
+                self.expanded_year_digits):
+            date_info_keys.append(item[3])
         time_info_keys = []
-        for expr_regex, substitute, format_, name in (
-                parser_spec.get_time_translate_info()):
-            time_info_keys.append(name)
+        for item in parser_spec.get_time_translate_info():
+            time_info_keys.append(item[3])
         date_info = {}
         time_info = {}
         time_zone_info = {}
