@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 """Ordered Dictionary data structure used extensively in cylc."""
 
-
 try:
     # Python 2.7+ native.
     from collections import OrderedDict
@@ -74,18 +73,21 @@ class OrderedDictWithDefaults(OrderedDict):
         return [(key, self[key]) for key in self.keys()]
 
     def iterkeys(self):
-        """Include default keys - no memory saving over .keys()."""
-        for k in self.keys():
-            yield k
+        """Include default keys"""
+        for key in OrderedDict.iterkeys(self):
+            yield key
+        for key in getattr(self, 'defaults_', []):
+            if not OrderedDict.__contains__(self, key):
+                yield key
 
     def itervalues(self):
-        """Include default values - no memory saving over .values()."""
-        for k in self.keys():
+        """Include default values."""
+        for k in self.iterkeys():
             yield self[k]
 
     def iteritems(self):
-        """Include default key-value pairs - no memory saving over .items()"""
-        for k in self.keys():
+        """Include default key-value pairs."""
+        for k in self.iterkeys():
             yield (k, self[k])
 
     def __contains__(self, key):
@@ -97,3 +99,29 @@ class OrderedDictWithDefaults(OrderedDict):
     def __nonzero__(self):
         """Include any default keys in the nonzero calculation."""
         return bool(self.keys())
+
+    def prepend(self, key, value, dict_setitem=dict.__setitem__):
+        """Prepend new item in the ordered dict.
+
+        https://stackoverflow.com/questions/16664874/
+           how-can-i-add-an-element-at-the-top-of-an-ordereddict-in-python
+
+        Tested with Python 2.7 collections.OrderedDict and our bundled
+        ActiveState implementation for Python 2.6.
+
+        """
+        root = self._OrderedDict__root
+        first = root[1]
+
+        if key in self:
+            link = self._OrderedDict__map[key]
+            link_prev, link_next, _ = link
+            link_prev[1] = link_next
+            link_next[0] = link_prev
+            link[0] = root
+            link[1] = first
+            root[1] = first[0] = link
+        else:
+            root[1] = first[0] = self._OrderedDict__map[key] = [
+                root, first, key]
+        dict_setitem(self, key, value)
