@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2017 NIWA
+# Copyright (C) 2008-2018 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -90,7 +90,8 @@ class upgrader(object):
             item = item[key]
         del item[keys[-1]]
 
-    def show_keys(self, keys):
+    @staticmethod
+    def show_keys(keys):
         return '[' + ']['.join(keys) + ']'
 
     def expand(self, upg):
@@ -98,7 +99,7 @@ class upgrader(object):
         if '__MANY__' not in upg['old']:
             return [upg]
         if upg['old'].count('__MANY__') > 1:
-            print >> sys.stderr, upg['old']
+            sys.stderr.write('%s\n' % upg['old'])
             raise UpgradeError("Multiple simultaneous __MANY__ not supported")
         exp_upgs = []
         pre = []
@@ -157,13 +158,13 @@ class upgrader(object):
             for u in upgs:
                 try:
                     exp = self.expand(u)
-                except:
+                except (KeyError, UpgradeError):
                     continue
 
                 for upg in exp:
                     try:
                         old = self.get_item(upg['old'])
-                    except:
+                    except KeyError:
                         # OK: deprecated item not found
                         pass
                     else:
@@ -180,12 +181,13 @@ class upgrader(object):
                         if upg['cvt'].describe() != "DELETED (OBSOLETE)":
                             self.put_item(upg['new'], upg['cvt'].convert(old))
         if do_warn and cylc.flags.verbose:
-            print >> sys.stderr, (
-                "WARNING: deprecated items were automatically upgraded in '" +
-                self.descr + "':")
+            sys.stderr.write(
+                'WARNING: deprecated items were automatically upgraded in' +
+                " '%s':\n" % self.descr)
             for vn, msgs in warnings.items():
                 for m in msgs:
-                    print >> sys.stderr, " * (" + vn + ")", m
+                    sys.stderr.write(' * (%s) %s\n' % (vn, m))
+
 
 if __name__ == "__main__":
     from util import printcfg
@@ -212,7 +214,7 @@ if __name__ == "__main__":
     x2 = converter(lambda x: 2 * x, 'value x 2')
 
     printcfg(cfg)
-    print
+    sys.stdout.write('\n')
 
     upg = upgrader(cfg, 'test file')
     # successive upgrades are incremental - at least until I think of a
@@ -231,5 +233,5 @@ if __name__ == "__main__":
 
     upg.upgrade()
 
-    print
+    sys.stdout.write('\n')
     printcfg(cfg)
